"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Assertions = void 0;
const tsfun_1 = require("tsfun");
const configuration_errors_1 = require("./configuration-errors");
const helpers_1 = require("./helpers");
const named_1 = require("../../tools/named");
const library_form_definition_1 = require("../model/form/library-form-definition");
const custom_form_definition_1 = require("../model/form/custom-form-definition");
const valuelist_1 = require("../../model/configuration/valuelist");
const library_category_definition_1 = require("../model/category/library-category-definition");
const field_1 = require("../../model/configuration/field");
/**
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
var Assertions;
(function (Assertions) {
    function performAssertions(builtInCategories, libraryCategories, libraryForms, commonFields, valuelistsConfiguration, customForms) {
        assertStructurallyValid(builtInCategories, libraryCategories, libraryForms, valuelistsConfiguration, customForms);
        assertSubtypingIsLegal(builtInCategories, libraryCategories);
        if (customForms)
            assertSubtypingIsLegal(builtInCategories, customForms);
        assertNoCommonFieldWithValuelistFromProjectFieldGetsNewValuelist(commonFields, libraryForms, customForms !== null && customForms !== void 0 ? customForms : {});
        assertInputTypesAreSet(builtInCategories, commonFields);
        assertInputTypesAreSet(libraryCategories, commonFields);
    }
    Assertions.performAssertions = performAssertions;
    function assertNoDuplicationInSelection(customForms) {
        return (forms) => {
            Object.keys(customForms).reduce((selectedFormNames, formName) => {
                const form = forms[formName];
                if (!form)
                    return selectedFormNames;
                if (!selectedFormNames.includes(form.categoryName)) {
                    return selectedFormNames.concat([form.categoryName]);
                }
                else {
                    throw [[configuration_errors_1.ConfigurationErrors.DUPLICATION_IN_SELECTION, form.categoryName]];
                }
            }, []);
            return forms;
        };
    }
    Assertions.assertNoDuplicationInSelection = assertNoDuplicationInSelection;
    function assertInputTypesAreSet(categories, commonFields) {
        helpers_1.iterateOverFields(categories, (categoryName, _, fieldName, field) => {
            assertInputTypePresentIfNotCommonField(categoryName, fieldName, field, commonFields);
        });
    }
    function assertInputTypePresentIfNotCommonField(categoryName, fieldName, field, commonFields) {
        if (!field.inputType && !Object.keys(commonFields).includes(fieldName)) {
            throw [[configuration_errors_1.ConfigurationErrors.MISSING_FIELD_PROPERTY, 'inputType', categoryName, fieldName]];
        }
    }
    function assertValuelistIdsProvided(definitions) {
        helpers_1.iterateOverFields(definitions, (formName, _, fieldName, field) => {
            if (field.inputType && field_1.Field.InputType.VALUELIST_INPUT_TYPES.includes(field.inputType)) {
                if (!field.valuelistId && !field.valuelist && !field.valuelistFromProjectField) {
                    throw [[configuration_errors_1.ConfigurationErrors.NO_VALUELIST_PROVIDED, formName, fieldName]];
                }
            }
        });
        return definitions;
    }
    Assertions.assertValuelistIdsProvided = assertValuelistIdsProvided;
    function assertSubtypingIsLegal(builtInCategories, categories) {
        tsfun_1.flow(categories, tsfun_1.filter((_, categoryName) => !builtInCategories[categoryName]), helpers_1.getDefinedParents, tsfun_1.forEach((parent) => {
            const found = Object.keys(builtInCategories).find(tsfun_1.is(parent));
            if (!found)
                throw [configuration_errors_1.ConfigurationErrors.INVALID_CONFIG_PARENT_NOT_DEFINED, parent];
            const foundBuiltIn = builtInCategories[found];
            if (!foundBuiltIn.supercategory || !foundBuiltIn.userDefinedSubcategoriesAllowed) {
                throw [configuration_errors_1.ConfigurationErrors.TRYING_TO_SUBTYPE_A_NON_EXTENDABLE_CATEGORY, parent];
            }
        }));
    }
    function assertStructurallyValid(builtInCategories, libraryCategories, libraryForms, valuelistsConfiguration, customForms) {
        assertLibraryCategoriesStrucuturallyValid(libraryCategories, builtInCategories);
        assertLibraryFormsStrucuturallyValid(libraryForms, builtInCategories, libraryCategories);
        if (customForms) {
            assertCustomFormsStrucuturallyValid(customForms, builtInCategories, libraryCategories, libraryForms);
        }
        assertValuelistsStructurallyValid(valuelistsConfiguration);
    }
    function assertLibraryCategoriesStrucuturallyValid(libraryCategories, builtInCategories) {
        tsfun_1.keysValues(libraryCategories).forEach(library_category_definition_1.LibraryCategoryDefinition.makeAssertIsValid(Object.keys(builtInCategories)));
    }
    function assertLibraryFormsStrucuturallyValid(libraryForms, builtInCategories, libraryCategories) {
        const categoryNames = Object.keys(builtInCategories).concat(Object.keys(libraryCategories));
        const assert = library_form_definition_1.LibraryFormDefinition.makeAssertIsValid(categoryNames);
        tsfun_1.keysValues(libraryForms).forEach(assert);
    }
    function assertCustomFormsStrucuturallyValid(customForms, builtInCategories, libraryCategories, libraryForms) {
        const formNames = tsfun_1.set(Object.keys(builtInCategories)
            .concat(Object.keys(libraryCategories))
            .concat(Object.keys(libraryForms)));
        const assert = custom_form_definition_1.CustomFormDefinition.makeAssertIsValid(formNames);
        tsfun_1.keysValues(customForms).forEach(assert);
    }
    function assertValuelistsStructurallyValid(valuelistDefinitions) {
        tsfun_1.forEach(valuelistDefinitions, (vd, vdId) => {
            const result = valuelist_1.Valuelist.assertIsValid(vd);
            if (result !== undefined && result.length > 1 && result[0] === 'missing') {
                throw [configuration_errors_1.ConfigurationErrors.MISSING_CATEGORY_PROPERTY, result[1], vdId];
            }
        });
    }
    function assertNoCommonFieldWithValuelistFromProjectFieldGetsNewValuelist(commonFields, libraryForms, customForms) {
        const forms = named_1.Named.mapToNamedArray(libraryForms).concat(named_1.Named.mapToNamedArray(customForms));
        for (let form of forms) {
            if (!form['valuelists'])
                return;
            for (let fieldName of Object.keys(form['valuelists'])) {
                if (commonFields[fieldName] && commonFields[fieldName].valuelistFromProjectField) {
                    throw [
                        configuration_errors_1.ConfigurationErrors.COMMON_FIELD_VALUELIST_FROM_PROJECTDOC_NOT_TO_BE_OVERWRITTEN,
                        form.name,
                        fieldName
                    ];
                }
            }
        }
    }
})(Assertions = exports.Assertions || (exports.Assertions = {}));
//# sourceMappingURL=assertions.js.map