"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigLoader = void 0;
const tsfun_1 = require("tsfun");
const project_configuration_1 = require("../../services/project-configuration");
const build_raw_project_configuration_1 = require("./build-raw-project-configuration");
const configuration_validation_1 = require("./configuration-validation");
const DEFAULT_LANGUAGES = ['de', 'en', 'es', 'fr', 'it', 'tr', 'uk'];
/**
 * Lets clients subscribe for the app
 * configuration. In order for this to work, they
 * have to call <code>validateFieldDefinitions_</code> and <code>getProjectConfiguration</code>
 *  (the call order does not matter).
 *
 * It is recommended to handle a promise rejection of
 * <code>getProjectConfiguration</code> at a single place in your app.
 *
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 * @author Fabian Z.
 */
class ConfigLoader {
    constructor(configReader) {
        this.configReader = configReader;
    }
    go(commonFields, builtInCategories, relations, builtInFields, configurationDocument, includeAllRelations = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const libraryCategories = this.readLibraryFile('Categories.json');
            const libraryForms = this.readLibraryFile('Forms.json');
            const libraryValuelists = this.readValuelists();
            const missingRelationCategoryErrors = configuration_validation_1.ConfigurationValidation.findMissingRelationType(relations, Object.keys(builtInCategories));
            if (missingRelationCategoryErrors.length > 0)
                throw missingRelationCategoryErrors;
            return this.loadConfiguration(builtInCategories, libraryCategories, libraryForms, libraryValuelists, commonFields, relations, builtInFields, configurationDocument, includeAllRelations);
        });
    }
    readLibraryFile(fileName) {
        try {
            return this.configReader.read('/Library/' + fileName);
        }
        catch (msgWithParams) {
            throw [msgWithParams];
        }
    }
    loadConfiguration(builtInCategories, libraryCategories, libraryForms, libraryValuelists, commonFields, relations, builtInFields, configurationDocument, includeAllRelations) {
        return __awaiter(this, void 0, void 0, function* () {
            let customForms;
            let languageConfigurations;
            let projectLanguages;
            let categoriesOrder;
            let customValuelists;
            try {
                customForms = configurationDocument.resource.forms;
                const defaultLanguageConfigurations = this.readDefaultLanguageConfigurations();
                languageConfigurations = {
                    complete: this.mergeLanguageConfigurations(defaultLanguageConfigurations, configurationDocument.resource.languages),
                    custom: this.mergeLanguageConfigurations({}, configurationDocument.resource.languages),
                    default: defaultLanguageConfigurations
                };
                categoriesOrder = configurationDocument.resource.order;
                customValuelists = configurationDocument.resource.valuelists;
                projectLanguages = configurationDocument.resource.projectLanguages;
            }
            catch (msgWithParams) {
                throw [msgWithParams];
            }
            try {
                return new project_configuration_1.ProjectConfiguration(build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, libraryCategories, libraryForms, customForms, commonFields, libraryValuelists, customValuelists, builtInFields, relations, languageConfigurations, projectLanguages, categoriesOrder, (categories) => {
                    const fieldValidationErrors = configuration_validation_1.ConfigurationValidation.validateFieldDefinitions(Object.values(categories));
                    if (fieldValidationErrors.length > 0)
                        throw fieldValidationErrors;
                    return categories;
                }, undefined, includeAllRelations));
            }
            catch (msgWithParams) {
                throw msgWithParams;
            }
        });
    }
    readDefaultLanguageConfigurations() {
        return DEFAULT_LANGUAGES.reduce((configurations, language) => {
            configurations[language] = [];
            configurations[language].push(this.readLanguageConfiguration('/Library/Language.' + language + '.json'));
            configurations[language].push(this.readLanguageConfiguration('/Core/Language.' + language + '.json'));
            return configurations;
        }, {});
    }
    mergeLanguageConfigurations(defaultLanguageConfigurations, customLanguageConfiguration) {
        return Object.keys(customLanguageConfiguration).reduce((result, language) => {
            if (!result[language])
                result[language] = [];
            result[language].unshift(customLanguageConfiguration[language]);
            return result;
        }, tsfun_1.clone(defaultLanguageConfigurations));
    }
    readTemplates() {
        const templates = this.configReader.read('/Library/Templates/Templates.json');
        Object.keys(templates).forEach(templateId => templates[templateId].name = templateId);
        return DEFAULT_LANGUAGES.reduce((result, language) => {
            const path = '/Library/Templates/Language.' + language + '.json';
            if (!this.configReader.exists(path))
                return result;
            const labels = this.configReader.read(path);
            Object.values(result).forEach(template => {
                var _a, _b;
                if (!template.label)
                    template.label = {};
                template.label[language] = (_a = labels[template.name]) === null || _a === void 0 ? void 0 : _a.label;
                if ((_b = labels[template.name]) === null || _b === void 0 ? void 0 : _b.languageConfiguration) {
                    if (!template.configuration.languages)
                        template.configuration.languages = {};
                    template.configuration.languages[language] = labels[template.name].languageConfiguration;
                }
            });
            return result;
        }, templates);
    }
    readValuelists() {
        const valuelists = this.readLibraryFile('Valuelists/Valuelists.json');
        const languages = this.configReader.getValuelistsLanguages();
        this.setValuelistsLanguages(valuelists, languages);
        return valuelists;
    }
    setValuelistsLanguages(valuelists, languages) {
        Object.keys(valuelists).forEach(valuelistId => {
            if (!valuelists[valuelistId].description)
                valuelists[valuelistId].description = {};
        });
        Object.keys(languages).forEach((section) => {
            Object.keys(languages[section]).forEach(language => {
                Object.keys(languages[section][language]).forEach(valuelistId => {
                    this.setValuelistDescription(valuelists, section, language, valuelistId, languages);
                    this.setValueLabels(valuelists, section, language, valuelistId, languages);
                });
            });
        });
    }
    setValuelistDescription(valuelists, section, language, valuelistId, languages) {
        if (!languages[section][language][valuelistId].description)
            return;
        valuelists[valuelistId].description[language]
            = languages[section][language][valuelistId].description;
    }
    setValueLabels(valuelists, section, language, valuelistId, languages) {
        if (!languages[section][language][valuelistId].values)
            return;
        Object.keys(languages[section][language][valuelistId].values)
            .filter(valueId => { var _a; return (_a = languages[section][language][valuelistId].values[valueId]) === null || _a === void 0 ? void 0 : _a.length; })
            .forEach(valueId => {
            if (!valuelists[valuelistId].values[valueId].label) {
                valuelists[valuelistId].values[valueId].label = {};
            }
            valuelists[valuelistId].values[valueId].label[language]
                = languages[section][language][valuelistId].values[valueId];
        });
    }
    readLanguageConfiguration(path) {
        if (!this.configReader.exists(path))
            return undefined;
        return this.configReader.read(path);
    }
}
exports.ConfigLoader = ConfigLoader;
//# sourceMappingURL=config-loader.js.map