"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeCategoryForest = void 0;
const tsfun_1 = require("tsfun");
const category_form_1 = require("../../model/configuration/category-form");
const group_1 = require("../../model/configuration/group");
const relation_1 = require("../../model/configuration/relation");
const forest_1 = require("../../tools/forest");
const category_forest_1 = require("../category-forest");
const hide_fields_1 = require("./hide-fields");
const named_1 = require("../../tools/named");
const TEMP_FIELDS = 'fields';
const TEMP_GROUPS = 'tempGroups';
const TEMP_HIDDEN = 'hidden';
/**
 * @author Thomas Kleinke
 * @author Daniel de Oliveira
 * @author Sebastian Cuy
 */
const makeCategoryForest = (relations, categories, selectedParentForms, includeAllRelations = false) => (forms) => {
    const [parentDefs, childDefs] = tsfun_1.flow(forms, tsfun_1.values, tsfun_1.separate(form => !form.parent && !categories[form.categoryName].parent));
    const parentCategories = tsfun_1.flow(parentDefs, tsfun_1.map(formDefinition => buildCategoryFromDefinition(formDefinition, categories)), forest_1.Forest.wrap);
    return tsfun_1.flow(childDefs, tsfun_1.reduce(addChildCategory(categories, selectedParentForms), parentCategories), forest_1.Forest.map(tsfun_1.compose(createGroups(relations, includeAllRelations), tsfun_1.detach(TEMP_FIELDS), tsfun_1.detach(TEMP_GROUPS), tsfun_1.detach(TEMP_HIDDEN))), category_forest_1.linkParentAndChildInstances);
};
exports.makeCategoryForest = makeCategoryForest;
const createGroups = (relationDefinitions, includeAllRelations) => (category) => {
    const categoryRelations = tsfun_1.clone(relation_1.Relation.getRelations(relationDefinitions, category.name));
    hide_fields_1.applyHiddenForFields(categoryRelations, category[TEMP_HIDDEN]);
    const fields = combineFieldsWithRelations(category, categoryRelations);
    category.groups = category[TEMP_GROUPS].map(groupDefinition => {
        const group = group_1.Group.create(groupDefinition.name);
        group.fields = tsfun_1.set(groupDefinition.fields)
            .map((fieldName) => fields[fieldName])
            .filter(field => field !== undefined);
        return group;
    });
    if (includeAllRelations && category.groups.length > 0) {
        addAllRelationsToGroups(category, categoryRelations);
    }
    putUnassignedFieldsToGroups(category, fields);
    return category;
};
function combineFieldsWithRelations(category, categoryRelations) {
    let fields = {};
    Object.assign(fields, category[TEMP_FIELDS]);
    fields = categoryRelations.reduce((result, relation) => {
        result[relation.name] = relation;
        return result;
    }, fields);
    return fields;
}
function addAllRelationsToGroups(category, categoryRelations) {
    categoryRelations.filter(relation => {
        return !tsfun_1.flatten(category.groups.map(tsfun_1.to(group_1.Group.FIELDS)))
            .map(tsfun_1.to(named_1.Named.NAME))
            .includes(relation.name);
    }).forEach(relation => {
        category.groups[0].fields.push(relation);
    });
}
function putUnassignedFieldsToGroups(category, fields) {
    const fieldsInGroups = tsfun_1.flatten(1, category[TEMP_GROUPS].map(group => group.fields));
    Object.values(fields)
        .filter(field => !fieldsInGroups.includes(field.name)
        && (fields[field.name].visible || fields[field.name].editable))
        .forEach(field => putUnassignedFieldToGroup(category, field));
}
function putUnassignedFieldToGroup(category, field) {
    const groupName = getGroupNameForUnassaginedField(field.name);
    let group = category.groups.find(group => group.name === groupName);
    if (!group) {
        group = group_1.Group.create(groupName);
        category.groups.push(group);
    }
    group.fields.push(field);
}
function getGroupNameForUnassaginedField(fieldName) {
    if (relation_1.Relation.Position.ALL.includes(fieldName)) {
        return group_1.Groups.POSITION;
    }
    else if (relation_1.Relation.Time.ALL.includes(fieldName)) {
        return group_1.Groups.TIME;
    }
    else if (relation_1.Relation.Type.ALL.includes(fieldName)) {
        return group_1.Groups.IDENTIFICATION;
    }
    else if (['hasChildren', 'includesStratigraphicalUnits'].includes(fieldName)) {
        return group_1.Groups.HIERARCHY;
    }
    else {
        return group_1.Groups.OTHER;
    }
}
const addChildCategory = (categories, selectedParentForms) => (categoryTree, childDefinition) => {
    var _a, _b;
    const parent = (_a = childDefinition.parent) !== null && _a !== void 0 ? _a : (_b = categories[childDefinition.categoryName]) === null || _b === void 0 ? void 0 : _b.parent;
    const found = categoryTree.find(({ item: category }) => {
        return category.name === parent
            && (!selectedParentForms || selectedParentForms.includes(category.libraryId));
    });
    if (!found)
        return categoryTree;
    const { item: parentCategory, trees: trees } = found;
    const childCategory = buildCategoryFromDefinition(childDefinition, categories, parentCategory);
    trees.push({ item: childCategory, trees: [] });
    return categoryTree;
};
function buildCategoryFromDefinition(formDefinition, categories, parentCategoryForm) {
    var _a, _b, _c;
    const categoryDefinition = categories[formDefinition.categoryName];
    const parentCategoryDefinition = categories[(_a = formDefinition.parent) !== null && _a !== void 0 ? _a : categoryDefinition.parent];
    const category = {};
    category.name = categoryDefinition ? categoryDefinition.name : formDefinition.categoryName;
    category.mustLieWithin = parentCategoryDefinition
        ? parentCategoryDefinition.mustLieWithin : categoryDefinition.mustLieWithin;
    category.isAbstract = (categoryDefinition === null || categoryDefinition === void 0 ? void 0 : categoryDefinition.abstract) || false;
    category.userDefinedSubcategoriesAllowed = (categoryDefinition === null || categoryDefinition === void 0 ? void 0 : categoryDefinition.userDefinedSubcategoriesAllowed) || false;
    category.scanCodesAllowed = (categoryDefinition === null || categoryDefinition === void 0 ? void 0 : categoryDefinition.scanCodesAllowed) || false;
    category.required = (categoryDefinition === null || categoryDefinition === void 0 ? void 0 : categoryDefinition.required) || false;
    category.libraryId = formDefinition.name;
    category.label = formDefinition.label;
    category.categoryLabel = categoryDefinition ? categoryDefinition.label : tsfun_1.clone(formDefinition.label);
    category.source = formDefinition.source;
    category.customFields = formDefinition.customFields;
    category.description = formDefinition.description;
    category.defaultLabel = formDefinition.defaultLabel;
    category.defaultDescription = formDefinition.defaultDescription;
    category.groups = [];
    category.color = (_b = formDefinition.color) !== null && _b !== void 0 ? _b : category_form_1.CategoryForm.generateColorForCategory(category.name);
    category.defaultColor = (_c = formDefinition.defaultColor) !== null && _c !== void 0 ? _c : (category.libraryId
        ? category_form_1.CategoryForm.generateColorForCategory(category.name)
        : category.color);
    category.identifierPrefix = formDefinition.identifierPrefix;
    category.resourceLimit = formDefinition.resourceLimit;
    if (category.scanCodesAllowed || (parentCategoryDefinition === null || parentCategoryDefinition === void 0 ? void 0 : parentCategoryDefinition.scanCodesAllowed)) {
        category.scanCodes = buildScanCodeConfiguration(formDefinition, parentCategoryForm);
    }
    category.createdBy = formDefinition.createdBy;
    category.creationDate = formDefinition.creationDate ? new Date(formDefinition.creationDate) : undefined;
    category.references = formDefinition.references;
    category.children = [];
    category[TEMP_FIELDS] = formDefinition.fields || {};
    Object.keys(category[TEMP_FIELDS]).forEach(fieldName => category[TEMP_FIELDS][fieldName].name = fieldName);
    category[TEMP_GROUPS] = formDefinition.groups || [];
    category[TEMP_HIDDEN] = formDefinition.hidden || [];
    return category;
}
function buildScanCodeConfiguration(formDefinition, parentCategoryForm) {
    var _a, _b, _c, _d, _e;
    if (!(parentCategoryForm === null || parentCategoryForm === void 0 ? void 0 : parentCategoryForm.scanCodes) && !formDefinition.scanCodes)
        return undefined;
    return {
        type: (_b = (_a = parentCategoryForm === null || parentCategoryForm === void 0 ? void 0 : parentCategoryForm.scanCodes) === null || _a === void 0 ? void 0 : _a.type) !== null && _b !== void 0 ? _b : (_c = formDefinition.scanCodes) === null || _c === void 0 ? void 0 : _c.type,
        autoCreate: ((_d = parentCategoryForm === null || parentCategoryForm === void 0 ? void 0 : parentCategoryForm.scanCodes) === null || _d === void 0 ? void 0 : _d.autoCreate) === true
            || ((_e = formDefinition.scanCodes) === null || _e === void 0 ? void 0 : _e.autoCreate) === true,
        printedFields: buildPrintedFields(formDefinition, parentCategoryForm)
    };
}
function buildPrintedFields(formDefinition, parentCategoryForm) {
    var _a, _b, _c, _d;
    const parentFields = (_b = (_a = parentCategoryForm === null || parentCategoryForm === void 0 ? void 0 : parentCategoryForm.scanCodes) === null || _a === void 0 ? void 0 : _a.printedFields) !== null && _b !== void 0 ? _b : [];
    const formFields = (_d = (_c = formDefinition === null || formDefinition === void 0 ? void 0 : formDefinition.scanCodes) === null || _c === void 0 ? void 0 : _c.printedFields) !== null && _d !== void 0 ? _d : [];
    const result = parentFields.concat(formFields);
    return result.length > 3 ? result.slice(0, 3) : result;
}
//# sourceMappingURL=make-category-forest.js.map