"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeWithCustomForms = void 0;
const tsfun_1 = require("tsfun");
const transient_form_definition_1 = require("../model/form/transient-form-definition");
const configuration_errors_1 = require("./configuration-errors");
const add_fields_to_form_1 = require("./add-fields-to-form");
/**
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
function mergeWithCustomForms(customForms, categories, builtInFields, commonFields, relations, selectedForms) {
    return (forms) => {
        return getSortedCustomForms(customForms, forms, categories).reduce((mergedForms, [customFormName, customForm]) => {
            const parentForm = getParentForm(customForm, customFormName, mergedForms, categories, selectedForms);
            return tsfun_1.assoc(customFormName, mergedForms[customFormName]
                ? handleDirectExtension(customForm, mergedForms[customFormName], categories, builtInFields, commonFields, relations, parentForm)
                : handleChildExtension(customFormName, customForm, parentForm, categories, builtInFields, commonFields, relations))(mergedForms);
        }, tsfun_1.clone(forms));
    };
}
exports.mergeWithCustomForms = mergeWithCustomForms;
function handleDirectExtension(customForm, extendedForm, categories, builtInFields, commonFields, relations, parentForm) {
    const clonedCustomForm = inheritValuelists(customForm, parentForm, extendedForm);
    clonedCustomForm.categoryName = extendedForm.categoryName;
    const result = add_fields_to_form_1.addFieldsToForm(clonedCustomForm, categories, builtInFields, commonFields, relations, parentForm, extendedForm);
    if (parentForm)
        result.hidden = getMergedHiddenArray(result, parentForm);
    return mergeFormProperties(extendedForm, result);
}
function handleChildExtension(customFormName, customForm, parentForm, categories, builtInFields, commonFields, relations) {
    if (!customForm.parent)
        throw [configuration_errors_1.ConfigurationErrors.MUST_HAVE_PARENT, customFormName];
    const clonedCustomForm = inheritValuelists(customForm, parentForm);
    clonedCustomForm.name = customFormName;
    clonedCustomForm.categoryName = customFormName;
    clonedCustomForm.hidden = getMergedHiddenArray(customForm, parentForm);
    clonedCustomForm.customFields = clonedCustomForm.groups ? tsfun_1.flatten(clonedCustomForm.groups.map(tsfun_1.to('fields'))) : [];
    return add_fields_to_form_1.addFieldsToForm(clonedCustomForm, categories, builtInFields, commonFields, relations, parentForm);
}
function mergeFormProperties(target, source) {
    if (source.valuelists) {
        if (!target.valuelists)
            target.valuelists = {};
        tsfun_1.keysValues(source.valuelists).forEach(([valuelistId, valuelist]) => {
            target.valuelists[valuelistId] = valuelist;
        });
    }
    target.fields = source.fields;
    target.defaultColor = target.color;
    if (source.color)
        target.color = source.color;
    target.hidden = source.hidden;
    if (source.groups) {
        const sourceFields = tsfun_1.flatten(source.groups.map(tsfun_1.to('fields')));
        const targetFields = target.groups ? tsfun_1.flatten(target.groups.map(tsfun_1.to('fields'))) : [];
        target.customFields = sourceFields.filter(fieldName => !targetFields.includes(fieldName));
        target.groups = source.groups;
    }
    else {
        target.customFields = [];
    }
    tsfun_1.flow(source, Object.keys, tsfun_1.filter(tsfun_1.isnt(transient_form_definition_1.TransientFormDefinition.FIELDS)), tsfun_1.filter(tsfun_1.isNot(tsfun_1.includedIn(Object.keys(target)))), tsfun_1.map(tsfun_1.pairWith(tsfun_1.lookup(source))), tsfun_1.forEach(overwriteIn(target)));
    return target;
}
function getSortedCustomForms(customForms, forms, categories) {
    return tsfun_1.keysValues(customForms).sort((element1, element2) => {
        const parentCategoryName1 = getParentCategoryName(element1[1], element1[0], forms, categories);
        const parentCategoryName2 = getParentCategoryName(element2[1], element2[0], forms, categories);
        if (parentCategoryName1 && !parentCategoryName2)
            return 1;
        if (!parentCategoryName1 && parentCategoryName2)
            return -1;
        return 0;
    });
}
function getParentForm(customForm, customFormName, forms, categories, selectedForms) {
    const parentCategoryName = getParentCategoryName(customForm, customFormName, forms, categories);
    return parentCategoryName
        && Object.values(forms).filter(form => selectedForms.includes(form.name))
            .find(form => form.categoryName === parentCategoryName);
}
function getParentCategoryName(customForm, customFormName, forms, categories) {
    let parentCategory = customForm.parent;
    if (!parentCategory) {
        const category = forms[customFormName]
            ? categories[forms[customFormName].categoryName]
            : categories[customFormName];
        parentCategory = category.parent;
    }
    return parentCategory;
}
function inheritValuelists(childForm, parentForm, extendedForm) {
    var _a, _b;
    const clonedChildForm = tsfun_1.clone(childForm);
    if (!clonedChildForm.valuelists)
        clonedChildForm.valuelists = {};
    const extendedFormValuelists = (_a = extendedForm === null || extendedForm === void 0 ? void 0 : extendedForm.valuelists) !== null && _a !== void 0 ? _a : {};
    const parentFormValuelists = (_b = parentForm === null || parentForm === void 0 ? void 0 : parentForm.valuelists) !== null && _b !== void 0 ? _b : {};
    Object.keys(extendedFormValuelists).forEach(fieldName => {
        if (!clonedChildForm.valuelists[fieldName]) {
            clonedChildForm.valuelists[fieldName] = extendedFormValuelists[fieldName];
        }
    });
    Object.keys(parentFormValuelists).forEach(fieldName => {
        clonedChildForm.valuelists[fieldName] = parentFormValuelists[fieldName];
    });
    return clonedChildForm;
}
function getMergedHiddenArray(childForm, parentForm) {
    var _a, _b;
    return tsfun_1.set(((_a = parentForm.hidden) !== null && _a !== void 0 ? _a : []).concat((_b = childForm.hidden) !== null && _b !== void 0 ? _b : []));
}
function overwriteIn(target) {
    return ([key, value]) => target[key] = value;
}
//# sourceMappingURL=merge-with-custom-forms.js.map