import { IndexFacade } from '../index/index-facade';
import { Document } from '../model/document';
import { NewDocument } from '../model/document';
import { Query } from '../model/query';
import { Name } from '../tools/named';
import { DocumentConverter } from './document-converter';
import { DocumentCache } from './document-cache';
import { PouchdbDatastore } from './pouchdb/pouchdb-datastore';
import { ProjectConfiguration } from '../services/project-configuration';
export declare type FindOptions = {
    includeResourcesWithoutValidParent?: boolean;
};
/**
 * This datastore provides everything necessary
 * to power a idai-field application:
 *
 * 1) A PouchDB based datastore layer, with gives us synchronization.
 *
 * 1) A document cache for faster access and also to allow
 *    for clients to work with references to documents.
 *
 * 2) Returns fully checked instances of
 *    Document,
 *    so that the rest of the app can rely that the declared
 *    fields are present.
 *
 * The errors with which the methods reject, like GENERIC_SAVE_ERROR,
 * are constants of {@link DatastoreErrors}, so GENERIC_SAVE_ERROR really
 * is DatastoreErrors.GENERIC_SAVE_ERROR. The brackets [] are array indicators,
 * so [GENERIC_SAVE_ERROR] is an array containing one element, which is the string
 * corresponding to GENERIC_SAVE_ERROR.
 *
 * @author Thomas Kleinke
 * @author Daniel de Oliveira
 * @author Sebastian Cuy
 */
export declare class Datastore {
    private datastore;
    private indexFacade;
    private documentCache;
    private documentConverter;
    private projectConfiguration;
    private getUser;
    updating: boolean;
    constructor(datastore: PouchdbDatastore, indexFacade: IndexFacade, documentCache: DocumentCache, documentConverter: DocumentConverter, projectConfiguration: ProjectConfiguration, getUser: () => Name);
    /**
     * Persists a given document. If document.resource.id is not set,
     * it will be set to a generated value. In case of an error it remains undefined.
     *
     * In case of a successful call, document.modified and document.created get set,
     * otherwise they remain undefined.
     *
     * @param doc
     * @returns {Promise<Document>} a document
     * @throws [GENERIC_ERROR (, cause: any)] - in case of error, optionally including a cause
     * @throws [DOCUMENT_RESOURCE_ID_EXISTS] - if a document with doc.resource.id already exists
     * @throws [INVALID_DOCUMENT] - in case doc is not valid
     * @throws if resource.category is unknown
     */
    create(document: NewDocument): Promise<Document>;
    bulkCreate(documents: Array<NewDocument>): Promise<Array<Document>>;
    /**
     * Updates an existing document
     *
     * As lambda, to allow passing as Get (see companion namespace below).
     *
     * @param doc
     * @param squashRevisionsIds
     * @returns {Promise<Document>} a document
     * @throws [GENERIC_ERROR (, cause: any)] - in case of error, optionally including a cause
     * @throws [SAVE_CONFLICT] - in case of conflict
     * @throws [DOCUMENT_NO_RESOURCE_ID] - if doc has no resource id
     * @throws [INVALID_DOCUMENT] - in case doc is not valid
     * @throws [DOCUMENT_NOT_FOUND] - if document has a resource id, but does not exist in the db
     */
    update: Datastore.Update;
    bulkUpdate(documents: Array<Document>): Promise<Array<Document>>;
    private updateIndex;
    private updateIndexWithFullReindexing;
    /**
    * Removes an existing document
    *
    * @param doc
    * @returns {Promise<undefined>} undefined
    * @throws [DOCUMENT_NO_RESOURCE_ID] - if document has no resource id
    * @throws [DOCUMENT_NOT_FOUND] - if document has a resource id, but does not exist in the db
    * @throws [GENERIC_ERROR (, cause: any)] - in case of error, optionally including a cause
    */
    remove(document: Document): Promise<void>;
    /**
     * As lambda, to allow passing as Get (see companion namespace below).
     *
     * @param resourceId the desired document's resource id
     * @param options.skipCache: boolean
     * @param options to control implementation specific behaviour
     * @returns {Promise<Document>} a document (rejects with msgWithParams in case of error)
     * @throws [DOCUMENT_NOT_FOUND] - in case document is missing
     * @throws [INVALID_DOCUMENT] - in case document is not valid
     */
    get: Datastore.Get;
    /**
      * @param resourceIds the resource ids of the documents to find
      * @returns {Promise<Array<Document>>} list of found documents
      */
    getMultiple(ids: string[]): Promise<Array<Document>>;
    /**
     * Perform a fulltext query
     * Find sorts the documents by identifier ascending
     *
     * As lambda, to allow passing as Find (see companion namespace below).
     *
     * @param query the query object
     * @returns {Promise<IdaiFieldFindResult>} result object
     * @throws [GENERIC_ERROR (, cause: any)] - in case of error, optionally including a cause
     */
    find: Datastore.Find;
    convert: Datastore.Convert;
    /**
     * As lambda, to allow passing as FindIds (see companion namespace below).
     *
     * @param query
     * @returns
     */
    findIds: Datastore.FindIds;
    /**
     * Fetches a specific revision directly from the underlying datastore layer.
     * Bypasses the cache and alway returns a new instance.
     *
     * @throws [DOCUMENT_NOT_FOUND] - in case of error
     */
    getRevision(docId: string, revisionId: string): Promise<Document>;
    /**
     * @param query
     * @return an array of the resource ids of the documents the query matches.
     *   The sort order of the ids is determined in that way that ids of documents with newer modified
     *   dates come first. They are sorted by last modified descending, so to speak.
     *   If two or more documents have the same last modified date, their sort order is unspecified.
     *   The modified date is taken from document.modified[document.modified.length-1].date
     */
    private getIds;
    private getDocumentsForIds;
    private getDocumentsFromCache;
    private getDocumentsFromDatastore;
    private mergeDocuments;
    private updateWarnings;
}
export declare namespace Datastore {
    type Get = (id: string, options?: {
        skipCache?: boolean;
        conflicts?: boolean;
    }) => Promise<Document>;
    type Find = (query: Query, options?: FindOptions) => Promise<FindResult>;
    type FindIds = (query: Query) => FindIdsResult;
    type Update = (document: Document, squashRevisionsIds?: string[]) => Promise<Document>;
    type Convert = (document: Document, notifyObservers?: boolean) => void;
    interface FindIdsResult {
        ids: string[];
        totalCount: number;
        queryId?: string;
    }
    interface FindResult extends FindIdsResult {
        documents: Array<Document>;
    }
    namespace FindResult {
        const DOCUMENTS = "documents";
        const TOTALCOUNT = "totalCount";
    }
}
