"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Datastore = void 0;
const tsfun_1 = require("tsfun");
const datastore_errors_1 = require("./datastore-errors");
const warnings_updater_1 = require("./warnings-updater");
const index_1 = require("../index");
/**
 * This datastore provides everything necessary
 * to power a idai-field application:
 *
 * 1) A PouchDB based datastore layer, with gives us synchronization.
 *
 * 1) A document cache for faster access and also to allow
 *    for clients to work with references to documents.
 *
 * 2) Returns fully checked instances of
 *    Document,
 *    so that the rest of the app can rely that the declared
 *    fields are present.
 *
 * The errors with which the methods reject, like GENERIC_SAVE_ERROR,
 * are constants of {@link DatastoreErrors}, so GENERIC_SAVE_ERROR really
 * is DatastoreErrors.GENERIC_SAVE_ERROR. The brackets [] are array indicators,
 * so [GENERIC_SAVE_ERROR] is an array containing one element, which is the string
 * corresponding to GENERIC_SAVE_ERROR.
 *
 * @author Thomas Kleinke
 * @author Daniel de Oliveira
 * @author Sebastian Cuy
 */
class Datastore {
    constructor(datastore, indexFacade, documentCache, documentConverter, projectConfiguration, getUser) {
        this.datastore = datastore;
        this.indexFacade = indexFacade;
        this.documentCache = documentCache;
        this.documentConverter = documentConverter;
        this.projectConfiguration = projectConfiguration;
        this.getUser = getUser;
        this.updating = false;
        /**
         * Updates an existing document
         *
         * As lambda, to allow passing as Get (see companion namespace below).
         *
         * @param doc
         * @param squashRevisionsIds
         * @returns {Promise<Document>} a document
         * @throws [GENERIC_ERROR (, cause: any)] - in case of error, optionally including a cause
         * @throws [SAVE_CONFLICT] - in case of conflict
         * @throws [DOCUMENT_NO_RESOURCE_ID] - if doc has no resource id
         * @throws [INVALID_DOCUMENT] - in case doc is not valid
         * @throws [DOCUMENT_NOT_FOUND] - if document has a resource id, but does not exist in the db
         */
        this.update = (document, squashRevisionsIds) => __awaiter(this, void 0, void 0, function* () {
            this.updating = true;
            delete document.warnings;
            try {
                return yield this.updateIndex(yield this.datastore.update(document, this.getUser(), squashRevisionsIds), true);
            }
            finally {
                this.updating = false;
            }
        });
        /**
         * As lambda, to allow passing as Get (see companion namespace below).
         *
         * @param resourceId the desired document's resource id
         * @param options.skipCache: boolean
         * @param options to control implementation specific behaviour
         * @returns {Promise<Document>} a document (rejects with msgWithParams in case of error)
         * @throws [DOCUMENT_NOT_FOUND] - in case document is missing
         * @throws [INVALID_DOCUMENT] - in case document is not valid
         */
        this.get = (id, options) => __awaiter(this, void 0, void 0, function* () {
            const cachedDocument = this.documentCache.get(id);
            if ((!options || !options.skipCache) && cachedDocument) {
                return cachedDocument;
            }
            const document = yield this.datastore.fetch(id, options);
            yield this.convert(document);
            return cachedDocument
                ? this.documentCache.reassign(document)
                : this.documentCache.set(document);
        });
        /**
         * Perform a fulltext query
         * Find sorts the documents by identifier ascending
         *
         * As lambda, to allow passing as Find (see companion namespace below).
         *
         * @param query the query object
         * @returns {Promise<IdaiFieldFindResult>} result object
         * @throws [GENERIC_ERROR (, cause: any)] - in case of error, optionally including a cause
         */
        this.find = (query, options = {}) => __awaiter(this, void 0, void 0, function* () {
            const clonedQuery = tsfun_1.clone(query);
            if (!options.includeResourcesWithoutValidParent) {
                if (!clonedQuery.constraints)
                    clonedQuery.constraints = {};
                clonedQuery.constraints['missingOrInvalidParent:exist'] = 'UNKNOWN';
            }
            const { ids } = this.findIds(clonedQuery);
            const { documents, totalCount } = yield this.getDocumentsForIds(ids, clonedQuery.limit, clonedQuery.offset);
            return {
                documents: documents,
                ids: ids,
                totalCount: totalCount
            };
        });
        this.convert = (document, notifyObservers = true) => __awaiter(this, void 0, void 0, function* () {
            this.documentConverter.convert(document);
            yield this.updateWarnings(document, notifyObservers);
        });
        /**
         * As lambda, to allow passing as FindIds (see companion namespace below).
         *
         * @param query
         * @returns
         */
        this.findIds = (query) => {
            const orderedResults = this.getIds(query);
            return {
                ids: orderedResults,
                totalCount: orderedResults.length
            };
        };
    }
    /**
     * Persists a given document. If document.resource.id is not set,
     * it will be set to a generated value. In case of an error it remains undefined.
     *
     * In case of a successful call, document.modified and document.created get set,
     * otherwise they remain undefined.
     *
     * @param doc
     * @returns {Promise<Document>} a document
     * @throws [GENERIC_ERROR (, cause: any)] - in case of error, optionally including a cause
     * @throws [DOCUMENT_RESOURCE_ID_EXISTS] - if a document with doc.resource.id already exists
     * @throws [INVALID_DOCUMENT] - in case doc is not valid
     * @throws if resource.category is unknown
     */
    create(document) {
        return __awaiter(this, void 0, void 0, function* () {
            this.updating = true;
            try {
                return yield this.updateIndex(yield this.datastore.create(document, this.getUser()));
            }
            finally {
                this.updating = false;
            }
        });
    }
    bulkCreate(documents) {
        return __awaiter(this, void 0, void 0, function* () {
            this.updating = true;
            try {
                const resultDocuments = [];
                for (let document of yield this.datastore.bulkCreate(documents, this.getUser())) {
                    resultDocuments.push(yield this.updateIndex(document, false));
                }
                this.indexFacade.notifyObservers();
                return resultDocuments;
            }
            finally {
                this.updating = false;
            }
        });
    }
    bulkUpdate(documents) {
        return __awaiter(this, void 0, void 0, function* () {
            this.updating = true;
            documents.forEach(document => delete document.warnings);
            try {
                let resultDocuments = [];
                const updatedDocuments = yield this.datastore.bulkUpdate(documents, this.getUser());
                if (updatedDocuments.length < 100) {
                    for (let document of updatedDocuments) {
                        resultDocuments.push(yield this.updateIndex(document, false));
                    }
                    this.indexFacade.notifyObservers();
                }
                else {
                    resultDocuments = yield this.updateIndexWithFullReindexing(updatedDocuments);
                }
                return resultDocuments;
            }
            finally {
                this.updating = false;
            }
        });
    }
    updateIndex(document, notifyObservers = true) {
        return __awaiter(this, void 0, void 0, function* () {
            const previousVersion = this.documentCache.get(document.resource.id);
            yield this.convert(document, notifyObservers);
            return !previousVersion
                ? this.documentCache.set(document)
                : this.documentCache.reassign(document);
        });
    }
    updateIndexWithFullReindexing(documents) {
        return __awaiter(this, void 0, void 0, function* () {
            const resultDocuments = [];
            for (let document of documents) {
                const previousVersion = this.documentCache.get(document.resource.id);
                this.documentConverter.convert(document);
                const updatedDocument = !previousVersion
                    ? this.documentCache.set(document)
                    : this.documentCache.reassign(document);
                resultDocuments.push(updatedDocument);
            }
            yield index_1.Indexer.reindex(this.indexFacade, this.datastore.getDb(), this.documentCache, this.documentConverter, this.projectConfiguration, true);
            return resultDocuments;
        });
    }
    /**
    * Removes an existing document
    *
    * @param doc
    * @returns {Promise<undefined>} undefined
    * @throws [DOCUMENT_NO_RESOURCE_ID] - if document has no resource id
    * @throws [DOCUMENT_NOT_FOUND] - if document has a resource id, but does not exist in the db
    * @throws [GENERIC_ERROR (, cause: any)] - in case of error, optionally including a cause
    */
    remove(document) {
        return __awaiter(this, void 0, void 0, function* () {
            // we want the doc removed from the indices asap,
            // in order to not risk someone finding it still with findIds due to
            // issues that are theoretically possible because we cannot know
            // when .on('change' (pouchdbdatastore) fires. so we do remove it here,
            // although we know it will be done again for the same doc
            // in .on('change'
            this.indexFacade.remove(document);
            yield this.datastore.remove(document);
            this.documentCache.remove(document.resource.id);
            yield warnings_updater_1.WarningsUpdater.updateResourceLimitWarnings(this, this.indexFacade, this.projectConfiguration, this.projectConfiguration.getCategory(document.resource.category));
            this.indexFacade.notifyObservers();
        });
    }
    /**
      * @param resourceIds the resource ids of the documents to find
      * @returns {Promise<Array<Document>>} list of found documents
      */
    getMultiple(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield this.getDocumentsForIds(ids)).documents;
        });
    }
    /**
     * Fetches a specific revision directly from the underlying datastore layer.
     * Bypasses the cache and alway returns a new instance.
     *
     * @throws [DOCUMENT_NOT_FOUND] - in case of error
     */
    getRevision(docId, revisionId) {
        return __awaiter(this, void 0, void 0, function* () {
            const revision = yield this.datastore.fetchRevision(docId, revisionId);
            this.documentConverter.convert(revision);
            return revision;
        });
    }
    /**
     * @param query
     * @return an array of the resource ids of the documents the query matches.
     *   The sort order of the ids is determined in that way that ids of documents with newer modified
     *   dates come first. They are sorted by last modified descending, so to speak.
     *   If two or more documents have the same last modified date, their sort order is unspecified.
     *   The modified date is taken from document.modified[document.modified.length-1].date
     */
    getIds(query) {
        try {
            return this.indexFacade.find(query);
        }
        catch (err) {
            throw [datastore_errors_1.DatastoreErrors.GENERIC_ERROR, err];
        }
    }
    getDocumentsForIds(ids, limit, offset) {
        return __awaiter(this, void 0, void 0, function* () {
            let totalCount = ids.length;
            let idsToFetch = ids;
            if (offset)
                idsToFetch.splice(0, offset);
            if (limit !== undefined) {
                if (limit === 0)
                    return { documents: [], totalCount: totalCount };
                if (limit < idsToFetch.length)
                    idsToFetch = idsToFetch.slice(0, limit);
            }
            const { documentsFromCache, notCachedIds } = yield this.getDocumentsFromCache(idsToFetch);
            let documents = documentsFromCache;
            if (notCachedIds.length > 0) {
                try {
                    const documentsFromDatastore = yield this.getDocumentsFromDatastore(notCachedIds);
                    documents = this.mergeDocuments(documentsFromCache, documentsFromDatastore, idsToFetch);
                    totalCount -= (idsToFetch.length - documents.length);
                }
                catch (e) {
                    console.error('Error while fetching documents from datastore', e);
                    return { documents: [], totalCount: 0 };
                }
            }
            return {
                documents: documents,
                totalCount: totalCount
            };
        });
    }
    getDocumentsFromCache(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const documents = [];
            const notCachedIds = [];
            for (let id of ids) {
                const document = this.documentCache.get(id);
                if (document) {
                    documents.push(document);
                }
                else {
                    notCachedIds.push(id);
                }
            }
            return {
                documentsFromCache: documents,
                notCachedIds: notCachedIds
            };
        });
    }
    getDocumentsFromDatastore(ids) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = [];
            const documents = yield this.datastore.bulkFetch(ids);
            for (let document of documents) {
                document = this.documentCache.set(document);
                yield this.convert(document);
                result.push(document);
            }
            return result;
        });
    }
    mergeDocuments(documentsFromCache, documentsFromDatastore, idsInOrder) {
        const documents = documentsFromCache.concat(documentsFromDatastore);
        documents.sort((a, b) => {
            return idsInOrder.indexOf(a.resource.id) < idsInOrder.indexOf(b.resource.id)
                ? -1
                : 1;
        });
        return documents;
    }
    updateWarnings(document, notifyObservers) {
        return __awaiter(this, void 0, void 0, function* () {
            warnings_updater_1.WarningsUpdater.updateIndexIndependentWarnings(document, this.projectConfiguration);
            this.indexFacade.put(document);
            const previousVersion = this.documentCache.get(document.resource.id);
            const previousIdentifier = previousVersion === null || previousVersion === void 0 ? void 0 : previousVersion.resource.identifier;
            yield warnings_updater_1.WarningsUpdater.updateIndexDependentWarnings(document, this.indexFacade, this.documentCache, this.projectConfiguration, this, previousIdentifier, true);
            if (notifyObservers)
                this.indexFacade.notifyObservers();
        });
    }
}
exports.Datastore = Datastore;
(function (Datastore) {
    let FindResult;
    (function (FindResult) {
        FindResult.DOCUMENTS = 'documents';
        FindResult.TOTALCOUNT = 'totalCount';
    })(FindResult = Datastore.FindResult || (Datastore.FindResult = {}));
})(Datastore = exports.Datastore || (exports.Datastore = {}));
//# sourceMappingURL=datastore.js.map