/// <reference types="node" />
/// <reference types="pouchdb-core" />
import { FilesystemAdapterInterface } from './filesystem-adapter-interface';
import { ThumbnailGeneratorInterface } from './thumbnail-generator-interface';
export declare enum ImageVariant {
    ORIGINAL = "original_image",
    THUMBNAIL = "thumbnail_image",
    DISPLAY = "display_image"
}
export interface FileVariantInformation {
    size: number;
    name: ImageVariant;
}
export interface FileInfo {
    deleted: boolean;
    useOriginalForDisplay?: boolean;
    types: ImageVariant[];
    variants: FileVariantInformation[];
}
export declare const THUMBNAIL_TARGET_HEIGHT: number;
export declare const thumbnailDirectory = "thumbs/";
export declare const displayDirectory = "display/";
export declare const tombstoneSuffix = ".deleted";
export declare const useOriginalSuffix = ".original";
/**
 * @author Sebastian Cuy
 * @author Thomas Kleinke
 * @author Daniel De Oliviera
 * @author Simon Hohl
 */
export declare class ImageStore {
    private filesystem;
    private converter;
    private absolutePath;
    private activeProject;
    constructor(filesystem: FilesystemAdapterInterface, converter: ThumbnailGeneratorInterface);
    getAbsoluteRootPath: () => string | undefined;
    getActiveProject: () => string | undefined;
    /**
     * Initializiation function.
     * @param fileSystemBasePath The base path for the project's image store. Will be used to construct absolute
     * paths for the injected {@link FilesystemAdapterInterface} implementation.
     * @param activeProject the application's active project, will be used as the default project parameter for
     * other functions.
     */
    init(fileSystemBasePath: string, activeProject: string): Promise<void>;
    /**
     * Store data with the provided id.
     * @param uuid the identifier for the data
     * @param data the binary data to be stored
     * @param project (optional) the project identifier, will default to the application's current active project
     * @param type (optional) image's type, will default to {@link ImageVariant.ORIGINAL}.
     */
    store(uuid: string, data: Buffer, project?: string, type?: ImageVariant): Promise<void>;
    /**
     * Returns the raw Buffer data for the requested image.
     * @param uuid the identifier for the image
     * @param type variant type of the image, see {@link ImageVariant}.
     * @param project (optional) the project identifier, will default to the application's current active project
     */
    getData(uuid: string, type: ImageVariant, project?: string): Promise<Buffer>;
    /**
     * Removes the image from the filesystem and creates an empty tombstone file with
     * the same name plus a {@link tombstoneSuffix}.
     * @param uuid the identifier for the image to be removed
     * @param project (optional) the project identifier, will default to the application's current active project
     */
    remove(uuid: string, project?: string): Promise<any>;
    /**
     * Adds a marker file with the same name plus a {@link useOriginalSuffix} indicating that the original variant
     * should be used for the display variant.
     * @param uuid the identifier of the image
     * @param project (optional) the project identifier, will default to the application's current active project
     */
    addUseOriginalMarker(uuid: string, project?: string): Promise<any>;
    /**
     * Remove the image store data for the given project.
     * @param project the project identifier
     */
    deleteData(project: string): Promise<any>;
    /**
     * Returns all known images and lists their available variants in a project.
     * @param project the project identifier
     * @param types (optional) List of {@link ImageVariant} one is interested in. If an empty list is provided (default), images are not filtered
     * by their variants.
     * @returns Object where each key represents an image UUID and each value is the image's {@link FileInfo}.
     */
    getFileInfos(project: string, types?: ImageVariant[]): Promise<{
        [uuid: string]: FileInfo;
    }>;
    createThumbnail(imageId: string, data: Buffer, project?: string): Promise<void>;
    private aggregateFileMap;
    private setupDirectories;
    private getFileStats;
    private readFileSystem;
    private getDirectoryPath;
    private getFilePath;
    static getFileSizeSums(files: {
        [uuid: string]: FileInfo;
    }): {
        original_image: number;
        thumbnail_image: number;
        display_image: number;
    };
    static byteCountToDescription(byteCount: number, transform: (value: any) => string | null): string;
}
