"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImageStore = exports.useOriginalSuffix = exports.tombstoneSuffix = exports.displayDirectory = exports.thumbnailDirectory = exports.THUMBNAIL_TARGET_HEIGHT = exports.ImageVariant = void 0;
var ImageVariant;
(function (ImageVariant) {
    ImageVariant["ORIGINAL"] = "original_image";
    ImageVariant["THUMBNAIL"] = "thumbnail_image";
    ImageVariant["DISPLAY"] = "display_image";
})(ImageVariant = exports.ImageVariant || (exports.ImageVariant = {}));
exports.THUMBNAIL_TARGET_HEIGHT = 320;
exports.thumbnailDirectory = 'thumbs/';
exports.displayDirectory = 'display/';
exports.tombstoneSuffix = '.deleted';
exports.useOriginalSuffix = '.original';
/**
 * @author Sebastian Cuy
 * @author Thomas Kleinke
 * @author Daniel De Oliviera
 * @author Simon Hohl
 */
class ImageStore {
    constructor(filesystem, converter) {
        this.filesystem = filesystem;
        this.converter = converter;
        this.absolutePath = undefined;
        this.activeProject = undefined;
        this.getAbsoluteRootPath = () => this.absolutePath;
        this.getActiveProject = () => this.activeProject;
    }
    /**
     * Initializiation function.
     * @param fileSystemBasePath The base path for the project's image store. Will be used to construct absolute
     * paths for the injected {@link FilesystemAdapterInterface} implementation.
     * @param activeProject the application's active project, will be used as the default project parameter for
     * other functions.
     */
    init(fileSystemBasePath, activeProject) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.absolutePath = fileSystemBasePath.endsWith('/') ? fileSystemBasePath : fileSystemBasePath + '/';
                this.activeProject = activeProject;
                yield this.setupDirectories(activeProject);
            }
            catch (e) {
                this.absolutePath = undefined;
                throw (e);
            }
        });
    }
    /**
     * Store data with the provided id.
     * @param uuid the identifier for the data
     * @param data the binary data to be stored
     * @param project (optional) the project identifier, will default to the application's current active project
     * @param type (optional) image's type, will default to {@link ImageVariant.ORIGINAL}.
     */
    store(uuid, data, project = this.activeProject, type = ImageVariant.ORIGINAL) {
        return __awaiter(this, void 0, void 0, function* () {
            const filePath = this.getFilePath(project, type, uuid);
            yield this.setupDirectories(project);
            yield this.filesystem.writeFile(filePath, data);
        });
    }
    /**
     * Returns the raw Buffer data for the requested image.
     * @param uuid the identifier for the image
     * @param type variant type of the image, see {@link ImageVariant}.
     * @param project (optional) the project identifier, will default to the application's current active project
     */
    getData(uuid, type, project = this.activeProject) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.readFileSystem(uuid, type, project);
        });
    }
    /**
     * Removes the image from the filesystem and creates an empty tombstone file with
     * the same name plus a {@link tombstoneSuffix}.
     * @param uuid the identifier for the image to be removed
     * @param project (optional) the project identifier, will default to the application's current active project
     */
    remove(uuid, project = this.activeProject) {
        return __awaiter(this, void 0, void 0, function* () {
            yield Promise.all([
                this.filesystem.remove(this.getFilePath(project, ImageVariant.ORIGINAL, uuid)),
                this.filesystem.writeFile(this.getFilePath(project, ImageVariant.ORIGINAL, uuid) + exports.tombstoneSuffix, Buffer.from([])),
                this.filesystem.remove(this.getFilePath(project, ImageVariant.THUMBNAIL, uuid)),
                this.filesystem.writeFile(this.getFilePath(project, ImageVariant.THUMBNAIL, uuid) + exports.tombstoneSuffix, Buffer.from([])),
                this.filesystem.remove(this.getFilePath(project, ImageVariant.DISPLAY, uuid)),
                this.filesystem.remove(this.getFilePath(project, ImageVariant.DISPLAY, uuid + exports.useOriginalSuffix))
            ]);
        });
    }
    /**
     * Adds a marker file with the same name plus a {@link useOriginalSuffix} indicating that the original variant
     * should be used for the display variant.
     * @param uuid the identifier of the image
     * @param project (optional) the project identifier, will default to the application's current active project
     */
    addUseOriginalMarker(uuid, project = this.activeProject) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setupDirectories(project);
            yield this.filesystem.writeFile(this.getFilePath(project, ImageVariant.DISPLAY, uuid) + exports.useOriginalSuffix, Buffer.from([]));
        });
    }
    /**
     * Remove the image store data for the given project.
     * @param project the project identifier
     */
    deleteData(project) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.filesystem.remove(this.getDirectoryPath(project), true);
        });
    }
    /**
     * Returns all known images and lists their available variants in a project.
     * @param project the project identifier
     * @param types (optional) List of {@link ImageVariant} one is interested in. If an empty list is provided (default), images are not filtered
     * by their variants.
     * @returns Object where each key represents an image UUID and each value is the image's {@link FileInfo}.
     */
    getFileInfos(project, types = []) {
        return __awaiter(this, void 0, void 0, function* () {
            let originalFileStats = [];
            let thumbnailFileStats = [];
            let displayFileStats = [];
            if (types.length === 0 || types.includes(ImageVariant.ORIGINAL)) {
                originalFileStats = yield this.getFileStats(this.getDirectoryPath(project, ImageVariant.ORIGINAL));
            }
            if (types.length === 0 || types.includes(ImageVariant.THUMBNAIL)) {
                thumbnailFileStats = yield this.getFileStats(this.getDirectoryPath(project, ImageVariant.THUMBNAIL));
            }
            if (types.length === 0 || types.includes(ImageVariant.DISPLAY)) {
                displayFileStats = yield this.getFileStats(this.getDirectoryPath(project, ImageVariant.DISPLAY));
            }
            let result = this.aggregateFileMap({}, originalFileStats, ImageVariant.ORIGINAL);
            result = this.aggregateFileMap(result, thumbnailFileStats, ImageVariant.THUMBNAIL);
            result = this.aggregateFileMap(result, displayFileStats, ImageVariant.DISPLAY);
            return result;
        });
    }
    createThumbnail(imageId, data, project = this.activeProject) {
        return __awaiter(this, void 0, void 0, function* () {
            const buffer = yield this.converter.generate(data, exports.THUMBNAIL_TARGET_HEIGHT);
            const thumbnailPath = this.getFilePath(project, ImageVariant.THUMBNAIL, imageId);
            yield this.filesystem.writeFile(thumbnailPath, buffer);
        });
    }
    aggregateFileMap(aggregated, fileStatList, variant) {
        for (const stat of fileStatList) {
            let uuid = stat.path;
            let deleted = false;
            let useOriginal = false;
            if (uuid.endsWith(exports.tombstoneSuffix)) {
                deleted = true;
                uuid = uuid.replace(exports.tombstoneSuffix, '');
            }
            else if (uuid.endsWith(exports.useOriginalSuffix)) {
                useOriginal = true;
                uuid = uuid.replace(exports.useOriginalSuffix, '');
            }
            if (uuid in aggregated) {
                aggregated[uuid].types.push(variant);
                aggregated[uuid].variants.push({
                    size: stat.size, name: variant
                });
            }
            else {
                aggregated[uuid] = {
                    types: [variant],
                    variants: [{
                            size: stat.size,
                            name: variant
                        }],
                    deleted
                };
            }
            ;
            if (useOriginal)
                aggregated[uuid].useOriginalForDisplay = true;
        }
        return aggregated;
    }
    setupDirectories(project) {
        return __awaiter(this, void 0, void 0, function* () {
            const originalsPath = this.getDirectoryPath(project, ImageVariant.ORIGINAL);
            if (!(yield this.filesystem.exists(originalsPath))) {
                yield this.filesystem.mkdir(originalsPath, true);
            }
            const thumbnailsPath = this.getDirectoryPath(project, ImageVariant.THUMBNAIL);
            if (!(yield this.filesystem.exists(thumbnailsPath))) {
                yield this.filesystem.mkdir(thumbnailsPath, true);
            }
            const displayPath = this.getDirectoryPath(project, ImageVariant.DISPLAY);
            if (!(yield this.filesystem.exists(displayPath))) {
                yield this.filesystem.mkdir(displayPath, true);
            }
        });
    }
    getFileStats(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const listFiles = yield this.filesystem.listFiles(path);
            return listFiles.map((filePath) => {
                // Strip all directories from path so just file names remain.
                filePath.path = filePath.path.slice((path).length);
                return filePath;
            });
        });
    }
    readFileSystem(imageId, variant, project) {
        return __awaiter(this, void 0, void 0, function* () {
            const path = this.getFilePath(project, variant, imageId);
            if (variant === ImageVariant.DISPLAY && (yield this.filesystem.exists(path + exports.useOriginalSuffix))) {
                return this.readFileSystem(imageId, ImageVariant.ORIGINAL, project);
            }
            if (variant === ImageVariant.THUMBNAIL && !(yield this.filesystem.exists(path))) {
                const originalFilePath = this.getFilePath(project, ImageVariant.ORIGINAL, imageId);
                if (yield this.filesystem.exists(originalFilePath)) {
                    yield this.createThumbnail(imageId, yield this.filesystem.readFile(originalFilePath), project);
                }
            }
            return this.filesystem.readFile(path);
        });
    }
    getDirectoryPath(project, type) {
        switch (type) {
            case ImageVariant.ORIGINAL:
            case undefined:
                return this.absolutePath + project + '/';
            case ImageVariant.THUMBNAIL:
                return this.absolutePath + project + '/' + exports.thumbnailDirectory;
            case ImageVariant.DISPLAY:
                return this.absolutePath + project + '/' + exports.displayDirectory;
        }
    }
    getFilePath(project, type, uuid) {
        return this.getDirectoryPath(project, type) + uuid;
    }
    static getFileSizeSums(files) {
        const sums = {
            original_image: 0,
            thumbnail_image: 0,
            display_image: 0
        };
        for (const fileInfo of Object.values(files)) {
            for (const variant of fileInfo.variants) {
                sums[variant.name] += variant.size;
            }
        }
        return sums;
    }
    static byteCountToDescription(byteCount, transform) {
        byteCount = byteCount * 0.00000095367;
        let unitTypeOriginal = 'MB';
        if (byteCount > 1000) {
            byteCount = byteCount * 0.00097656;
            unitTypeOriginal = 'GB';
        }
        return `${transform(byteCount.toFixed(2))} ${unitTypeOriginal}`;
    }
}
exports.ImageStore = ImageStore;
//# sourceMappingURL=image-store.js.map