"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImageSyncService = void 0;
const sync_status_1 = require("../sync-status");
const image_store_1 = require("./image-store");
class ImageSyncService {
    constructor(imageStore, remoteImageStore, datastore) {
        this.imageStore = imageStore;
        this.remoteImageStore = remoteImageStore;
        this.longIntervalDuration = 1000 * 60 * 5;
        this.shortIntervalDuration = 1000;
        this.active = [];
        this.schedules = {};
        this.status = {
            'original_image': sync_status_1.SyncStatus.Offline,
            'thumbnail_image': sync_status_1.SyncStatus.Offline
        };
        this.inProcess = {};
        datastore.changesNotifications().subscribe((_) => __awaiter(this, void 0, void 0, function* () { return this.triggerUpdate(); }));
        datastore.deletedNotifications().subscribe(_ => this.triggerUpdate());
    }
    getStatus() {
        return this.status;
    }
    /**
     * Start syncing images of the given {@link ImageVariant}.
     */
    startSync(preference) {
        console.log(`Starting sync for ${preference.variant}.`);
        if (!this.isVariantSyncActive(preference.variant)) {
            this.active.push(preference);
        }
        if (preference.variant in this.schedules) {
            // If there is a sync schedule, stop schedule because we will
            // sync immediately.
            clearTimeout(this.schedules[preference.variant]);
        }
        this.status[preference.variant] = sync_status_1.SyncStatus.Connecting;
        this.sync(preference);
    }
    /**
     * Stop syncing images of the given {@link ImageVariant}.
     */
    stopSync(variant) {
        console.log(`Stopping sync for ${variant}.`);
        this.active = this.active.filter((value) => value.variant !== variant);
        if (variant in this.schedules) {
            clearTimeout(this.schedules[variant]);
            if (this.inProcess[variant])
                delete this.inProcess[variant];
        }
        this.status[variant] = sync_status_1.SyncStatus.Offline;
    }
    /**
     * Stop syncing for all {@link ImageVariant}.
     */
    stopAllSyncing() {
        this.stopSync(image_store_1.ImageVariant.THUMBNAIL);
        this.stopSync(image_store_1.ImageVariant.ORIGINAL);
    }
    triggerUpdate() {
        this.active.forEach(preference => {
            this.scheduleNextSync(preference, this.shortIntervalDuration);
        });
    }
    scheduleNextSync(preference, duration = this.longIntervalDuration) {
        if (!this.isVariantSyncActive(preference.variant))
            return;
        if (this.schedules[preference.variant])
            clearTimeout(this.schedules[preference.variant]);
        this.schedules[preference.variant] = setTimeout(this.sync.bind(this), duration, preference);
    }
    sync(preference) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.inProcess[preference.variant]) {
                return this.scheduleNextSync(preference, this.shortIntervalDuration);
            }
            this.inProcess[preference.variant] = true;
            try {
                const activeProject = this.imageStore.getActiveProject();
                const [remoteData, localData] = yield Promise.all([
                    this.remoteImageStore.getFileInfos(activeProject, [preference.variant]),
                    this.imageStore.getFileInfos(activeProject, [preference.variant])
                ]);
                const differences = yield ImageSyncService.evaluateDifference(localData, remoteData, preference.variant);
                const downloadInfo = `(sync ${preference.download ? 'active' : 'inactive'})`;
                const uploadInfo = `(sync ${preference.upload ? 'active' : 'inactive'})`;
                console.log(`Image syncing differences for ${preference.variant}`);
                console.log(` missing locally ${downloadInfo}: ${Object.keys(differences.missingLocally).length}`);
                console.log(` not yet deleted locally: ${differences.deleteLocally.length}`);
                console.log(` missing remotely ${uploadInfo}: ${Object.keys(differences.missingRemotely).length}`);
                console.log(` not yet deleted remotely: ${differences.deleteRemotely.length}`);
                if (preference.download) {
                    for (const uuid of Object.keys(differences.missingLocally)) {
                        if (!this.isVariantSyncActive(preference.variant))
                            return; // Stop if sync was disabled while iterating
                        this.status[preference.variant] = sync_status_1.SyncStatus.Pulling;
                        const data = yield this.remoteImageStore.getData(uuid, preference.variant, activeProject);
                        if (data !== null) {
                            yield this.imageStore.store(uuid, data, activeProject, preference.variant);
                        }
                        else {
                            throw Error(`Expected remote image ${uuid}, ${preference.variant} for project ${activeProject}, received null.`);
                        }
                    }
                }
                for (const uuid of differences.deleteLocally) {
                    if (!this.isVariantSyncActive(preference.variant))
                        return; // Stop if sync was disabled while iterating
                    this.status[preference.variant] = sync_status_1.SyncStatus.Pulling;
                    yield this.imageStore.remove(uuid, activeProject);
                }
                if (preference.upload) {
                    for (const uuid of Object.keys(differences.missingRemotely)) {
                        if (!this.isVariantSyncActive(preference.variant))
                            return; // Stop if sync was disabled while iterating
                        this.status[preference.variant] = sync_status_1.SyncStatus.Pushing;
                        const data = yield this.imageStore.getData(uuid, preference.variant, activeProject);
                        const status = yield this.remoteImageStore.store(uuid, data, activeProject, preference.variant);
                        if (status === 409) {
                            console.log(`Got status code 409. Syncing target currently does not accept large files:`);
                            console.log(`Stopping further upload attempts for ${preference.variant} files for ${this.longIntervalDuration} ms.`);
                            break;
                        }
                    }
                }
                for (const uuid of differences.deleteRemotely) {
                    if (!this.isVariantSyncActive(preference.variant))
                        return; // Stop if sync was disabled while iterating
                    this.status[preference.variant] = sync_status_1.SyncStatus.Pushing;
                    yield this.remoteImageStore.remove(uuid, activeProject);
                }
                // Set SyncStatus.Offline if sync was disabled while running sync, otherwise set SyncStatus.InSync
                this.status[preference.variant] = this.isVariantSyncActive(preference.variant) ? sync_status_1.SyncStatus.InSync : sync_status_1.SyncStatus.Offline;
            }
            catch (e) {
                this.status[preference.variant] = sync_status_1.SyncStatus.Error;
                console.error(e);
            }
            this.inProcess[preference.variant] = false;
            this.scheduleNextSync(preference);
        });
    }
    static evaluateDifference(localData, remoteData, variant) {
        return __awaiter(this, void 0, void 0, function* () {
            const localUUIDs = Object.keys(localData);
            const remoteUUIDs = Object.keys(remoteData);
            let uuidsMissingLocally = remoteUUIDs.filter((remoteUUID) => {
                return !localUUIDs.includes(remoteUUID)
                    || !(localData[remoteUUID].variants.map(variant => variant.name).includes(variant));
            }).filter(
            // We do not want to download files marked as deleted remotely.
            (remoteUUID) => !remoteData[remoteUUID].deleted);
            let deleteLocally = remoteUUIDs.filter((remoteUUID) => {
                return remoteData[remoteUUID].deleted
                    && localData[remoteUUID]
                    && !localData[remoteUUID].deleted;
            });
            let uuidsMissingRemotely = localUUIDs.filter((localUUID) => {
                return !remoteUUIDs.includes(localUUID)
                    || (remoteData[localUUID].variants
                        ? !remoteData[localUUID].variants.map(variant => variant.name).includes(variant)
                        : !remoteData[localUUID].types.includes(variant));
                // TODO Remove fallback check for types in a future version; types array is deprecated
            }).filter(
            // We do not want to upload files marked as deleted locally.
            (localUUID) => !localData[localUUID].deleted);
            let deleteRemotely = localUUIDs.filter((localUUID) => {
                return localData[localUUID].deleted
                    && remoteData[localUUID]
                    && !remoteData[localUUID].deleted;
            });
            const missingLocally = {};
            for (const uuid of uuidsMissingLocally) {
                missingLocally[uuid] = remoteData[uuid];
            }
            const missingRemotely = {};
            for (const uuid of uuidsMissingRemotely) {
                missingRemotely[uuid] = localData[uuid];
            }
            return {
                missingLocally: missingLocally,
                missingRemotely: missingRemotely,
                deleteLocally: deleteLocally,
                deleteRemotely: deleteRemotely
            };
        });
    }
    isVariantSyncActive(variant) {
        return this.active.find(val => val.variant === variant);
    }
}
exports.ImageSyncService = ImageSyncService;
//# sourceMappingURL=image-sync-service.js.map