"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PouchdbDatastore = void 0;
const document_1 = require("../../model/document");
const tools_1 = require("../../tools");
const datastore_errors_1 = require("../datastore-errors");
const change_history_merge_1 = require("./change-history-merge");
/**
 * @author Sebastian Cuy
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
class PouchdbDatastore {
    constructor(pouchDbFactory, idGenerator) {
        this.pouchDbFactory = pouchDbFactory;
        this.idGenerator = idGenerator;
        this.changesObservers = [];
        this.deletedObservers = [];
        // There is an issue where docs pop up in }).on('change',
        // despite them being deleted in remove before. When they
        // pop up in 'change', they do not have the deleted property.
        // So in order to identify them as to remove from the indices
        // they are marked 'manually'.
        this.deletedOnes = [];
        this.getDb = () => this.db;
        this.destroyDb = (dbName) => this.pouchDbFactory(dbName).destroy();
        this.setDb_e2e = (db) => this.db = db;
        this.changesNotifications = () => tools_1.ObserverUtil.register(this.changesObservers);
        this.deletedNotifications = () => tools_1.ObserverUtil.register(this.deletedObservers);
    }
    createEmptyDb(name, destroyExisting = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const db = this.pouchDbFactory(name);
            const info = yield db.info();
            if (info.update_seq !== 0) {
                if (destroyExisting) {
                    yield db.destroy();
                    return this.createEmptyDb(name);
                }
                else {
                    throw 'DB not empty';
                }
            }
            return db;
        });
    }
    createDbForTesting(dbName) {
        this.db = this.pouchDbFactory(dbName);
        return this.db;
    }
    /**
     * Creates a new database. Unless specified specifically
     * with destroyBeforeCreate set to true,
     * a possible existing database with the specified name will get used
     * and not overwritten.
     */
    createDb(name, projectDocument, configurationDocument, destroyBeforeCreate = false) {
        return __awaiter(this, void 0, void 0, function* () {
            let db = this.pouchDbFactory(name);
            if (destroyBeforeCreate) {
                yield db.destroy();
                db = this.pouchDbFactory(name);
            }
            // Create project & configuration documents only if they do not exist,
            // which can happen if the db already existed
            if (projectDocument) {
                try {
                    yield db.get('project');
                }
                catch (_a) {
                    yield db.put(projectDocument);
                    if (configurationDocument) {
                        try {
                            yield db.get('configuration');
                        }
                        catch (_b) {
                            yield db.put(configurationDocument);
                        }
                    }
                }
            }
            this.db = db;
            return db;
        });
    }
    close() {
        if (this.db)
            this.db.close();
    }
    /**
     * @returns newest revision of the document fetched from db
     * @throws [DOCUMENT_RESOURCE_ID_EXISTS]
     * @throws [INVALID_DOCUMENT] - in case either the document given as param or
     *   the document fetched directly after db.put is not valid
     */
    create(document, username) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!document_1.Document.isValid(document, true))
                throw [datastore_errors_1.DatastoreErrors.INVALID_DOCUMENT];
            if (document.resource.id) {
                yield this.assertNotExists(document.resource.id);
                this.deletedOnes = this.deletedOnes.filter(id => id !== document.resource.id);
            }
            try {
                return yield this.performPut(PouchdbDatastore.initializeDocument(document, username, this.idGenerator.generateId()));
            }
            catch (err) {
                throw [datastore_errors_1.DatastoreErrors.GENERIC_ERROR, err];
            }
        });
    }
    bulkCreate(documents, username) {
        return __awaiter(this, void 0, void 0, function* () {
            const initializedDocuments = yield Promise.all(documents.map((document) => __awaiter(this, void 0, void 0, function* () {
                if (!document_1.Document.isValid(document, true)) {
                    throw [datastore_errors_1.DatastoreErrors.INVALID_DOCUMENT];
                }
                if (document.resource.id)
                    yield this.assertNotExists(document.resource.id);
                return PouchdbDatastore.initializeDocument(document, username, this.idGenerator.generateId());
            })));
            return this.performBulkDocs(initializedDocuments);
        });
    }
    /**
     * @returns newest revision of the document fetched from db
     * @throws [DOCUMENT_NOT_FOUND]
     * @throws [INVALID_DOCUMENT] - in case either the document given as param or
     *   the document fetched directly after db.put is not valid
     */
    update(document, username, squashRevisionsIds) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!document.resource.id)
                throw [datastore_errors_1.DatastoreErrors.DOCUMENT_NO_RESOURCE_ID];
            if (!document_1.Document.isValid(document))
                throw [datastore_errors_1.DatastoreErrors.INVALID_DOCUMENT];
            const fetchedDocument = yield this.fetch(document.resource.id, undefined, true);
            const clonedDocument = document_1.Document.clone(document);
            clonedDocument.created = fetchedDocument.created;
            clonedDocument.modified = fetchedDocument.modified;
            if (squashRevisionsIds) {
                yield this.mergeModifiedDates(clonedDocument, squashRevisionsIds);
                yield this.removeRevisions(clonedDocument.resource.id, squashRevisionsIds);
            }
            clonedDocument.modified.push({ user: username, date: new Date() });
            clonedDocument['_id'] = clonedDocument.resource.id;
            try {
                return yield this.performPut(clonedDocument);
            }
            catch (err) {
                if (err.name && err.name === 'conflict') {
                    throw [datastore_errors_1.DatastoreErrors.SAVE_CONFLICT];
                }
                else {
                    console.error('Error while updating resource:', err);
                    throw [datastore_errors_1.DatastoreErrors.GENERIC_ERROR, err];
                }
            }
        });
    }
    bulkUpdate(documents, username) {
        for (let document of documents) {
            if (!document.resource.id)
                throw [datastore_errors_1.DatastoreErrors.DOCUMENT_NO_RESOURCE_ID];
            if (!document_1.Document.isValid(document, true)) {
                throw [datastore_errors_1.DatastoreErrors.INVALID_DOCUMENT];
            }
            document.modified.push({ user: username, date: new Date() });
        }
        return this.performBulkDocs(documents);
    }
    /**
     * @throws [DOCUMENT_NOT_FOUND]
     */
    remove(document) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!document.resource.id)
                throw [datastore_errors_1.DatastoreErrors.DOCUMENT_NO_RESOURCE_ID];
            this.deletedOnes.push(document.resource.id);
            const fetchedDocument = yield this.fetch(document.resource.id, {}, true);
            if (fetchedDocument._conflicts && fetchedDocument._conflicts.length > 0) {
                yield this.removeRevisions(fetchedDocument.resource.id, fetchedDocument._conflicts);
            }
            try {
                yield this.db.remove(fetchedDocument);
            }
            catch (genericerror) {
                throw [datastore_errors_1.DatastoreErrors.GENERIC_ERROR, genericerror];
            }
        });
    }
    /**
     * @param resourceId
     * @param options
     * @param skipValidation
     * @throws [DOCUMENT_NOT_FOUND]
     * @throws [INVALID_DOCUMENT] if not Document.isValid
     * @returns Document. It is made sure that it isValid and the dates are converted to type Date.
     */
    fetch(resourceId, options = {}, skipValidation = false) {
        // Beware that for this to work we need to make sure
        // the document _id/id and the resource.id are always the same.
        options.conflicts = true;
        return this.db.get(resourceId, options)
            .then((result) => {
            PouchdbDatastore.autoFixCategory(result);
            if (!skipValidation) {
                if (!document_1.Document.isValid(result))
                    return Promise.reject([datastore_errors_1.DatastoreErrors.INVALID_DOCUMENT]);
            }
            PouchdbDatastore.convertDates(result);
            return Promise.resolve(result);
        }, (_) => Promise.reject([datastore_errors_1.DatastoreErrors.DOCUMENT_NOT_FOUND]));
    }
    bulkFetch(resourceIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const options = {
                keys: resourceIds,
                conflicts: true,
                include_docs: true
            };
            return (yield this.db.allDocs(options)).rows.map((row) => {
                if (!row.doc) {
                    console.warn('Document not found: ' + row.key);
                    return undefined;
                }
                PouchdbDatastore.autoFixCategory(row.doc);
                if (!document_1.Document.isValid(row.doc)) {
                    console.warn('Invalid document', row.doc);
                    return undefined;
                }
                PouchdbDatastore.convertDates(row.doc);
                return row.doc;
            }).filter((document) => document !== undefined);
        });
    }
    /**
     * @throws [DOCUMENT_NOT_FOUND]
     * @throws [INVALID_DOCUMENT]
     */
    fetchRevision(resourceId, revisionId) {
        return this.fetch(resourceId, { rev: revisionId });
    }
    setupChangesEmitter() {
        this.db.changes({
            live: true,
            include_docs: false,
            conflicts: true,
            since: 'now'
        }).on('change', (change) => {
            // it is noteworthy that currently often after a deletion of a document we get a change that does not reflect deletion.
            // neither is change.deleted set nor is sure if the document already is deleted (meaning fetch still works)
            if (!change || !change.id)
                return;
            if (change.id.indexOf('_design') === 0)
                return; // starts with _design
            if (change.deleted || this.deletedOnes.indexOf(change.id) != -1) {
                tools_1.ObserverUtil.notify(this.deletedObservers, { resource: { id: change.id } });
                return;
            }
            this.handleNonDeletionChange(change.id);
        }).on('complete', (info) => {
            // console.debug('changes stream was canceled', info);
        }).on('error', (err) => {
            console.error('changes stream errored', err);
        });
    }
    getLatestChange() {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            return (_c = (_b = (_a = (yield this.db.changes({ descending: true, limit: 1 }))) === null || _a === void 0 ? void 0 : _a.results) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.id;
        });
    }
    performPut(document) {
        return __awaiter(this, void 0, void 0, function* () {
            const cleanedDocument = PouchdbDatastore.cleanUp(document);
            yield this.db.put(cleanedDocument, { force: true });
            return this.fetch(cleanedDocument.resource.id);
        });
    }
    performBulkDocs(documents) {
        return __awaiter(this, void 0, void 0, function* () {
            const cleanedUpDocuments = documents.map(PouchdbDatastore.cleanUp);
            yield this.db.bulkDocs(cleanedUpDocuments);
            return this.bulkFetch(cleanedUpDocuments.map(document => document.resource.id));
        });
    }
    mergeModifiedDates(document, squashRevisionsIds) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let revisionId of squashRevisionsIds) {
                change_history_merge_1.ChangeHistoryMerge.mergeChangeHistories(document, yield this.fetchRevision(document.resource.id, revisionId));
            }
        });
    }
    removeRevisions(resourceId, squashRevisionsIds) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!resourceId)
                return;
            try {
                for (let revisionId of squashRevisionsIds)
                    yield this.db.remove(resourceId, revisionId);
            }
            catch (err) {
                throw [datastore_errors_1.DatastoreErrors.REMOVE_REVISIONS_ERROR, err];
            }
        });
    }
    handleNonDeletionChange(changeId) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                tools_1.ObserverUtil.notify(this.changesObservers, yield this.fetch(changeId));
            }
            catch (e) {
                console.warn('PouchdbDatastore.handleNonDeletionChange: Document not found or not valid:', changeId, e);
            }
        });
    }
    assertNotExists(resourceId) {
        return __awaiter(this, void 0, void 0, function* () {
            let exists = false;
            try {
                yield this.db.get(resourceId);
                exists = true;
            }
            catch (_a) { }
            if (exists)
                throw [datastore_errors_1.DatastoreErrors.DOCUMENT_RESOURCE_ID_EXISTS];
        });
    }
    static convertDates(result) {
        result.created.date = new Date(result.created.date);
        for (let modified of result.modified)
            modified.date = new Date(modified.date);
        return result;
    }
    static initializeDocument(document, username, generatedId) {
        const clonedDocument = document_1.Document.clone(document);
        if (!clonedDocument.resource.id)
            clonedDocument.resource.id = generatedId;
        clonedDocument['_id'] = clonedDocument.resource.id;
        clonedDocument['created'] = { user: username, date: new Date() };
        clonedDocument['modified'] = [];
        return clonedDocument;
    }
    static cleanUp(document) {
        const clonedDocument = document_1.Document.clone(document);
        return document_1.Document.removeEmptyRelationArrays(clonedDocument);
    }
    static autoFixCategory(document) {
        if (document.resource.id === 'project')
            document.resource.category = 'Project';
        if (document.resource.id === 'configuration')
            document.resource.category = 'Configuration';
    }
}
exports.PouchdbDatastore = PouchdbDatastore;
//# sourceMappingURL=pouchdb-datastore.js.map