"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SyncService = void 0;
const rxjs_1 = require("rxjs");
const observer_util_1 = require("../../tools/observer-util");
const sync_status_1 = require("../sync-status");
/**
 * @author Thomas Kleinke
 * @author Sebastian Cuy
 */
class SyncService {
    constructor(pouchdbDatastore) {
        this.pouchdbDatastore = pouchdbDatastore;
        this.status = sync_status_1.SyncStatus.Offline;
        this.password = '';
        this.sync = null;
        this.replication = null;
        this.syncStatusSubscription = null;
        this.statusObservers = [];
        this.getStatus = () => this.status;
        this.statusNotifications = () => observer_util_1.ObserverUtil.register(this.statusObservers);
    }
    init(syncTarget, project, password, checkDatabaseExistence) {
        this.syncTarget = syncTarget;
        this.project = project;
        this.password = password;
        this.checkDatabaseExistence = checkDatabaseExistence;
    }
    /**
     * @throws error if db is not empty
     */
    startReplication(target, password, project, updateSequence, destroyExisting) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.replication) {
                console.warn('replication already running, will not start replication again');
                return;
            }
            this.stopSync();
            const url = SyncService.generateUrl(target, project, password);
            const db = yield this.pouchdbDatastore.createEmptyDb(project, destroyExisting); // may throw, if not empty
            this.replication = db.replicate.from(url, {
                retry: true,
                batch_size: updateSequence < 200 ? 10 : 50,
                batches_limit: 1,
                timeout: 600000
            });
            return rxjs_1.Observable.create((obs) => {
                this.replication.on('change', (info) => { obs.next(info.last_seq); })
                    .on('complete', (info) => {
                    this.replication = undefined;
                    if (info.status === 'complete') {
                        obs.complete();
                    }
                    else {
                        this.pouchdbDatastore.destroyDb(project);
                        obs.error('canceled');
                    }
                    this.startSync();
                })
                    .on('denied', (err) => {
                    this.handleReplicationError(obs, err, project);
                })
                    .on('error', (err) => {
                    this.handleReplicationError(obs, err, project);
                });
            });
        });
    }
    ;
    handleReplicationError(observer, error, project) {
        // it's ok to remove db, because we know it was a new one
        this.pouchdbDatastore.destroyDb(project);
        this.replication = undefined;
        this.startSync();
        observer.error(error);
    }
    stopReplication() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.replication)
                return;
            this.replication.cancel();
            this.replication = undefined;
        });
    }
    startSync(setConnectingStatus = true, filter) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.syncTarget || !this.project)
                return false;
            if (this.sync) {
                console.warn('sync already running, will not start sync again');
                return false;
            }
            if (setConnectingStatus)
                this.setStatus(sync_status_1.SyncStatus.Connecting);
            if (this.currentSyncTimeout)
                clearTimeout(this.currentSyncTimeout);
            const url = SyncService.generateUrl(this.syncTarget, this.project, this.password);
            if (!(yield this.checkDatabaseExistence(url))) {
                this.setStatus(sync_status_1.SyncStatus.AuthenticationError);
                return false;
            }
            const syncStatusObserver = yield this.setupSync(url, filter);
            this.syncStatusSubscription = syncStatusObserver.subscribe(status => this.setStatus(status), err => {
                const syncStatus = SyncService.getStatusFromError(err);
                if (syncStatus !== sync_status_1.SyncStatus.AuthenticationError && syncStatus !== sync_status_1.SyncStatus.AuthorizationError) {
                    console.error('Synchronization error', err);
                }
                this.cancelSync();
                this.setStatus(syncStatus);
                this.currentSyncTimeout = setTimeout(() => this.startSync(false), 5000); // retry
            });
            return true;
        });
    }
    stopSync() {
        this.cancelSync();
        this.setStatus(sync_status_1.SyncStatus.Offline);
    }
    setStatus(status) {
        this.status = status;
        observer_util_1.ObserverUtil.notify(this.statusObservers, this.status);
    }
    setupSync(url, filter) {
        console.log('Start syncing', url);
        this.sync = this.pouchdbDatastore.getDb().sync(url, {
            live: true,
            retry: false,
            batch_size: 50,
            batches_limit: 1,
            timeout: 600000,
            filter
        });
        return rxjs_1.Observable.create((obs) => {
            this.sync.on('change', (info) => obs.next(SyncService.getStatusFromInfo(info)))
                .on('paused', () => obs.next(sync_status_1.SyncStatus.InSync))
                .on('active', () => obs.next(sync_status_1.SyncStatus.Pulling))
                .on('complete', (info) => {
                obs.next(sync_status_1.SyncStatus.Offline);
                obs.complete();
            })
                .on('error', (err) => obs.error(err));
        });
    }
    cancelSync() {
        if (this.currentSyncTimeout)
            clearTimeout(this.currentSyncTimeout);
        if (this.sync)
            this.sync.cancel();
        if (this.syncStatusSubscription)
            this.syncStatusSubscription.unsubscribe();
        this.currentSyncTimeout = null;
        this.sync = null;
        this.syncStatusSubscription = null;
    }
    static generateUrl(syncTarget, project, password) {
        if (syncTarget.indexOf('http') == -1)
            syncTarget = 'http://' + syncTarget;
        const url = !password
            ? syncTarget
            : syncTarget.replace(/(https?):\/\//, '$1://' +
                project + ':' + encodeURIComponent(password) + '@');
        return url + '/db/' + project;
    }
}
exports.SyncService = SyncService;
SyncService.getStatusFromInfo = (info) => info.direction === 'push' ? sync_status_1.SyncStatus.Pushing : sync_status_1.SyncStatus.Pulling;
SyncService.getStatusFromError = (err) => err.status === 401
    ? err.reason === 'Name or password is incorrect.'
        ? sync_status_1.SyncStatus.AuthenticationError
        : sync_status_1.SyncStatus.AuthorizationError
    : sync_status_1.SyncStatus.Error;
//# sourceMappingURL=sync-service.js.map