"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WarningsUpdater = void 0;
const tsfun_1 = require("tsfun");
const document_1 = require("../model/document");
const named_1 = require("../tools/named");
const category_form_1 = require("../model/configuration/category-form");
const field_1 = require("../model/configuration/field");
const valuelist_util_1 = require("../tools/valuelist-util");
const resource_1 = require("../model/resource");
const image_resource_1 = require("../model/image-resource");
const warnings_1 = require("../model/warnings");
const forest_1 = require("../tools/forest");
const field_resource_1 = require("../model/field-resource");
const model_1 = require("../model");
/**
 * @author Thomas Kleinke
 */
var WarningsUpdater;
(function (WarningsUpdater) {
    const FIELDS_TO_SKIP = [
        resource_1.Resource.ID, resource_1.Resource.IDENTIFIER, resource_1.Resource.CATEGORY, resource_1.Resource.RELATIONS, field_resource_1.FieldResource.SCANCODE,
        image_resource_1.ImageResource.GEOREFERENCE, image_resource_1.ImageResource.ORIGINAL_FILENAME
    ];
    /**
     * Updates all warnings for whose determination the index is not required. These warnings do no rely on
     * analyzing the document in the context of other documents.
     */
    function updateIndexIndependentWarnings(document, projectConfiguration) {
        if (document.resource.category === 'Configuration')
            return;
        const category = projectConfiguration.getCategory(document.resource.category);
        const warnings = createWarnings(document, category);
        if (warnings_1.Warnings.hasWarnings(warnings)) {
            document.warnings = warnings;
        }
        else {
            delete document.warnings;
        }
    }
    WarningsUpdater.updateIndexIndependentWarnings = updateIndexIndependentWarnings;
    /**
     * Updates all warnings for whose determination the documents must have been previously indexed.
     */
    function updateIndexDependentWarnings(document, indexFacade, documentCache, projectConfiguration, datastore, previousIdentifier, updateAll = false) {
        return __awaiter(this, void 0, void 0, function* () {
            const category = projectConfiguration.getCategory(document.resource.category);
            if (!category)
                return;
            yield updateNonUniqueIdentifierWarning(document, indexFacade, datastore, previousIdentifier, updateAll);
            yield updateResourceLimitWarning(document, category, indexFacade, projectConfiguration, datastore, updateAll);
            yield updateRelationTargetWarning(document, indexFacade, documentCache, datastore, updateAll);
            yield updateMissingOrInvalidParentWarning(document, projectConfiguration, indexFacade, documentCache, datastore, updateAll);
            yield updateOutlierWarning(document, projectConfiguration, category, indexFacade, documentCache, datastore, updateAll);
        });
    }
    WarningsUpdater.updateIndexDependentWarnings = updateIndexDependentWarnings;
    function updateNonUniqueIdentifierWarning(document, indexFacade, datastore, previousIdentifier, updateAll = false) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (indexFacade.getCount('identifier:match', document.resource.identifier) > 1) {
                if (!document.warnings)
                    document.warnings = warnings_1.Warnings.createDefault();
                if (!document.warnings.nonUniqueIdentifier) {
                    document.warnings.nonUniqueIdentifier = true;
                    updateIndex(indexFacade, document, ['nonUniqueIdentifier:exist']);
                    if (updateAll) {
                        yield updateNonUniqueIdentifierWarnings(datastore, indexFacade, document.resource.identifier);
                    }
                }
            }
            else if ((_a = document.warnings) === null || _a === void 0 ? void 0 : _a.nonUniqueIdentifier) {
                delete document.warnings.nonUniqueIdentifier;
                if (!warnings_1.Warnings.hasWarnings(document.warnings))
                    delete document.warnings;
                updateIndex(indexFacade, document, ['nonUniqueIdentifier:exist']);
            }
            if (updateAll && previousIdentifier && previousIdentifier !== document.resource.identifier
                && indexFacade.getCount('identifier:match', previousIdentifier) > 0) {
                yield updateNonUniqueIdentifierWarnings(datastore, indexFacade, previousIdentifier);
            }
        });
    }
    WarningsUpdater.updateNonUniqueIdentifierWarning = updateNonUniqueIdentifierWarning;
    function updateResourceLimitWarning(document, category, indexFacade, projectConfiguration, datastore, updateAll = false) {
        var _a, _b, _c, _d, _e;
        return __awaiter(this, void 0, void 0, function* () {
            if (!category)
                return;
            const resourceLimit = (_b = (_a = category.parentCategory) === null || _a === void 0 ? void 0 : _a.resourceLimit) !== null && _b !== void 0 ? _b : category.resourceLimit;
            const parentCategoryName = (_d = (_c = category.parentCategory) === null || _c === void 0 ? void 0 : _c.name) !== null && _d !== void 0 ? _d : category.name;
            const query = {
                categories: projectConfiguration.getCategoryWithSubcategories(parentCategoryName).map(tsfun_1.to(named_1.Named.NAME)),
                sort: { mode: 'none' }
            };
            if (resourceLimit !== undefined && indexFacade.find(query).length > resourceLimit) {
                if (!document.warnings)
                    document.warnings = warnings_1.Warnings.createDefault();
                document.warnings.resourceLimitExceeded = true;
                updateIndex(indexFacade, document, ['resourceLimitExceeded:exist']);
                if (updateAll)
                    yield updateResourceLimitWarnings(datastore, indexFacade, projectConfiguration, category);
            }
            else if ((_e = document.warnings) === null || _e === void 0 ? void 0 : _e.resourceLimitExceeded) {
                delete document.warnings.resourceLimitExceeded;
                if (!warnings_1.Warnings.hasWarnings(document.warnings))
                    delete document.warnings;
                updateIndex(indexFacade, document, ['resourceLimitExceeded:exist']);
            }
        });
    }
    WarningsUpdater.updateResourceLimitWarning = updateResourceLimitWarning;
    function updateResourceLimitWarnings(datastore, indexFacade, projectConfiguration, category) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            if (!category)
                return;
            const parentCategoryName = (_b = (_a = category.parentCategory) === null || _a === void 0 ? void 0 : _a.name) !== null && _b !== void 0 ? _b : category.name;
            const documents = (yield datastore.find({
                categories: projectConfiguration.getCategoryWithSubcategories(parentCategoryName).map(tsfun_1.to(named_1.Named.NAME)),
                sort: { mode: 'none' }
            })).documents;
            for (let document of documents) {
                yield updateResourceLimitWarning(document, projectConfiguration.getCategory(document.resource.category), indexFacade, projectConfiguration);
            }
        });
    }
    WarningsUpdater.updateResourceLimitWarnings = updateResourceLimitWarnings;
    function updateRelationTargetWarning(document, indexFacade, documentCache, datastore, updateRelationTargets = false) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const warnings = { relationNames: [], targetIds: [] };
            const relationNames = Object.keys(document.resource.relations).filter(relationName => {
                return !model_1.Relation.Hierarchy.ALL.includes(relationName);
            });
            for (let relationName of relationNames) {
                for (let targetId of document.resource.relations[relationName]) {
                    if (!documentCache.get(targetId)) {
                        if (!warnings.relationNames.includes(relationName))
                            warnings.relationNames.push(relationName);
                        if (!warnings.targetIds.includes(targetId))
                            warnings.targetIds.push(targetId);
                    }
                }
            }
            if (warnings.relationNames.length > 0) {
                if (!document.warnings)
                    document.warnings = warnings_1.Warnings.createDefault();
                document.warnings.missingRelationTargets = warnings;
                updateIndex(indexFacade, document, ['missingRelationTargets:exist']);
                updateIndex(indexFacade, document, ['missingRelationTargetIds:contain']);
            }
            else if ((_a = document.warnings) === null || _a === void 0 ? void 0 : _a.missingRelationTargets) {
                delete document.warnings.missingRelationTargets;
                if (!warnings_1.Warnings.hasWarnings(document.warnings))
                    delete document.warnings;
                updateIndex(indexFacade, document, ['missingRelationTargets:exist']);
                updateIndex(indexFacade, document, ['missingRelationTargetIds:contain']);
            }
            if (updateRelationTargets) {
                yield updateRelationTargetWarnings(datastore, documentCache, indexFacade, document.resource.id);
            }
        });
    }
    WarningsUpdater.updateRelationTargetWarning = updateRelationTargetWarning;
    function updateMissingOrInvalidParentWarning(document, projectConfiguration, indexFacade, documentCache, datastore, updateAll = false) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            let hasValidParent = true;
            const ancestors = getAncestorDocuments(document, documentCache);
            for (let i = 0; i <= ancestors.length; i++) {
                const baseDocument = i ? ancestors[i - 1] : document;
                const parentDocument = i < ancestors.length ? ancestors[i] : undefined;
                const recordedInTargetIds = baseDocument.resource.relations[model_1.Relation.Hierarchy.RECORDEDIN];
                const recordedInTargetId = (recordedInTargetIds === null || recordedInTargetIds === void 0 ? void 0 : recordedInTargetIds.length) ? recordedInTargetIds[0] : undefined;
                const recordedInTarget = recordedInTargetId ? documentCache.get(recordedInTargetId) : undefined;
                if (!document_1.Document.isValidParent(baseDocument, parentDocument, recordedInTarget, projectConfiguration)) {
                    hasValidParent = false;
                    break;
                }
            }
            if (!hasValidParent) {
                if (!document.warnings)
                    document.warnings = warnings_1.Warnings.createDefault();
                document.warnings.missingOrInvalidParent = true;
                updateIndex(indexFacade, document, ['missingOrInvalidParent:exist']);
            }
            else if ((_a = document.warnings) === null || _a === void 0 ? void 0 : _a.missingOrInvalidParent) {
                delete document.warnings.missingOrInvalidParent;
                if (!warnings_1.Warnings.hasWarnings(document.warnings))
                    delete document.warnings;
                updateIndex(indexFacade, document, ['missingOrInvalidParent:exist']);
            }
            if (updateAll) {
                yield updateMissingOrInvalidParentWarningsForDescendants(document, datastore, documentCache, indexFacade, projectConfiguration);
            }
        });
    }
    WarningsUpdater.updateMissingOrInvalidParentWarning = updateMissingOrInvalidParentWarning;
    function updateOutlierWarning(document, projectConfiguration, category, indexFacade, documentCache, datastore, updateAll = false) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const fields = category_form_1.CategoryForm.getFields(category).filter(field => {
                return field_1.Field.InputType.VALUELIST_INPUT_TYPES.concat([field_1.Field.InputType.COMPOSITE])
                    .includes(field.inputType);
            });
            const parent = getParentDocument(document, documentCache);
            const outlierWarnings = { fields: {}, values: [] };
            for (let field of fields) {
                const outlierValues = getOutlierValues(document.resource, field, documentCache.get('project'), parent === null || parent === void 0 ? void 0 : parent.resource, projectConfiguration);
                if (tsfun_1.isArray(outlierValues) && !outlierValues.length)
                    continue;
                if (tsfun_1.isObject(outlierValues) && !Object.keys(outlierValues).length)
                    continue;
                outlierWarnings.fields[field.name] = outlierValues;
                if (tsfun_1.isArray(outlierValues)) {
                    outlierWarnings.values = outlierWarnings.values.concat(outlierValues);
                }
                else {
                    outlierWarnings.values = Object.values(outlierValues).reduce((result, values) => {
                        return result.concat(values);
                    }, outlierWarnings.values);
                }
            }
            if (!tsfun_1.equal(outlierWarnings, (_b = (_a = document.warnings) === null || _a === void 0 ? void 0 : _a.outliers) !== null && _b !== void 0 ? _b : { fieldNames: [], values: [] })) {
                if (outlierWarnings.values.length) {
                    if (!document.warnings)
                        document.warnings = warnings_1.Warnings.createDefault();
                    document.warnings.outliers = outlierWarnings;
                    updateIndex(indexFacade, document, ['outliers:exist', 'outlierValues:contain']);
                }
                else if ((_c = document.warnings) === null || _c === void 0 ? void 0 : _c.outliers) {
                    delete document.warnings.outliers;
                    if (!warnings_1.Warnings.hasWarnings(document.warnings))
                        delete document.warnings;
                    updateIndex(indexFacade, document, ['outliers:exist', 'outlierValues:contain']);
                }
            }
            if (updateAll) {
                if (document.resource.category === 'Project') {
                    yield updateProjectFieldOutlierWarnings(datastore, documentCache, indexFacade, projectConfiguration);
                }
                yield updateOutlierWarningsForChildren(document, datastore, documentCache, indexFacade, projectConfiguration);
            }
        });
    }
    WarningsUpdater.updateOutlierWarning = updateOutlierWarning;
    function getOutlierValues(fieldContainer, field, projectDocument, parentResource, projectConfiguration) {
        var _a;
        const fieldContent = fieldContainer[field.name];
        if (!fieldContent)
            return [];
        if (field.inputType === field_1.Field.InputType.COMPOSITE) {
            return getOutlierValuesForCompositeField(field, fieldContent, projectDocument, parentResource, projectConfiguration);
        }
        const valuelist = valuelist_util_1.ValuelistUtil.getValuelist(field, projectDocument, projectConfiguration, parentResource);
        return valuelist
            ? tsfun_1.set((_a = valuelist_util_1.ValuelistUtil.getValuesNotIncludedInValuelist(fieldContent, valuelist)) !== null && _a !== void 0 ? _a : [])
            : [];
    }
    function getOutlierValuesForCompositeField(field, fieldContent, projectDocument, parentResource, projectConfiguration) {
        if (!tsfun_1.isArray(fieldContent))
            return {};
        let outliers = {};
        for (let entry of fieldContent) {
            if (!tsfun_1.isObject(entry))
                continue;
            for (let subfield of field.subfields) {
                if (!field_1.Field.InputType.VALUELIST_INPUT_TYPES.includes(subfield.inputType))
                    continue;
                const subfieldOutliers = getOutlierValues(entry, subfield, projectDocument, parentResource, projectConfiguration);
                if (!subfieldOutliers.length)
                    continue;
                if (outliers[subfield.name]) {
                    outliers[subfield.name] = tsfun_1.set(outliers[subfield.name].concat(subfieldOutliers));
                }
                else {
                    outliers[subfield.name] = subfieldOutliers;
                }
            }
        }
        return outliers;
    }
    function updateNonUniqueIdentifierWarnings(datastore, indexFacade, identifier) {
        return __awaiter(this, void 0, void 0, function* () {
            const documents = (yield datastore.find({
                constraints: { 'identifier:match': identifier },
                sort: { mode: 'none' }
            }, { includeResourcesWithoutValidParent: true })).documents;
            for (let document of documents) {
                yield updateNonUniqueIdentifierWarning(document, indexFacade);
            }
        });
    }
    function updateRelationTargetWarnings(datastore, documentCache, indexFacade, id) {
        return __awaiter(this, void 0, void 0, function* () {
            const documents = (yield datastore.find({
                constraints: { 'missingRelationTargetIds:contain': id },
                sort: { mode: 'none' }
            }, { includeResourcesWithoutValidParent: true })).documents;
            for (let document of documents) {
                yield updateRelationTargetWarning(document, indexFacade, documentCache, datastore);
            }
        });
    }
    function updateProjectFieldOutlierWarnings(datastore, documentCache, indexFacade, projectConfiguration) {
        return __awaiter(this, void 0, void 0, function* () {
            const categoryNames = forest_1.Tree.flatten(projectConfiguration.getCategories()).filter(category => {
                return category_form_1.CategoryForm.getFields(category).find(field => field.valuelistFromProjectField);
            }).map(tsfun_1.to(named_1.Named.NAME));
            const documents = (yield datastore.find({ categories: categoryNames, sort: { mode: 'none' } }, { includeResourcesWithoutValidParent: true })).documents;
            for (let document of documents) {
                const category = projectConfiguration.getCategory(document.resource.category);
                if (!category)
                    continue;
                yield updateOutlierWarning(document, projectConfiguration, category, indexFacade, documentCache, datastore);
            }
        });
    }
    function updateOutlierWarningsForChildren(document, datastore, documentCache, indexFacade, projectConfiguration) {
        return __awaiter(this, void 0, void 0, function* () {
            const documents = (yield datastore.find({
                constraints: { 'isChildOf:contain': document.resource.id },
                sort: { mode: 'none' }
            }, { includeResourcesWithoutValidParent: true })).documents;
            for (let document of documents) {
                const category = projectConfiguration.getCategory(document.resource.category);
                if (!category)
                    continue;
                yield updateOutlierWarning(document, projectConfiguration, category, indexFacade, documentCache, datastore);
            }
        });
    }
    function updateMissingOrInvalidParentWarningsForDescendants(document, datastore, documentCache, indexFacade, projectConfiguration) {
        return __awaiter(this, void 0, void 0, function* () {
            const documents = (yield datastore.find({
                constraints: {
                    'isChildOf:contain': { value: document.resource.id, searchRecursively: true },
                    'missingOrInvalidParent:exist': 'KNOWN'
                },
                sort: { mode: 'none' }
            }, { includeResourcesWithoutValidParent: true })).documents;
            for (let document of documents) {
                yield updateMissingOrInvalidParentWarning(document, projectConfiguration, indexFacade, documentCache, datastore);
            }
        });
    }
    function createWarnings(document, category) {
        const warnings = warnings_1.Warnings.createDefault();
        if (!category) {
            warnings.unconfiguredCategory = true;
            return warnings;
        }
        const fieldDefinitions = category_form_1.CategoryForm.getFields(category);
        if (document._conflicts)
            warnings.conflicts = true;
        if (isIdentifierPrefixMissing(document, category))
            warnings.missingIdentifierPrefix = true;
        return Object.keys(document.resource)
            .filter(fieldName => !FIELDS_TO_SKIP.includes(fieldName))
            .reduce((result, fieldName) => {
            const fieldContent = document.resource[fieldName];
            const field = fieldDefinitions.find(tsfun_1.on(named_1.Named.NAME, tsfun_1.is(fieldName)));
            updateWarningsForField(warnings, fieldName, field, fieldContent);
            return result;
        }, warnings);
    }
    function isIdentifierPrefixMissing(document, category) {
        if (!document.resource.identifier)
            return false;
        return category.identifierPrefix && !document.resource.identifier.startsWith(category.identifierPrefix);
    }
    function updateWarningsForField(warnings, fieldName, field, fieldContent) {
        if (!field) {
            warnings.unconfiguredFields.push(fieldName);
        }
        else if (!field_1.Field.isValidFieldData(fieldContent, field)) {
            warnings.invalidFields.push(fieldName);
        }
    }
    function updateIndex(indexFacade, document, indexNames) {
        indexNames.forEach(indexName => indexFacade.putToSingleIndex(document, indexName));
        indexFacade.putToSingleIndex(document, 'warnings:exist');
    }
    function getAncestorDocuments(document, documentCache) {
        const result = [];
        let parent;
        do {
            parent = getParentDocument(parent !== null && parent !== void 0 ? parent : document, documentCache);
            if (parent) {
                if (result.includes(parent))
                    return [];
                result.push(parent);
            }
        } while (parent);
        return result;
    }
    function getParentDocument(document, documentCache) {
        return resource_1.Resource.hasRelations(document.resource, model_1.Relation.Hierarchy.LIESWITHIN)
            ? documentCache.get(document.resource.relations.liesWithin[0])
            : resource_1.Resource.hasRelations(document.resource, model_1.Relation.Hierarchy.RECORDEDIN)
                ? documentCache.get(document.resource.relations.isRecordedIn[0])
                : undefined;
    }
})(WarningsUpdater = exports.WarningsUpdater || (exports.WarningsUpdater = {}));
//# sourceMappingURL=warnings-updater.js.map