"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConstraintIndex = void 0;
const tsfun_1 = require("tsfun");
const category_form_1 = require("../model/configuration/category-form");
const field_1 = require("../model/configuration/field");
/**
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
var ConstraintIndex;
(function (ConstraintIndex) {
    function make(defaultIndexDefinitions, categories) {
        const constraintIndex = {
            indexDefinitions: {},
            containIndex: {},
            existIndex: {},
            matchIndex: {},
            linksIndex: {},
            allIndex: {}
        };
        constraintIndex.indexDefinitions = getIndexDefinitions(defaultIndexDefinitions, categories);
        const validationError = validateIndexDefinitions(Object.values(constraintIndex.indexDefinitions));
        if (validationError)
            throw validationError;
        setUp(constraintIndex);
        return constraintIndex;
    }
    ConstraintIndex.make = make;
    ConstraintIndex.clear = (index) => setUp(index);
    function put(index, document, skipRemoval = false, indexDefinitionName) {
        if (indexDefinitionName) {
            const definition = index.indexDefinitions[indexDefinitionName];
            if (!definition)
                throw 'Index definition not found: ' + indexDefinitionName;
            if (!skipRemoval)
                remove(index, document, indexDefinitionName);
            putFor(index, index.indexDefinitions[indexDefinitionName], document);
        }
        else {
            if (!skipRemoval)
                remove(index, document);
            for (let key in index.indexDefinitions) {
                putFor(index, index.indexDefinitions[key], document);
            }
            addToAllIndex(index.allIndex, document);
        }
    }
    ConstraintIndex.put = put;
    function remove(index, document, indexDefinitionName) {
        if (indexDefinitionName) {
            const definition = index.indexDefinitions[indexDefinitionName];
            if (!definition)
                throw 'Index definition not found: ' + indexDefinitionName;
            removeFromIndex(index, index.indexDefinitions[indexDefinitionName], document);
        }
        else {
            Object.values(index.indexDefinitions).forEach(definition => removeFromIndex(index, definition, document));
            if (index.allIndex[document.resource.id])
                delete index.allIndex[document.resource.id];
        }
    }
    ConstraintIndex.remove = remove;
    function get(index, indexName, matchTerms) {
        const indexDefinition = index.indexDefinitions[indexName];
        if (!indexDefinition)
            throw 'Ignoring unknown constraint "' + indexName + '".';
        const matchedDocuments = getMatches(index, indexDefinition, matchTerms);
        if (!matchedDocuments)
            return [];
        return matchedDocuments;
    }
    ConstraintIndex.get = get;
    function getWithDescendants(index, indexName, matchTerm) {
        const indexDefinition = index.indexDefinitions[indexName];
        if (!indexDefinition)
            throw 'Ignoring unknown constraint "' + indexName + '".';
        if (!indexDefinition.recursivelySearchable) {
            throw 'Illegal argument: The given index is not recursively searchable: ' + indexName;
        }
        return tsfun_1.flow(matchTerm, tsfun_1.cond(tsfun_1.not(tsfun_1.isArray), tsfun_1.singleton), tsfun_1.map(getDescendants(index, indexDefinition)), tsfun_1.flatten()) /* TODO review any*/;
    }
    ConstraintIndex.getWithDescendants = getWithDescendants;
    function getCount(index, indexName, matchTerm) {
        const indexDefinition = index.indexDefinitions[indexName];
        if (!indexDefinition)
            throw 'Ignoring unknown constraint "' + indexName + '".';
        const indexItems = getMatchesForTerm(index, indexDefinition, matchTerm);
        return indexItems
            ? Object.keys(indexItems).length
            : 0;
    }
    ConstraintIndex.getCount = getCount;
    function addIndexDefinitionsForField(index, field) {
        if (!field.constraintIndexed)
            return;
        makeIndexDefinitions(field).forEach(item => {
            index.indexDefinitions[item.name] = item.indexDefinition;
            getIndex(index, item.indexDefinition)[item.indexDefinition.path] = {};
        });
    }
    ConstraintIndex.addIndexDefinitionsForField = addIndexDefinitionsForField;
    function putFor(index, definition, document) {
        const contentAtPath = getOn(definition.pathArray, document);
        const elements = getElements(contentAtPath);
        for (let element of elements) {
            switch (definition.type) {
                case 'exist':
                    if (!isMissing(element))
                        addToExistIndex(index.existIndex, document, definition.path);
                    break;
                case 'match':
                    if ((!element && element !== false) || Array.isArray(element))
                        break;
                    addToIndex(index.matchIndex, document, definition.path, element.toString());
                    break;
                case 'contain':
                case 'contained':
                    if (!element || !Array.isArray(element))
                        break;
                    for (let target of element) {
                        addToIndex(index.containIndex, document, definition.path, target);
                    }
                    break;
                case 'links':
                    if (!element || !Array.isArray(element))
                        break;
                    addToLinksIndex(index.linksIndex, document, definition.path, element);
                    break;
            }
        }
    }
    function getElements(data) {
        if (data === undefined)
            return [undefined];
        if (tsfun_1.isObject(data))
            return Object.values(data);
        if (tsfun_1.isArray(data)) {
            data = data.reduce((result, element) => {
                if (tsfun_1.isObject(element)) {
                    result = result.concat(Object.values(element));
                }
                else {
                    result.push(element);
                }
                return result;
            }, []);
        }
        return [data];
    }
    function getOn(path, object, index = 0) {
        if (!object)
            return undefined;
        return (path.length === index + 1)
            ? object[path[index]]
            : getOn(path, object[path[index]], index + 1);
    }
    function isMissing(elementForPath) {
        return (!elementForPath && elementForPath !== false)
            || (elementForPath instanceof Array && (!elementForPath.length || elementForPath.length === 0));
    }
    function setUp(index) {
        index.containIndex = {};
        index.matchIndex = {};
        index.existIndex = {};
        index.linksIndex = {};
        for (let indexDefinition of Object.values(index.indexDefinitions)) {
            getIndex(index, indexDefinition)[indexDefinition.path] = {};
        }
    }
    function getIndex(index, definition) {
        switch (definition.type) {
            case 'contain':
            case 'contained':
                return index.containIndex;
            case 'match':
                return index.matchIndex;
            case 'exist':
                return index.existIndex;
            case 'links':
                return index.linksIndex;
        }
    }
    function getMatches(index, definition, matchTerms) {
        return Array.isArray(matchTerms)
            ? getMatchesForTerms(index, definition, matchTerms)
            : getMatchesForTerm(index, definition, matchTerms);
    }
    function getMatchesForTerms(index, definition, matchTerms) {
        const result = matchTerms.map(matchTerm => {
            return getMatchesForTerm(index, definition, matchTerm);
        }).reduce((result, indexItems) => {
            if (!indexItems)
                return result;
            return result.concat(indexItems);
        }, []);
        return Object.keys(result).length > 0 ? result : undefined;
    }
    function getMatchesForTerm(index, definition, matchTerm) {
        if (definition.type === 'exist' || definition.type === 'contained') {
            return getMatchesForKnownOrUnknown(index, definition, matchTerm);
        }
        const result = getIndex(index, definition)[definition.path][matchTerm.toLowerCase()];
        if (!result)
            return undefined;
        return Object.keys(result);
    }
    function getMatchesForKnownOrUnknown(index, definition, matchTerm) {
        var _a;
        const knownResourceIds = Object.keys((_a = getIndex(index, definition)[definition.path]) !== null && _a !== void 0 ? _a : {});
        return matchTerm === 'KNOWN'
            ? knownResourceIds
            : tsfun_1.subtract(knownResourceIds)(Object.keys(index.allIndex));
    }
    function getIndexType(field) {
        switch (field.inputType) {
            case 'checkboxes':
            case 'multiInput':
            case 'simpleMultiInput':
                return 'contain';
            default:
                return 'match';
        }
    }
    ConstraintIndex.getIndexType = getIndexType;
    function getIndexDefinitions(defaultIndexDefinitions, categories) {
        const fieldsToIndex = getFieldsToIndex(categories);
        return combine(tsfun_1.flatMap(makeIndexDefinitions, fieldsToIndex), defaultIndexDefinitions);
    }
    function getFieldsToIndex(categories) {
        return tsfun_1.flow(categories, tsfun_1.map(category_form_1.CategoryForm.getFields), tsfun_1.flatten(), getUniqueFields, tsfun_1.filter(tsfun_1.to(field_1.Field.CONSTRAINTINDEXED)));
    }
    function getUniqueFields(fields) {
        return tsfun_1.clone(fields.filter((field, index, self) => {
            return self.indexOf(self.find((f) => {
                return resultsInSameIndexDefinition(f, field);
            })) === index;
        }));
    }
    /**
     * '1'
     *  - '2'
     *    - '3'
     *
     * matchTerm: '1'
     * -> ['2', '3']
     */
    function getDescendants(index, definition) {
        return (matchTerm) => {
            const indexItems = tsfun_1.flow(getMatchesForTerm(index, definition, matchTerm), tsfun_1.cond(tsfun_1.isDefined, Object.values, []));
            return indexItems
                .concat(tsfun_1.flow(indexItems, tsfun_1.map(getDescendants(index, definition)), tsfun_1.flatten()));
        };
    }
    function resultsInSameIndexDefinition(field1, field2) {
        return field1.name === field2.name
            && field1.constraintIndexed === field2.constraintIndexed
            && ConstraintIndex.getIndexType(field1) === ConstraintIndex.getIndexType(field2);
    }
    function makeIndexDefinitions(field) {
        return tsfun_1.flatten([
            makeIndexDefinitionForField(field, getIndexType(field)),
            makeIndexDefinitionForField(field, 'exist')
        ]);
    }
    function makeIndexDefinitionForField(field, indexType) {
        if (field.inputType === field_1.Field.InputType.DROPDOWNRANGE) {
            return [
                {
                    name: field.name + '.value' + ':' + indexType,
                    indexDefinition: {
                        path: 'resource.' + field.name + '.value',
                        pathArray: ['resource', field.name, 'value'],
                        type: indexType
                    }
                },
                {
                    name: field.name + '.endValue' + ':' + indexType,
                    indexDefinition: {
                        path: 'resource.' + field.name + '.endValue',
                        pathArray: ['resource', field.name, 'endValue'],
                        type: indexType
                    }
                }
            ];
        }
        return [{
                name: field.name + ':' + indexType,
                indexDefinition: {
                    path: 'resource.' + field.name,
                    pathArray: ['resource', field.name],
                    type: indexType
                }
            }];
    }
    function combine(indexDefinitionsFromConfiguration, defaultIndexDefinitions) {
        return indexDefinitionsFromConfiguration.reduce((result, definition) => {
            result[definition.name] = definition.indexDefinition;
            return result;
        }, defaultIndexDefinitions);
    }
    function validateIndexDefinitions(indexDefinitions) {
        const types = ['match', 'contain', 'contained', 'exist', 'links'];
        for (let indexDefinition of indexDefinitions) {
            if (!indexDefinition.type)
                return 'Index definition type is undefined';
            if (types.indexOf(indexDefinition.type) == -1)
                return 'Invalid paths definition type';
            if (indexDefinition.recursivelySearchable && indexDefinition.type !== 'contain') {
                throw 'only contain indices can be configured to be recursively searchable';
            }
        }
        return undefined;
    }
    function addToIndex(index, document, path, target) {
        target = target.toString().toLowerCase();
        if (!index[path][target])
            index[path][target] = {};
        index[path][target][document.resource.id] = true;
    }
    function addToExistIndex(index, document, path) {
        if (!index[path])
            index[path] = {};
        index[path][document.resource.id] = true;
    }
    function addToLinksIndex(index, document, path, targets) {
        const ts = {};
        for (let t of targets)
            ts[t] = true;
        index[path][document.resource.id] = ts;
    }
    function addToAllIndex(index, document) {
        index[document.resource.id] = true;
    }
    function removeFromIndex(index, indexDefinition, document) {
        const path = (getIndex(index, indexDefinition))[indexDefinition.path];
        if (indexDefinition.type === 'links' || indexDefinition.type === 'exist') {
            delete path[document.resource.id];
        }
        else {
            Object.keys(path).forEach(key => {
                delete path[key][document.resource.id];
            });
        }
    }
})(ConstraintIndex = exports.ConstraintIndex || (exports.ConstraintIndex = {}));
//# sourceMappingURL=constraint-index.js.map