"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FulltextIndex = void 0;
const tsfun_1 = require("tsfun");
const result_sets_1 = require("./result-sets");
const string_utils_1 = require("../tools/string-utils");
/**
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
var FulltextIndex;
(function (FulltextIndex) {
    const tokenizationPattern = /[ \-_.]/;
    function put(index, document, fieldsToIndex, skipRemoval = false) {
        if (!skipRemoval)
            remove(index, document);
        const category = getCategory(document);
        if (!index[category])
            index[category] = { '*': {} };
        index[category]['*'][document.resource.id] = true;
        tsfun_1.flow(fieldsToIndex, tsfun_1.filter((field) => document.resource[field.name] && document.resource[field.name] !== ''), tsfun_1.flatMap((field) => getTokens(document.resource[field.name], field)), tsfun_1.map(string_utils_1.StringUtils.toLowerCase), tsfun_1.map(string_utils_1.StringUtils.toArray), tsfun_1.forEach(indexToken(index, document)));
    }
    FulltextIndex.put = put;
    function remove(index, document) {
        Object.keys(index).forEach(category => Object.keys(index[category]).forEach(term => {
            delete index[category][term][document.resource.id];
        }));
    }
    FulltextIndex.remove = remove;
    /**
     * @param index
     * @param s search string, which gets tokenized, so that the result will include
     *   search hits for any of the tokens. If s is "hello world", all items which are
     *   indexed under either "hello" or "world" will be included in the result. The
     *   result will be a set in the sense that it will include each item only once.
     * @param categories if undefined, searches in all categories. If defined, only search hits
     *   indexed under the specified categories will be included in the results.
     */
    function get(index, s, categories) {
        if (tsfun_1.isEmpty(index))
            return [];
        const resultSets = s
            .split(tokenizationPattern)
            .filter(tsfun_1.not(tsfun_1.isEmpty))
            .reduce(getFromIndex(index, categories), result_sets_1.ResultSets.make());
        return result_sets_1.ResultSets.collapse(resultSets);
    }
    FulltextIndex.get = get;
    function getFromIndex(index, categories) {
        return (resultSets, token) => {
            const ids = getForToken(index, token, categories
                ? categories
                : Object.keys(index));
            result_sets_1.ResultSets.combine(resultSets, ids);
            return resultSets;
        };
    }
    function indexToken(index, document) {
        return (tokenAsCharArray) => {
            const categoryIndex = index[getCategory(document)];
            tokenAsCharArray.reduce((accumulator, letter) => {
                accumulator += letter;
                if (!categoryIndex[accumulator])
                    categoryIndex[accumulator] = {};
                categoryIndex[accumulator][document.resource.id] = true;
                return accumulator;
            }, '');
        };
    }
    function extractReplacementTokens(s) {
        const positionOpen = s.indexOf('[');
        const positionClose = s.indexOf(']');
        return positionOpen !== -1 && positionClose !== -1 && positionOpen < positionClose ?
            { hasPlaceholder: true, tokens: s.substr(positionOpen + 1, positionClose - positionOpen - 1) } :
            { hasPlaceholder: false, tokens: '' };
    }
    function getForToken(index, token, categories) {
        const s = token.toLowerCase();
        function get(resultSets, category) {
            const { hasPlaceholder, tokens } = extractReplacementTokens(s);
            return hasPlaceholder
                ? getWithPlaceholder(index, resultSets, s, category, tokens)
                : addKeyToResultSets(index, resultSets, category, s);
        }
        return result_sets_1.ResultSets.unifyAddSets(categories.reduce(get, result_sets_1.ResultSets.make()));
    }
    function getWithPlaceholder(index, resultSets, s, category, tokens) {
        return tokens.split('').reduce((_resultSets, nextChar) => addKeyToResultSets(index, _resultSets, category, s.replace('[' + tokens + ']', nextChar)), resultSets);
    }
    function addKeyToResultSets(index, resultSets, category, s) {
        if (!index[category] || !index[category][s])
            return resultSets;
        result_sets_1.ResultSets.combine(resultSets, Object.keys(index[category][s]));
        return resultSets;
    }
    function getTokens(value, field) {
        return tsfun_1.isString(value)
            ? field.valuelist
                ? getTokensFromValuelist(value, field.valuelist)
                : getTokensFromString(value)
            : getTokensFromI18nString(value);
    }
    function getTokensFromValuelist(valueId, valuelist) {
        const value = valuelist.values[valueId];
        return value
            ? value.label && !tsfun_1.isEmpty(value.label)
                ? getTokensFromI18nString(value.label)
                : getTokensFromString(valueId)
            : [];
    }
    function getTokensFromString(stringValue) {
        return string_utils_1.StringUtils.split(tokenizationPattern)(stringValue);
    }
    function getTokensFromI18nString(i18nString) {
        return tsfun_1.flatten(Object.values(i18nString).map(text => string_utils_1.StringUtils.split(tokenizationPattern)(text)));
    }
    function getCategory(document) {
        var _a;
        return ((_a = document.warnings) === null || _a === void 0 ? void 0 : _a.unconfiguredCategory)
            ? 'UNCONFIGURED'
            : document.resource.category;
    }
})(FulltextIndex = exports.FulltextIndex || (exports.FulltextIndex = {}));
//# sourceMappingURL=fulltext-index.js.map