"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSortedIds = void 0;
const tsfun_1 = require("tsfun");
const resource_1 = require("../model/resource");
const sort_util_1 = require("../tools/sort-util");
/**
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
const INSTANCES = 'instances';
/**
 * If not specified otherwise, indexItems get sorted
 * alphanumerically by their identifier property.
 *
 * @param indexItems
 * @param query
 *   - if query.categories includes only type categories,
 *     query.sort.matchType can be set
 *   . in order to perform a ranking of Type resources then.
 *     if query.sort.matchCategory is not set, a regular
 *     sort gets performed instead.
 *   - if query.sort.mode === 'exactMatchFirst', then, after sorting,
 *     puts an element which matches the query exactly, to the
 *     front of the resulting list.
 */
function getSortedIds(indexItems, query, typeCategories) {
    const rankEntries = shouldRankCategories(query, typeCategories)
        ? rankTypeResourceIndexItems(query.sort.matchCategory)
        : rankRegularIndexItems;
    const handleExactMatchIfQuerySaysSo = tsfun_1.cond(shouldHandleExactMatch(query), handleExactMatch(query.q));
    return tsfun_1.flow(indexItems, rankEntries, handleExactMatchIfQuerySaysSo, tsfun_1.map(tsfun_1.to(resource_1.Resource.ID)));
}
exports.getSortedIds = getSortedIds;
function shouldHandleExactMatch(query) {
    var _a;
    return ((_a = query.sort) === null || _a === void 0 ? void 0 : _a.mode) === 'exactMatchFirst' && !tsfun_1.isUndefinedOrEmpty(query.q);
}
function shouldRankCategories(query, typeCategories) {
    var _a;
    return ((_a = query.sort) === null || _a === void 0 ? void 0 : _a.matchCategory) && query.categories && tsfun_1.subsetOf(typeCategories, query.categories);
}
function comparePercentages([itemA, pctgA], [itemB, pctgB]) {
    if (pctgA < pctgB)
        return 1;
    if (pctgA > pctgB)
        return -1;
    if (tsfun_1.size(itemA.instances) < tsfun_1.size(itemB.instances))
        return 1;
    if (tsfun_1.size(itemA.instances) > tsfun_1.size(itemB.instances))
        return -1;
    return sort_util_1.SortUtil.alnumCompare(itemA.identifier, itemB.identifier);
}
/**
 * { id: '1', instances: { '2', 'T1', '3': 'T2' }}
 * categoryToMatch = 'T1'
 * ->
 * 50
 */
const calcPercentage = (categoryToMatch) => tsfun_1.compose(tsfun_1.to(INSTANCES), tsfun_1.cond(tsfun_1.isUndefinedOrEmpty, 0, tsfun_1.compose(tsfun_1.tuplify(tsfun_1.count(tsfun_1.is(categoryToMatch)), tsfun_1.size), ([numMatching, numTotal]) => numMatching * 100 / numTotal)));
/**
 * [{identifier: 'a'}, {identifier: 'b'}, {identifier: 'c'}]
 * q = 'b'
 * ->
 * [{identifier: 'b'}, {identifier: 'a'}, {identifier: 'c'}]
 */
const handleExactMatch = (q) => tsfun_1.compose(tsfun_1.separate(tsfun_1.on(resource_1.Resource.IDENTIFIER, tsfun_1.is(q))), tsfun_1.flatten());
const rankRegularIndexItems = tsfun_1.sort((a, b) => sort_util_1.SortUtil.alnumCompare(a.identifier, b.identifier));
/**
 * [{id: '3', instances: {'7': 'T2'}}
 *  {id: '2', instances: {'4': 'T1', '6': 'T2'}}
 *  {id: '1', instances: {'4': 'T1', '5': 'T1'}}
 *  {id: '0', instances: {'4': 'T1', '5': 'T1', '8': 'T1'}}]
 * categoryToMatch = 'T1'
 * ->
 * [{id: '0', instances: {'4': 'T1', '5': 'T1', '8': 'T1'}} // 100%, 3 matches
 *  {id: '1', instances: {'4': 'T1', '5': 'T1'}}            // 100%, 2 matches
 *  {id: '2', instances: {'4': 'T1', '6': 'T2'}}            // 50%
 *  {id: '3', instances: {'7': 'T2'}}]                      // 0%
 */
const rankTypeResourceIndexItems = (categoryToMatch) => tsfun_1.compose(tsfun_1.map(tsfun_1.pairWith(calcPercentage(categoryToMatch))), tsfun_1.sort(comparePercentages), tsfun_1.map(tsfun_1.left));
//# sourceMappingURL=get-sorted-ids.js.map