"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndexFacade = void 0;
const tsfun_1 = require("tsfun");
const resource_1 = require("../model/resource");
const forest_1 = require("../tools/forest");
const named_1 = require("../tools/named");
const observer_util_1 = require("../tools/observer-util");
const constraint_index_1 = require("./constraint-index");
const fulltext_index_1 = require("./fulltext-index");
const get_fields_to_index_1 = require("./get-fields-to-index");
const get_sorted_ids_1 = require("./get-sorted-ids");
const index_item_1 = require("./index-item");
const perform_query_1 = require("./perform-query");
const model_1 = require("../model");
const CONFIGURATION = 'Configuration';
const INSTANCES = 'instances';
const INSTANCE_OF = 'isInstanceOf';
/**
 * @author Thomas Kleinke
 * @author Daniel de Oliveira
 */
class IndexFacade {
    constructor(constraintIndex, fulltextIndex, projectConfiguration, showWarnings) {
        this.constraintIndex = constraintIndex;
        this.fulltextIndex = fulltextIndex;
        this.projectConfiguration = projectConfiguration;
        this.showWarnings = showWarnings;
        this.observers = [];
        this.indexItems = {};
        this.changesNotifications = () => observer_util_1.ObserverUtil.register(this.observers);
    }
    find(query) {
        const queryResult = perform_query_1.performQuery(query, this.constraintIndex, this.fulltextIndex);
        return this.getSortedResult(query, queryResult);
    }
    /**
     * @param document document.resource.identifier needs to be present, otherwise document does not get indexed
     */
    put(document) {
        this._put(document, false, true);
    }
    /**
     * This does not notify observers of IndexFacade notifications. notifyObservers has to be called after all changes
     * to the index have been completed.
     */
    putToSingleIndex(document, constraintIndexName) {
        IndexFacade.setChildOfRelation(document);
        constraint_index_1.ConstraintIndex.put(this.constraintIndex, document, false, constraintIndexName);
        IndexFacade.removeChildOfRelation(document);
    }
    putMultiple(documents, setProgress) {
        return __awaiter(this, void 0, void 0, function* () {
            const [typeDocuments, nonTypeDocuments] = tsfun_1.separate(tsfun_1.on(['resource', 'category'], tsfun_1.includedIn(this.projectConfiguration.getTypeCategories().map(tsfun_1.to(named_1.Named.NAME)))), documents);
            let count = 0;
            for (let document of typeDocuments) {
                this._put(document, true, false);
                count++;
                if (setProgress && (count % 250 === 0 || count === documents.length)) {
                    yield setProgress(count * 0.75);
                }
            }
            for (let document of nonTypeDocuments) {
                this._put(document, true, false);
                count++;
                if (setProgress && (count % 250 === 0 || count === documents.length)) {
                    yield setProgress(count * 0.75);
                }
            }
            observer_util_1.ObserverUtil.notify(this.observers, undefined);
        });
    }
    remove(document) {
        constraint_index_1.ConstraintIndex.remove(this.constraintIndex, document);
        fulltext_index_1.FulltextIndex.remove(this.fulltextIndex, document);
        delete this.indexItems[document.resource.id];
        if (!this.projectConfiguration.getTypeCategories().map(tsfun_1.to(named_1.Named.NAME)).includes(document.resource.category)) {
            IndexFacade.deleteAssociatedTypeItem(this.indexItems, document);
        }
        observer_util_1.ObserverUtil.notify(this.observers, document);
    }
    clear() {
        constraint_index_1.ConstraintIndex.clear(this.constraintIndex);
        this.fulltextIndex = {};
        this.indexItems = {};
    }
    getCount(constraintIndexName, matchTerm) {
        return constraint_index_1.ConstraintIndex.getCount(this.constraintIndex, constraintIndexName, matchTerm);
    }
    getDescendantIds(constraintIndexName, matchTerm) {
        return constraint_index_1.ConstraintIndex.getWithDescendants(this.constraintIndex, constraintIndexName, matchTerm);
    }
    addConstraintIndexDefinitionsForField(field) {
        constraint_index_1.ConstraintIndex.addIndexDefinitionsForField(this.constraintIndex, field);
    }
    notifyObservers() {
        observer_util_1.ObserverUtil.notify(this.observers, undefined);
    }
    getSortedResult(query, queryResult) {
        if (query.sort && query.sort.mode === 'none') {
            return queryResult;
        }
        else {
            const indexItems = queryResult.map(tsfun_1.lookup(this.indexItems));
            return get_sorted_ids_1.getSortedIds(indexItems, query, this.projectConfiguration.getTypeCategories().map(tsfun_1.to(named_1.Named.NAME)));
        }
    }
    _put(document, skipRemoval, notify) {
        IndexFacade.setChildOfRelation(document);
        const item = this.getIndexItem(document);
        if (!item || document.resource.category === CONFIGURATION)
            return;
        item.identifier = document.resource.identifier;
        if (this.projectConfiguration.getTypeCategories().map(tsfun_1.to(named_1.Named.NAME)).includes(document.resource.category)) {
            IndexFacade.updateTypeItem(item);
        }
        else {
            if (!skipRemoval) {
                IndexFacade.deleteAssociatedTypeItem(this.indexItems, document);
            }
            IndexFacade.createAssociatedTypeItem(this.indexItems, document);
        }
        constraint_index_1.ConstraintIndex.put(this.constraintIndex, document, skipRemoval);
        fulltext_index_1.FulltextIndex.put(this.fulltextIndex, document, get_fields_to_index_1.getFieldsToIndex(named_1.Named.arrayToMap(forest_1.Tree.flatten(this.projectConfiguration.getCategories())), document.resource.category), skipRemoval);
        IndexFacade.removeChildOfRelation(document);
        if (notify)
            observer_util_1.ObserverUtil.notify(this.observers, document);
    }
    getIndexItem(document) {
        const existingItem = this.indexItems[document.resource.id];
        const indexItem = existingItem !== undefined
            ? existingItem
            : index_item_1.IndexItem.from(document, this.showWarnings);
        if (!existingItem && indexItem)
            this.indexItems[document.resource.id] = indexItem;
        return indexItem;
    }
    static createAssociatedTypeItem(items, document) {
        if (!document.resource.relations[INSTANCE_OF])
            return;
        for (let target of document.resource.relations[INSTANCE_OF]) {
            const typeItem = items[target];
            if (typeItem) {
                typeItem.instances[document.resource.id] = document.resource.category;
            }
        }
    }
    static deleteAssociatedTypeItem(items, document) {
        tsfun_1.flow(items, tsfun_1.filter(tsfun_1.on(INSTANCES, tsfun_1.isDefined)), tsfun_1.values, tsfun_1.forEach((item) => {
            delete item[INSTANCES][document.resource.id];
        }));
    }
    static updateTypeItem(item) {
        if (!item.instances) { // keep existing instances on update
            item.instances = {};
        }
    }
    static setChildOfRelation(document) {
        if (resource_1.Resource.hasRelations(document.resource, model_1.Relation.Hierarchy.LIESWITHIN)) {
            document.resource.relations['isChildOf'] = [document.resource.relations.liesWithin[0]];
        }
        else if (resource_1.Resource.hasRelations(document.resource, model_1.Relation.Hierarchy.RECORDEDIN)) {
            document.resource.relations['isChildOf'] = [document.resource.relations.isRecordedIn[0]];
        }
    }
    static removeChildOfRelation(document) {
        delete document.resource.relations['isChildOf'];
    }
}
exports.IndexFacade = IndexFacade;
//# sourceMappingURL=index-facade.js.map