"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Field = void 0;
const tsfun_1 = require("tsfun");
const validation_util_1 = require("../../tools/validation-util");
const parse_date_1 = require("../../tools/parse-date");
const dating_1 = require("../dating");
const dimension_1 = require("../dimension");
const literature_1 = require("../literature");
const optional_range_1 = require("../optional-range");
const composite_1 = require("../composite");
var Field;
(function (Field) {
    Field.INPUTTYPE = 'inputType';
    Field.SOURCE = 'source';
    Field.VISIBLE = 'visible';
    Field.MANDATORY = 'mandatory';
    Field.EDITABLE = 'editable';
    Field.NAME = 'name';
    Field.LABEL = 'label';
    Field.DESCRIPTION = 'description';
    Field.FULLTEXTINDEXED = 'fulltextIndexed';
    Field.CONSTRAINTINDEXED = 'constraintIndexed';
    Field.GROUP = 'group';
    let Source;
    (function (Source) {
        Source.BUILTIN = 'builtIn';
        Source.LIBRARY = 'library';
        Source.CUSTOM = 'custom';
        Source.COMMON = 'common';
    })(Source = Field.Source || (Field.Source = {}));
    function isValidFieldData(fieldData, field) {
        var _a;
        if (fieldData === null || fieldData === undefined)
            return false;
        switch (field.inputType) {
            case InputType.SIMPLE_INPUT:
            case InputType.DROPDOWN:
            case InputType.RADIO:
            case InputType.CATEGORY:
            case InputType.IDENTIFIER:
                return tsfun_1.isString(fieldData);
            case InputType.INPUT:
            case InputType.TEXT:
                // TODO Improve validation for i18n strings
                return tsfun_1.isString(fieldData) || tsfun_1.isObject(fieldData);
            case InputType.SIMPLE_MULTIINPUT:
            case InputType.CHECKBOXES:
                return tsfun_1.isArray(fieldData) && fieldData.every(element => tsfun_1.isString(element));
            case InputType.MULTIINPUT:
                return tsfun_1.isArray(fieldData) && fieldData.every(element => tsfun_1.isString(element) || tsfun_1.isObject(element));
            case InputType.UNSIGNEDINT:
                return validation_util_1.validateUnsignedInt(fieldData);
            case InputType.UNSIGNEDFLOAT:
                return validation_util_1.validateUnsignedFloat(fieldData);
            case InputType.INT:
                return validation_util_1.validateInt(fieldData);
            case InputType.FLOAT:
                return validation_util_1.validateFloat(fieldData);
            case InputType.URL:
                return validation_util_1.validateUrl(fieldData);
            case InputType.BOOLEAN:
                return fieldData === true || fieldData === false;
            case InputType.DATE:
                return !isNaN((_a = parse_date_1.parseDate(fieldData)) === null || _a === void 0 ? void 0 : _a.getTime());
            case InputType.DROPDOWNRANGE:
                return optional_range_1.OptionalRange.buildIsOptionalRange(tsfun_1.isString)(fieldData);
            case InputType.DATING:
                return tsfun_1.isArray(fieldData) && fieldData.every(element => {
                    return dating_1.Dating.isDating(element) && dating_1.Dating.isValid(element);
                });
            case InputType.DIMENSION:
                return tsfun_1.isArray(fieldData) && fieldData.every(element => {
                    return dimension_1.Dimension.isDimension(element) && dimension_1.Dimension.isValid(element);
                });
            case InputType.LITERATURE:
                return tsfun_1.isArray(fieldData) && fieldData.every(element => {
                    return literature_1.Literature.isLiterature(element) && literature_1.Literature.isValid(element);
                });
            case InputType.COMPOSITE:
                return tsfun_1.isArray(fieldData) && fieldData.every(element => {
                    return composite_1.Composite.isValid(element, field.subfields);
                });
            case InputType.GEOMETRY:
                return fieldData.type !== undefined && fieldData.coordinates !== undefined;
            case InputType.RELATION:
            case InputType.INSTANCE_OF:
                return tsfun_1.isObject(fieldData) && Object.values(fieldData).every(relationTargets => {
                    return tsfun_1.isArray(relationTargets) && relationTargets.every(element => tsfun_1.isString(element));
                });
            default:
                return true;
        }
    }
    Field.isValidFieldData = isValidFieldData;
    let InputType;
    (function (InputType) {
        InputType.INPUT = 'input';
        InputType.SIMPLE_INPUT = 'simpleInput';
        InputType.UNSIGNEDINT = 'unsignedInt';
        InputType.UNSIGNEDFLOAT = 'unsignedFloat';
        InputType.INT = 'int';
        InputType.FLOAT = 'float';
        InputType.TEXT = 'text';
        InputType.MULTIINPUT = 'multiInput';
        InputType.SIMPLE_MULTIINPUT = 'simpleMultiInput';
        InputType.URL = 'url';
        InputType.DROPDOWN = 'dropdown';
        InputType.DROPDOWNRANGE = 'dropdownRange';
        InputType.RADIO = 'radio';
        InputType.BOOLEAN = 'boolean';
        InputType.CHECKBOXES = 'checkboxes';
        InputType.DATING = 'dating';
        InputType.DATE = 'date';
        InputType.DIMENSION = 'dimension';
        InputType.LITERATURE = 'literature';
        InputType.GEOMETRY = 'geometry';
        InputType.INSTANCE_OF = 'instanceOf';
        InputType.RELATION = 'relation';
        InputType.DERIVED_RELATION = 'derivedRelation';
        InputType.CATEGORY = 'category';
        InputType.IDENTIFIER = 'identifier';
        InputType.COMPOSITE = 'composite';
        InputType.NONE = 'none';
        InputType.DEFAULT = 'default';
        InputType.VALUELIST_INPUT_TYPES = [InputType.DROPDOWN, InputType.DROPDOWNRANGE, InputType.CHECKBOXES, InputType.RADIO, InputType.DIMENSION];
        InputType.NUMBER_INPUT_TYPES = [InputType.UNSIGNEDINT, InputType.UNSIGNEDFLOAT, InputType.INT, InputType.FLOAT];
        InputType.I18N_COMPATIBLE_INPUT_TYPES = [InputType.INPUT, InputType.SIMPLE_INPUT, InputType.TEXT, InputType.MULTIINPUT,
            InputType.SIMPLE_MULTIINPUT];
        InputType.I18N_INPUT_TYPES = [InputType.INPUT, InputType.TEXT, InputType.MULTIINPUT];
        InputType.SIMPLE_INPUT_TYPES = [InputType.SIMPLE_INPUT, InputType.SIMPLE_MULTIINPUT];
        InputType.SUBFIELD_INPUT_TYPES = [InputType.INPUT, InputType.SIMPLE_INPUT, InputType.TEXT, InputType.BOOLEAN, InputType.DROPDOWN, InputType.RADIO,
            InputType.CHECKBOXES, InputType.FLOAT, InputType.UNSIGNEDFLOAT, InputType.INT, InputType.UNSIGNEDINT, InputType.DATE, InputType.URL];
        InputType.RELATION_INPUT_TYPES = [InputType.RELATION, InputType.INSTANCE_OF, InputType.DERIVED_RELATION];
        const INTERCHANGEABLE_INPUT_TYPES = [
            [InputType.INPUT, InputType.SIMPLE_INPUT, InputType.TEXT, InputType.DROPDOWN, InputType.RADIO],
            [InputType.MULTIINPUT, InputType.SIMPLE_MULTIINPUT, InputType.CHECKBOXES]
        ];
        function getInterchangeableInputTypes(inputType) {
            const alternativeTypes = INTERCHANGEABLE_INPUT_TYPES.find(types => {
                return types.includes(inputType);
            });
            return alternativeTypes !== null && alternativeTypes !== void 0 ? alternativeTypes : [];
        }
        InputType.getInterchangeableInputTypes = getInterchangeableInputTypes;
    })(InputType = Field.InputType || (Field.InputType = {}));
})(Field = exports.Field || (exports.Field = {}));
//# sourceMappingURL=field.js.map