import { Name } from '../../tools/named';
import { Field } from './field';
export interface Relation extends Field {
    domain: string[];
    range: string[];
    inverse?: any;
    sameMainCategoryResource?: boolean;
}
/**
 * @author Thomas Kleinke
 * @author Daniel de Oliveira
 */
export declare namespace Relation {
    const DOMAIN = "domain";
    const RANGE = "range";
    /**
     * Regarding stratigraphical units (Feature category)
     *
     * Time relations are interpretations of users, based on position relations
     */
    module Time {
        const BEFORE = "isBefore";
        const AFTER = "isAfter";
        const CONTEMPORARY = "isContemporaryWith";
        const ALL: string[];
    }
    /**
     * Regarding stratigraphical units (Feature category)
     */
    module Position {
        const BELOW = "isBelow";
        const ABOVE = "isAbove";
        const BORDERS = "borders";
        const CUTS = "cuts";
        const CUTBY = "isCutBy";
        const FILLS = "fills";
        const FILLEDBY = "isFilledBy";
        const BONDSWITH = "bondsWith";
        const ABUTS = "abuts";
        const ABUTTEDBY = "isAbuttedBy";
        const ALL: string[];
    }
    /**
     * This relation spans a general tree with resources as nodes and leafs.
     *
     * Although the inspiration for this relation is that of spatial inclusion, the actual semantics
     * are not defined precisely as spatial inclusion of volumes within other volumes but rather depend
     * on the specific context, as per the categories involved. For example in the case of a Find and
     * a Feature we would use the relation to express that the Find is mostly included in that specific
     * Feature. In the case of an Inscription and a Find we would use it to express that the Find has that
     * specific inscription, which is then a specific instance of an inscription (a token, rather than a type)
     * found on a particular Find.
     *
     * Having most of the resources of the application in such a forest allows for widely used
     * and commonly understood hierarchical navigation, as you have it in file browsers etc.
     */
    const PARENT = "isChildOf";
    /**
     * This relation spans a directed acyclical graph between resources. Note that
     * there should not be any redundant paths. That means, if A -> B -> C, there should
     * not be an explicit relation between A and C. Instead the relation is implicit as per the
     * path from A to C.
     *
     * It can be used to model relationships where one entity belongs to multiple other entities in some
     * way, as it is the case for example with Doors and Room, where a Door can be part of two Rooms.
     */
    const IS_PRESENT_IN = "isPresentIn";
    namespace Hierarchy {
        const RECORDEDIN = "isRecordedIn";
        const LIESWITHIN = "liesWithin";
        const ALL: string[];
    }
    /**
     * Used to signal sameness (a claim of identity) in a generic manner, for example in order to say
     * two resources, recorded in different operations, are the same.
     *
     * This relation complements isChildOf in so far as the tree spanned by the latter gains the
     * possibility to express sameness, while retaining its relatively simple structure
     * (more complex graphs; see also: isPresentIn). Sameness in this way can be seen as an
     * *interpretation* that parts of an entity, each found in its own (spatial) context and also maybe
     * described by different persons, actually *are* (rather than are parts of) the *same* entity.
     * For example one could have two parts of a wall which are not (visibly) physically connected,
     * but which are taken as having been the *same* wall.
     */
    const SAME_AS = "isSameAs";
    const UNIDIRECTIONAL: string[];
    /**
     * Used to connect finds with type resources
     */
    module Type {
        const INSTANCEOF = "isInstanceOf";
        const HASINSTANCE = "hasInstance";
        const ALL: string[];
    }
    module Inventory {
        const ISSTORAGEPLACEOF = "isStoragePlaceOf";
        const ISSTOREDIN = "isStoredIn";
        const ALL: string[];
    }
    module Image {
        const DEPICTS = "depicts";
        const ISDEPICTEDIN = "isDepictedIn";
        const HASMAPLAYER = "hasMapLayer";
        const ISMAPLAYEROF = "isMapLayerOf";
        const HASDEFAULTMAPLAYER = "hasDefaultMapLayer";
        const ALL: string[];
    }
    /**
     * @returns {boolean} True if the given domain category is a valid domain name for a relation definition
     * which has the given range category & name
     */
    function isAllowedRelationDomainCategory(relations: Array<Relation>, domainCategory: Name, rangeCategory: Name, relation: Name): boolean;
    function getRelations(relations: Array<Relation>, categoryName: string, isRangeCategory?: boolean): Relation[];
    type InverseRelationsMap = {
        [_: string]: // relation name for every defined relation, independent if it has an inverse or not
        string | undefined;
    };
    function makeInverseRelationsMap(relationDefinitions: Array<Relation>): InverseRelationsMap;
}
