"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Relation = void 0;
const tsfun_1 = require("tsfun");
const assoc_reduce_1 = require("../../tools/assoc-reduce");
const utils_1 = require("../../tools/utils");
/**
 * @author Thomas Kleinke
 * @author Daniel de Oliveira
 */
var Relation;
(function (Relation) {
    Relation.DOMAIN = 'domain';
    Relation.RANGE = 'range';
    /**
     * Regarding stratigraphical units (Feature category)
     *
     * Time relations are interpretations of users, based on position relations
     */
    let Time;
    (function (Time) {
        Time.BEFORE = 'isBefore';
        Time.AFTER = 'isAfter';
        Time.CONTEMPORARY = 'isContemporaryWith';
        Time.ALL = [Time.AFTER, Time.BEFORE, Time.CONTEMPORARY];
    })(Time = Relation.Time || (Relation.Time = {}));
    /**
     * Regarding stratigraphical units (Feature category)
     */
    let Position;
    (function (Position) {
        Position.BELOW = 'isBelow'; // is read off by a user by sight
        Position.ABOVE = 'isAbove'; // is read off by a user by sight
        Position.BORDERS = 'borders'; //
        Position.CUTS = 'cuts'; //
        Position.CUTBY = 'isCutBy'; //
        Position.FILLS = 'fills';
        Position.FILLEDBY = 'isFilledBy';
        Position.BONDSWITH = 'bondsWith';
        Position.ABUTS = 'abuts';
        Position.ABUTTEDBY = 'isAbuttedBy';
        Position.ALL = [Position.BELOW, Position.ABOVE, Position.BORDERS, Position.CUTS, Position.CUTBY, Position.FILLS, Position.FILLEDBY, Position.BONDSWITH, Position.ABUTS, Position.ABUTTEDBY];
    })(Position = Relation.Position || (Relation.Position = {}));
    /**
     * This relation spans a general tree with resources as nodes and leafs.
     *
     * Although the inspiration for this relation is that of spatial inclusion, the actual semantics
     * are not defined precisely as spatial inclusion of volumes within other volumes but rather depend
     * on the specific context, as per the categories involved. For example in the case of a Find and
     * a Feature we would use the relation to express that the Find is mostly included in that specific
     * Feature. In the case of an Inscription and a Find we would use it to express that the Find has that
     * specific inscription, which is then a specific instance of an inscription (a token, rather than a type)
     * found on a particular Find.
     *
     * Having most of the resources of the application in such a forest allows for widely used
     * and commonly understood hierarchical navigation, as you have it in file browsers etc.
     */
    Relation.PARENT = 'isChildOf';
    /**
     * This relation spans a directed acyclical graph between resources. Note that
     * there should not be any redundant paths. That means, if A -> B -> C, there should
     * not be an explicit relation between A and C. Instead the relation is implicit as per the
     * path from A to C.
     *
     * It can be used to model relationships where one entity belongs to multiple other entities in some
     * way, as it is the case for example with Doors and Room, where a Door can be part of two Rooms.
     */
    Relation.IS_PRESENT_IN = 'isPresentIn';
    let Hierarchy;
    (function (Hierarchy) {
        Hierarchy.RECORDEDIN = 'isRecordedIn';
        Hierarchy.LIESWITHIN = 'liesWithin';
        Hierarchy.ALL = [Hierarchy.LIESWITHIN, Hierarchy.RECORDEDIN];
    })(Hierarchy = Relation.Hierarchy || (Relation.Hierarchy = {}));
    /**
     * Used to signal sameness (a claim of identity) in a generic manner, for example in order to say
     * two resources, recorded in different operations, are the same.
     *
     * This relation complements isChildOf in so far as the tree spanned by the latter gains the
     * possibility to express sameness, while retaining its relatively simple structure
     * (more complex graphs; see also: isPresentIn). Sameness in this way can be seen as an
     * *interpretation* that parts of an entity, each found in its own (spatial) context and also maybe
     * described by different persons, actually *are* (rather than are parts of) the *same* entity.
     * For example one could have two parts of a wall which are not (visibly) physically connected,
     * but which are taken as having been the *same* wall.
     */
    Relation.SAME_AS = 'isSameAs';
    Relation.UNIDIRECTIONAL = Hierarchy.ALL.concat([Relation.IS_PRESENT_IN]);
    /**
     * Used to connect finds with type resources
     */
    let Type;
    (function (Type) {
        Type.INSTANCEOF = 'isInstanceOf';
        Type.HASINSTANCE = 'hasInstance';
        Type.ALL = [Type.INSTANCEOF, Type.HASINSTANCE];
    })(Type = Relation.Type || (Relation.Type = {}));
    let Inventory;
    (function (Inventory) {
        Inventory.ISSTORAGEPLACEOF = 'isStoragePlaceOf';
        Inventory.ISSTOREDIN = 'isStoredIn';
        Inventory.ALL = [Inventory.ISSTORAGEPLACEOF, Inventory.ISSTOREDIN];
    })(Inventory = Relation.Inventory || (Relation.Inventory = {}));
    let Image;
    (function (Image) {
        Image.DEPICTS = 'depicts';
        Image.ISDEPICTEDIN = 'isDepictedIn';
        Image.HASMAPLAYER = 'hasMapLayer';
        Image.ISMAPLAYEROF = 'isMapLayerOf';
        Image.HASDEFAULTMAPLAYER = 'hasDefaultMapLayer';
        Image.ALL = [Image.DEPICTS, Image.ISDEPICTEDIN, Image.HASMAPLAYER, Image.ISMAPLAYEROF, Image.HASDEFAULTMAPLAYER];
    })(Image = Relation.Image || (Relation.Image = {}));
    /**
     * @returns {boolean} True if the given domain category is a valid domain name for a relation definition
     * which has the given range category & name
     */
    function isAllowedRelationDomainCategory(relations, domainCategory, rangeCategory, relation) {
        const relationDefinitions = getRelations(relations, rangeCategory, true);
        for (let relationDefinition of relationDefinitions) {
            if (relation === relationDefinition.name
                && relationDefinition.domain.indexOf(domainCategory) > -1)
                return true;
        }
        return false;
    }
    Relation.isAllowedRelationDomainCategory = isAllowedRelationDomainCategory;
    function getRelations(relations, categoryName, isRangeCategory = false) {
        const availableRelationFields = [];
        for (let relationField of relations) {
            const categories = isRangeCategory ? relationField.range : relationField.domain;
            if (categories.indexOf(categoryName) > -1)
                availableRelationFields.push(relationField);
        }
        return availableRelationFields;
    }
    Relation.getRelations = getRelations;
    function makeInverseRelationsMap(relationDefinitions) {
        return tsfun_1.flow(relationDefinitions, assoc_reduce_1.assocReduce(utils_1.toPair('name', 'inverse'), {}));
    }
    Relation.makeInverseRelationsMap = makeInverseRelationsMap;
})(Relation = exports.Relation || (exports.Relation = {}));
//# sourceMappingURL=relation.js.map