"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Dating = void 0;
const tsfun_1 = require("tsfun");
var Dating;
(function (Dating) {
    Dating.TYPE = 'type';
    Dating.BEGIN = 'begin';
    Dating.END = 'end';
    Dating.MARGIN = 'margin';
    Dating.SOURCE = 'source';
    Dating.IS_IMPRECISE = 'isImprecise';
    Dating.IS_UNCERTAIN = 'isUncertain';
    Dating.LABEL = 'label';
    Dating.YEAR = 'year';
    Dating.INPUT_YEAR = 'inputYear';
    Dating.INPUT_TYPE = 'inputType';
    const VALID_FIELDS = [Dating.TYPE, Dating.BEGIN, Dating.END, Dating.MARGIN, Dating.SOURCE, Dating.IS_IMPRECISE, Dating.IS_UNCERTAIN, Dating.LABEL];
    const VALID_ELEMENT_FIELDS = [Dating.YEAR, Dating.INPUT_YEAR, Dating.INPUT_TYPE];
    Dating.VALID_DATING_TYPES = ['bce', 'ce', 'bp'];
    Dating.VALID_TYPES = ['range', 'single', 'before', 'after', 'scientific'];
    function isDating(dating) {
        if (!tsfun_1.isObject(dating))
            return false;
        for (const fieldName in dating) {
            if (!VALID_FIELDS.includes(fieldName))
                return false;
        }
        if (dating.begin)
            for (const fieldName in dating.begin) {
                if (!VALID_ELEMENT_FIELDS.includes(fieldName))
                    return false;
            }
        if (dating.end)
            for (const fieldName in dating.end) {
                if (!VALID_ELEMENT_FIELDS.includes(fieldName))
                    return false;
            }
        if (dating.label)
            return true; // Support datings in deprecated format
        if (!dating.type || !['range', 'single', 'after', 'before', 'scientific'].includes(dating.type)) {
            return false;
        }
        return true;
    }
    Dating.isDating = isDating;
    function isValid(dating) {
        if (dating.label)
            return true; // Support datings in deprecated format
        if (['range', 'after', 'scientific'].includes(dating.type) && !dating.begin)
            return false;
        if (['range', 'single', 'before', 'scientific'].includes(dating.type) && !dating.end)
            return false;
        if (dating.type === 'scientific' && !dating.margin)
            return false;
        if (dating.begin && (!dating.begin.inputYear || !dating.begin.inputType
            || !Number.isInteger(dating.begin.inputYear)
            || dating.begin.inputYear < 0))
            return false;
        if (dating.end && (!dating.end.inputYear || !dating.end.inputType
            || !Number.isInteger(dating.end.inputYear)
            || dating.end.inputYear < 0))
            return false;
        return dating.type !== 'range' || validateRangeDating(dating);
    }
    Dating.isValid = isValid;
    function validateRangeDating(dating) {
        return dating.begin !== undefined && dating.end !== undefined &&
            Dating.getNormalizedYear(dating.begin.inputYear, dating.begin.inputType)
                < Dating.getNormalizedYear(dating.end.inputYear, dating.end.inputType);
    }
    const dissocIfEmpty = (path) => tsfun_1.cond(tsfun_1.on(path, tsfun_1.isUndefinedOrEmpty), tsfun_1.detach(path));
    function revert(dating) {
        return tsfun_1.flow(dating, tsfun_1.detach(['begin', 'year']), tsfun_1.detach(['end', 'year']), dissocIfEmpty('begin'), dissocIfEmpty('end'));
    }
    Dating.revert = revert;
    function addNormalizedValues(dating) {
        setNormalizedYears(dating);
        if (dating.type === 'scientific')
            applyMargin(dating);
    }
    Dating.addNormalizedValues = addNormalizedValues;
    function generateLabel(dating, getTranslation, getFromI18NString) {
        if (isValid(dating)) {
            let prefix = '';
            let year = '';
            let postfix = '';
            if (dating.type === 'range') {
                year = generateLabelForDate(dating.begin, getTranslation) + ' – '
                    + generateLabelForDate(dating.end, getTranslation);
            }
            if (dating.type === 'before' || dating.type == 'single') {
                year = generateLabelForDate(dating.end, getTranslation);
            }
            if (dating.type === 'after')
                year = generateLabelForDate(dating.begin, getTranslation);
            if (dating.type === 'scientific') {
                year = generateLabelForDate(dating.end, getTranslation);
                if (dating.margin && dating.margin > 0)
                    year += ' ± ' + dating.margin;
            }
            if (dating.isImprecise)
                prefix = 'ca. ';
            if (dating.isUncertain)
                postfix = ' (?)';
            if (dating.type === 'before')
                prefix = getTranslation('before') + ' ' + prefix;
            if (dating.type === 'after')
                prefix = getTranslation('after') + ' ' + prefix;
            if (dating.source)
                postfix += ' [' + getFromI18NString(dating.source) + ']';
            return prefix + year + postfix;
        }
        else {
            return JSON.stringify(dating);
        }
    }
    Dating.generateLabel = generateLabel;
    function generateLabelForDate(date, getTranslation) {
        if (!date) {
            return '';
        }
        else if (date.inputYear === 0) {
            return '0';
        }
        else {
            return date.inputYear + ' ' + getTranslation(date.inputType);
        }
    }
    function setNormalizedYears(dating) {
        if (dating.begin && dating.begin.inputYear && dating.begin.inputType) {
            dating.begin.year = getNormalizedYear(dating.begin.inputYear, dating.begin.inputType);
        }
        if (dating.end && dating.end.inputYear && dating.end.inputType) {
            dating.end.year = getNormalizedYear(dating.end.inputYear, dating.end.inputType);
        }
    }
    Dating.setNormalizedYears = setNormalizedYears;
    function getNormalizedYear(inputYear, inputType) {
        if (inputType === 'bce')
            return 0 - inputYear;
        if (inputType === 'bp')
            return 1950 - inputYear;
        return inputYear;
    }
    Dating.getNormalizedYear = getNormalizedYear;
    function applyMargin(dating) {
        if (!dating.begin || !dating.end || !dating.margin)
            return;
        dating.begin.inputYear = dating.end.inputYear;
        dating.begin.year = dating.begin.year - dating.margin;
        dating.end.year = dating.end.year + dating.margin;
    }
})(Dating = exports.Dating || (exports.Dating = {}));
//# sourceMappingURL=dating.js.map