"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Dimension = void 0;
const tsfun_1 = require("tsfun");
var Dimension;
(function (Dimension) {
    Dimension.VALUE = 'value';
    Dimension.LABEL = 'label';
    Dimension.ISRANGE = 'isRange';
    Dimension.RANGEMIN = 'rangeMin';
    Dimension.RANGEMAX = 'rangeMax';
    Dimension.INPUTVALUE = 'inputValue';
    Dimension.INPUTRANGEENDVALUE = 'inputRangeEndValue';
    Dimension.INPUTUNIT = 'inputUnit';
    Dimension.MEASUREMENTPOSITION = 'measurementPosition';
    Dimension.MEASUREMENTCOMMENT = 'measurementComment';
    Dimension.ISIMPRECISE = 'isImprecise';
    const VALID_FIELDS = [Dimension.VALUE, Dimension.LABEL, Dimension.ISRANGE, Dimension.RANGEMIN, Dimension.RANGEMAX,
        Dimension.INPUTVALUE, Dimension.INPUTRANGEENDVALUE, Dimension.INPUTUNIT, Dimension.MEASUREMENTPOSITION, Dimension.MEASUREMENTCOMMENT, Dimension.ISIMPRECISE];
    Dimension.VALID_INPUT_UNITS = ['mm', 'cm', 'm'];
    function isDimension(dimension) {
        if (!tsfun_1.isObject(dimension))
            return false;
        for (const fieldName in dimension) {
            if (!VALID_FIELDS.includes(fieldName))
                return false;
        }
        if (dimension.measurementPosition && !tsfun_1.isString(dimension.measurementPosition))
            return false;
        if (dimension.measurementComment && !tsfun_1.isObject(dimension.measurementComment)
            && !tsfun_1.isString(dimension.measurementComment)) {
            return false;
        }
        if (dimension.label)
            return true; // Support dimensions in deprecated format
        if (!Dimension.VALID_INPUT_UNITS.includes(dimension.inputUnit))
            return false;
        if (!tsfun_1.isNumber(dimension.inputValue))
            return false;
        return true;
    }
    Dimension.isDimension = isDimension;
    /**
     * Note: There still may be dimension['isRange'] boolean values stored in the database.
     * These should be handled properly if we're up to changing things here, for example if
     * we want to make sure only defined fields are present.
     *
     * @param dimension
     * @param options
     */
    function isValid(dimension, options) {
        if (dimension.label)
            return true; // Support dimensions in deprecated format
        if (!dimension.inputValue || !dimension.inputUnit)
            return false;
        if (dimension.inputRangeEndValue !== undefined) {
            if (!tsfun_1.isNumber(dimension.inputRangeEndValue))
                return false;
            if (dimension.inputRangeEndValue === dimension.inputValue)
                return false;
        }
        if (!(options === null || options === void 0 ? void 0 : options.permissive)) {
            if (dimension.inputValue < 0)
                return false;
            if (dimension.inputRangeEndValue !== undefined) {
                if (dimension.inputRangeEndValue < 0)
                    return false;
            }
        }
        return true;
    }
    Dimension.isValid = isValid;
    /**
     * Reverts the dimension back to the state before normalization
     * @param dimension gets modified in place
     */
    function revert(dimension) {
        return tsfun_1.flow(dimension, tsfun_1.detach('value'), tsfun_1.detach('rangeMin'), tsfun_1.detach('rangeMax'), tsfun_1.detach('isRange'));
    }
    Dimension.revert = revert;
    function addNormalizedValues(dimension) {
        if (dimension.inputRangeEndValue !== undefined) {
            dimension.rangeMin = convertValueFromInputUnitToMicrometre(dimension.inputUnit, dimension.inputValue);
            dimension.rangeMax = convertValueFromInputUnitToMicrometre(dimension.inputUnit, dimension.inputRangeEndValue);
            delete (dimension.value);
        }
        else {
            dimension.value = convertValueFromInputUnitToMicrometre(dimension.inputUnit, dimension.inputValue);
            delete (dimension.rangeMin);
            delete (dimension.rangeMax);
        }
    }
    Dimension.addNormalizedValues = addNormalizedValues;
    function generateLabel(dimension, transform, translate, getFromI18NString, measurementPositionLabel) {
        if (isValid(dimension)) {
            let label = (dimension.isImprecise ? 'ca. ' : '');
            if (dimension.inputRangeEndValue !== undefined) {
                label += transform(dimension.inputValue) + '-'
                    + transform(dimension.inputRangeEndValue);
            }
            else {
                label += transform(dimension.inputValue);
            }
            label += ' ' + dimension.inputUnit;
            if (dimension.measurementPosition) {
                label += ', ' + translate('asMeasuredBy') + ' '
                    + (measurementPositionLabel
                        ? measurementPositionLabel
                        : dimension.measurementPosition);
            }
            if (dimension.measurementComment)
                label += ' (' + getFromI18NString(dimension.measurementComment) + ')';
            return label;
        }
        else {
            return JSON.stringify(dimension);
        }
    }
    Dimension.generateLabel = generateLabel;
    function convertValueFromInputUnitToMicrometre(inputUnit, inputValue) {
        switch (inputUnit) {
            case 'mm':
                return Math.round(inputValue * 1000);
            case 'cm':
                return Math.round(inputValue * 10000);
            case 'm':
                return Math.round(inputValue * 1000000);
            default:
                return inputValue;
        }
    }
})(Dimension = exports.Dimension || (exports.Dimension = {}));
//# sourceMappingURL=dimension.js.map