"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Resource = void 0;
const tsfun_1 = require("tsfun");
const compare_1 = require("../tools/compare");
const sort_util_1 = require("../tools/sort-util");
const utils_1 = require("../tools/utils");
const category_form_1 = require("./configuration/category-form");
/**
 * @author Thomas Kleinke
 * @author Daniel de Oliveira
 */
var Resource;
(function (Resource) {
    Resource.ID = 'id';
    Resource.CATEGORY = 'category';
    Resource.IDENTIFIER = 'identifier';
    Resource.RELATIONS = 'relations';
    Resource.CONSTANT_FIELDS = [Resource.ID, Resource.CATEGORY, Resource.IDENTIFIER, Resource.RELATIONS];
    function getDifferingFields(resource1, resource2) {
        const differingFieldsNames = findDifferingFieldsInResource(resource1, resource2)
            .concat(findDifferingFieldsInResource(resource2, resource1));
        return tsfun_1.set(differingFieldsNames);
    }
    Resource.getDifferingFields = getDifferingFields;
    function getDifferingRelations(resource1, resource2) {
        const differingRelationNames = findDifferingFieldsInRelations(resource1.relations, resource2.relations)
            .concat(findDifferingFieldsInRelations(resource2.relations, resource1.relations));
        return tsfun_1.set(differingRelationNames);
    }
    Resource.getDifferingRelations = getDifferingRelations;
    function hasRelations(resource, relation) {
        return resource.relations[relation] && resource.relations[relation].length > 0;
    }
    Resource.hasRelations = hasRelations;
    function hasRelationTarget(resource, relation, target) {
        if (!resource.relations[relation])
            return false;
        return resource.relations[relation].indexOf(target) > -1;
    }
    Resource.hasRelationTarget = hasRelationTarget;
    function getRelationTargets(resource, allowedRelations) {
        const ownKeys = Object.keys(resource.relations)
            .filter(prop => resource.relations.hasOwnProperty(prop));
        const usableRelations = allowedRelations
            ? ownKeys.filter(tsfun_1.includedIn(allowedRelations))
            : ownKeys;
        return tsfun_1.flatMap(usableRelations, (prop) => resource.relations[prop]);
    }
    Resource.getRelationTargets = getRelationTargets;
    function getRelationTargetDocuments(resource, datastore) {
        return __awaiter(this, void 0, void 0, function* () {
            const targets = {};
            for (let relationName of Object.keys(resource.relations)) {
                targets[relationName] = (yield datastore.getMultiple(resource.relations[relationName]))
                    .sort((target1, target2) => sort_util_1.SortUtil.alnumCompare(target1.resource.identifier, target2.resource.identifier));
            }
            return targets;
        });
    }
    Resource.getRelationTargetDocuments = getRelationTargetDocuments;
    function removeEmptyRelations(resource) {
        Object.keys(resource.relations)
            .filter(key => resource.relations[key] === null || resource.relations[key].length === 0)
            .forEach(key => delete resource.relations[key]);
    }
    Resource.removeEmptyRelations = removeEmptyRelations;
    function findDifferingFieldsInRelations(relations1, relations2) {
        return Object.keys(relations1)
            .reduce(utils_1.concatIf(compare_1.notBothEqual(relations1, relations2)), []);
    }
    Resource.relationsEquivalent = (r1) => (r2) => tsfun_1.samemap(tsfun_1.sameset, tsfun_1.remove(tsfun_1.isUndefinedOrEmpty, r1.relations), tsfun_1.remove(tsfun_1.isUndefinedOrEmpty, r2.relations));
    function findDifferingFieldsInResource(resource1, resource2) {
        return Object.keys(resource1)
            .filter(tsfun_1.isnt(Resource.RELATIONS))
            .reduce(utils_1.concatIf(compare_1.notCompareInBoth(resource1, resource2)), []);
    }
    Resource.getShortDescriptionLabel = (resource, labels, projectConfiguration) => {
        const shortDescription = resource.shortDescription;
        if (!shortDescription)
            return undefined;
        return tsfun_1.isString(shortDescription)
            ? getShortDescriptionLabelFromString(resource, shortDescription, labels, projectConfiguration)
            : labels.getFromI18NString(shortDescription);
    };
    const getShortDescriptionLabelFromString = (resource, shortDescription, labels, projectConfiguration) => {
        const category = projectConfiguration.getCategory(resource.category);
        if (!category)
            return shortDescription;
        const valuelist = category_form_1.CategoryForm.getShortDescriptionValuelist(category);
        return valuelist
            ? labels.getValueLabel(valuelist, shortDescription)
            : shortDescription;
    };
})(Resource = exports.Resource || (exports.Resource = {}));
//# sourceMappingURL=resource.js.map