"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProjectConfiguration = void 0;
const tsfun_1 = require("tsfun");
const tools_1 = require("../tools");
const configuration_errors_1 = require("../configuration/boot/configuration-errors");
const category_form_1 = require("../model/configuration/category-form");
const relation_1 = require("../model/configuration/relation");
const field_1 = require("../model/configuration/field");
;
const TYPE_CATALOG = 'TypeCatalog';
const TYPE = 'Type';
/**
 * For naming of document types (for example "ConcreteField"-XYZ, "Image"-XYZ)
 * see document.ts
 *
 * @author Thomas Kleinke
 * @author Daniel de Oliveira
 * @author Sebastian Cuy
 * @author F.Z.
 */
class ProjectConfiguration {
    constructor(rawConfiguration) {
        this.categoryForms = rawConfiguration.forms;
        this.relations = rawConfiguration.relations || [];
        this.projectLanguages = rawConfiguration.projectLanguages;
        this.categoriesMap = this.createCategoriesMap();
        this.updateFilteredCategories();
    }
    update(newProjectConfiguration) {
        this.categoryForms = newProjectConfiguration.categoryForms;
        this.relations = newProjectConfiguration.relations;
        this.projectLanguages = newProjectConfiguration.projectLanguages;
        this.categoriesMap = this.createCategoriesMap();
        this.updateFilteredCategories();
    }
    getProjectLanguages() {
        return this.projectLanguages;
    }
    getCategory(arg) {
        if (arg === undefined)
            return undefined;
        const name = tsfun_1.isString(arg)
            ? arg
            : arg.resource.category;
        return this.categoriesMap[name];
    }
    getCategories(...selectedTopLevelCategories) {
        return selectedTopLevelCategories.length === 0
            ? this.categoryForms
            : this.categoryForms.filter(tsfun_1.on(tools_1.Tree.ITEMNAMEPATH, tsfun_1.includedIn(selectedTopLevelCategories)));
    }
    getHierarchyParentCategories(categoryName) {
        return this.getAllowedRelationRangeCategories('isRecordedIn', categoryName)
            .concat(this.getAllowedRelationRangeCategories('liesWithin', categoryName));
    }
    isSubcategory(category, superCategoryName) {
        if (!this.getCategory(category))
            throw [configuration_errors_1.ConfigurationErrors.UNKNOWN_CATEGORY_ERROR, category];
        return tools_1.isTopLevelItemOrChildThereof(this.categoryForms, category, superCategoryName);
    }
    isGeometryCategory(category) {
        return category_form_1.CategoryForm.getFields(this.getCategory(category))
            .map(tsfun_1.to(field_1.Field.NAME))
            .includes('geometry');
    }
    getRegularCategories() {
        return this.regularCategories;
    }
    getFieldCategories() {
        return this.fieldCategories;
    }
    getOverviewCategories() {
        return this.overviewCategories;
    }
    getConcreteOverviewCategories() {
        return this.concreteOverviewCategories;
    }
    getOverviewSupercategories() {
        return this.overviewSupercategories;
    }
    getTypeManagementCategories() {
        return this.typeManagementCategories;
    }
    getTypeManagementSupercategories() {
        return this.typeManagementSupercategories;
    }
    getTypeCategories() {
        return this.typeCategories;
    }
    getCategoryWithSubcategories(categoryName) {
        var _a;
        return (_a = this.categoriesWithSubcategories[categoryName]) !== null && _a !== void 0 ? _a : [];
    }
    getInventoryCategories() {
        return this.getCategoryWithSubcategories('StoragePlace');
    }
    getInventorySupercategories() {
        return tsfun_1.flow(this.getInventoryCategories(), tsfun_1.filter(tools_1.Named.onName(tsfun_1.includedIn(['StoragePlace']))));
    }
    getImageCategories() {
        return this.getCategoryWithSubcategories('Image');
    }
    getFeatureCategories() {
        return this.getCategoryWithSubcategories('Feature');
    }
    getOperationCategories() {
        return this.getCategoryWithSubcategories('Operation');
    }
    getQrCodeCategories() {
        return tools_1.Tree.flatten(this.categoryForms)
            .filter(categoryForm => categoryForm.scanCodes !== undefined);
    }
    getRelations() {
        return this.relations;
    }
    getRelationsForDomainCategory(categoryName) {
        return relation_1.Relation.getRelations(this.relations, categoryName, false);
    }
    getRelationsForRangeCategory(categoryName) {
        return relation_1.Relation.getRelations(this.relations, categoryName, true);
    }
    isAllowedRelationDomainCategory(domainCategory, rangeCategory, relation) {
        return relation_1.Relation.isAllowedRelationDomainCategory(this.relations, domainCategory, rangeCategory, relation);
    }
    getAllowedRelationDomainCategories(relationName, rangeCategoryName) {
        return tools_1.Tree.flatten(this.categoryForms)
            .filter(category => {
            return relation_1.Relation.isAllowedRelationDomainCategory(this.relations, category.name, rangeCategoryName, relationName) && (!category.parentCategory || !relation_1.Relation.isAllowedRelationDomainCategory(this.relations, category.parentCategory.name, rangeCategoryName, relationName));
        });
    }
    getAllowedRelationRangeCategories(relationName, domainCategoryName) {
        return tools_1.Tree.flatten(this.categoryForms)
            .filter(category => {
            return relation_1.Relation.isAllowedRelationDomainCategory(this.relations, domainCategoryName, category.name, relationName) && (!category.parentCategory || !relation_1.Relation.isAllowedRelationDomainCategory(this.relations, domainCategoryName, category.parentCategory.name, relationName));
        });
    }
    createCategoriesMap() {
        return tools_1.Tree.flatten(this.categoryForms)
            .reduce((result, category) => {
            result[category.name] = category;
            return result;
        }, {});
    }
    updateFilteredCategories() {
        this.regularCategories = this.filterRegularCategories();
        this.fieldCategories = this.filterFieldCategories();
        this.overviewCategories = this.filterOverviewCategories();
        this.concreteOverviewCategories = this.filterConcreteOverviewCategories();
        this.overviewSupercategories = this.filterOverviewSupercategories();
        this.typeManagementCategories = this.filterTypeManagementCategories();
        this.typeManagementSupercategories = this.filterTypeManagementSupercategories();
        this.typeCategories = this.filterTypeCategories();
        this.categoriesWithSubcategories = this.filterCategoriesWithSubcategories();
    }
    filterRegularCategories() {
        return tsfun_1.flow(this.categoryForms, tools_1.removeTrees('Place', 'Project', TYPE_CATALOG, TYPE, 'StoragePlace', 'Image', 'Operation'), tools_1.Tree.flatten);
    }
    filterFieldCategories() {
        return tsfun_1.flow(this.categoryForms, tools_1.removeTrees('Image', 'Project', TYPE_CATALOG, TYPE, 'StoragePlace'), tools_1.Tree.flatten);
    }
    filterOverviewCategories() {
        return tsfun_1.flow(this.categoryForms, tools_1.filterTrees('Operation', 'Place'), tools_1.Tree.flatten);
    }
    filterConcreteOverviewCategories() {
        return tsfun_1.flow(this.categoryForms, tools_1.filterTrees('Operation', 'Place'), tools_1.Tree.flatten, tsfun_1.remove(tools_1.Named.onName(tsfun_1.is('Operation'))));
    }
    filterOverviewSupercategories() {
        return tsfun_1.flow(this.categoryForms, tools_1.filterTrees('Operation', 'Place'), tools_1.Tree.flatten, tsfun_1.filter(tools_1.Named.onName(tsfun_1.includedIn(['Operation', 'Place']))));
    }
    filterTypeManagementCategories() {
        return tsfun_1.flow(this.categoryForms, tools_1.filterTrees(TYPE, TYPE_CATALOG), tools_1.Tree.flatten);
    }
    filterTypeManagementSupercategories() {
        return tsfun_1.flow(this.filterTypeManagementCategories(), tsfun_1.filter(tools_1.Named.onName(tsfun_1.includedIn([TYPE, TYPE_CATALOG]))));
    }
    filterTypeCategories() {
        return tsfun_1.flow(this.categoryForms, tools_1.filterTrees(TYPE), tools_1.Tree.flatten);
    }
    filterCategoriesWithSubcategories() {
        return tools_1.Tree.flatten(this.categoryForms)
            .filter(category => !category.parentCategory)
            .reduce((result, category) => {
            result[category.name] = this.filterCategoryWithSubcategories(category.name);
            return result;
        }, {});
    }
    filterCategoryWithSubcategories(categoryName) {
        return tsfun_1.flow(this.categoryForms, tools_1.filterTrees(categoryName), tools_1.Tree.flatten);
    }
}
exports.ProjectConfiguration = ProjectConfiguration;
//# sourceMappingURL=project-configuration.js.map