import { Document } from '../model/document';
import { Datastore } from '../datastore/datastore';
import { NewDocument } from '../model/document';
import { ProjectConfiguration } from './project-configuration';
/**
 * Maintains a consistent state of relations between the
 * documents by updating related documents relations during update or delete operations.
 *
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
export declare class RelationsManager {
    private datastore;
    private projectConfiguration;
    constructor(datastore: Datastore, projectConfiguration: ProjectConfiguration);
    /**
     * Persists document and all the objects that are or have been in relation
     * with the object before the method call.
     *
     * If the document is { resource: { id: 1, relations: { includes: [2] } } },
     * this means that also another document is updated, namely { resource: { id: 2 } } }.
     * which gets updated to { resource: { id: 2, relations: { belongsTo: [1] } } }.
     *
     * On top of that, one oldVersion and some revisionsToSquash can be specified.
     * These are compared with document to determine which relations have been removed.
     *
     * @param document an existing or a new document
     * @param oldVersion to be used only if document is an existing document.
     * @param revisionsToSquash these revisions get deleted while updating document
     * @returns a copy of the updated document
     * @throws msgWithParams
     */
    update(document: NewDocument | Document, oldVersion?: Document, // side effect: modified in place. for reasons of how the document cache behaves
    revisionsToSquash?: Document[]): Promise<Document>;
    /**
     * Removes the document from the datastore.
     *
     * Also removes all descendant documents (documents with an 'isRecordedIn' or 'liesWithin' relation pointing to
     * this document.
     * Deletes all corresponding inverse relations.
     *
     * @throws
     *   [DatastoreErrors.DOCUMENT_NO_RESOURCE_ID] - if document has no resource id
     *   [DatastoreErrors.DOCUMENT_DOES_NOT_EXIST_ERROR] - if document has a resource id, but does not exist in the db
     *   [DatastoreErrors.GENERIC_DELETE_ERROR] - if cannot delete for another reason
     */
    remove(document: Document, options?: {
        descendants?: true;
        descendantsToKeep?: Array<Document>;
    }): Promise<void>;
    private updateWithConnections;
    private removeWithConnectedDocuments;
    private fixIsRecordedInInLiesWithinDocs;
    private persistIt;
    private findLiesWithinDocs;
    private getInverseRelationsMap;
    private getRelationNames;
}
