"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RelationsManager = void 0;
const tsfun_1 = require("tsfun");
const document_1 = require("../model/document");
const relation_1 = require("../model/configuration/relation");
const connected_docs_1 = require("./utilities/connected-docs");
const constants_1 = require("../constants");
var RECORDED_IN = relation_1.Relation.Hierarchy.RECORDEDIN;
const basic_index_configuration_1 = require("../basic-index-configuration");
const named_1 = require("../tools/named");
/**
 * Maintains a consistent state of relations between the
 * documents by updating related documents relations during update or delete operations.
 *
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
class RelationsManager {
    constructor(datastore, projectConfiguration) {
        this.datastore = datastore;
        this.projectConfiguration = projectConfiguration;
    }
    /**
     * Persists document and all the objects that are or have been in relation
     * with the object before the method call.
     *
     * If the document is { resource: { id: 1, relations: { includes: [2] } } },
     * this means that also another document is updated, namely { resource: { id: 2 } } }.
     * which gets updated to { resource: { id: 2, relations: { belongsTo: [1] } } }.
     *
     * On top of that, one oldVersion and some revisionsToSquash can be specified.
     * These are compared with document to determine which relations have been removed.
     *
     * @param document an existing or a new document
     * @param oldVersion to be used only if document is an existing document.
     * @param revisionsToSquash these revisions get deleted while updating document
     * @returns a copy of the updated document
     * @throws msgWithParams
     */
    update(document, oldVersion = document, // side effect: modified in place. for reasons of how the document cache behaves
    revisionsToSquash = []) {
        return __awaiter(this, void 0, void 0, function* () {
            const persistedDocument = yield this.updateWithConnections(document, oldVersion, revisionsToSquash);
            yield this.fixIsRecordedInInLiesWithinDocs(persistedDocument);
            return persistedDocument;
        });
    }
    /**
     * Removes the document from the datastore.
     *
     * Also removes all descendant documents (documents with an 'isRecordedIn' or 'liesWithin' relation pointing to
     * this document.
     * Deletes all corresponding inverse relations.
     *
     * @throws
     *   [DatastoreErrors.DOCUMENT_NO_RESOURCE_ID] - if document has no resource id
     *   [DatastoreErrors.DOCUMENT_DOES_NOT_EXIST_ERROR] - if document has a resource id, but does not exist in the db
     *   [DatastoreErrors.GENERIC_DELETE_ERROR] - if cannot delete for another reason
     */
    remove(document, options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if ((options === null || options === void 0 ? void 0 : options.descendants) !== true) {
                if ((options === null || options === void 0 ? void 0 : options.descendantsToKeep) !== undefined)
                    throw 'illegal arguments - relationsManager.remove called with descendantsToKeep but descendants option not set';
                yield this.removeWithConnectedDocuments(document);
                return;
            }
            const descendants = (yield this.datastore.find(basic_index_configuration_1.childrenOf(document.resource.id))).documents;
            const documentsToBeDeleted = tsfun_1.flow(descendants, tsfun_1.subtract(constants_1.ON_RESOURCE_ID, (_a = options.descendantsToKeep) !== null && _a !== void 0 ? _a : []), tsfun_1.append(document));
            for (let document of documentsToBeDeleted)
                yield this.removeWithConnectedDocuments(document);
        });
    }
    updateWithConnections(document, oldVersion, revisionsToSquash) {
        return __awaiter(this, void 0, void 0, function* () {
            const revs = revisionsToSquash.map(_ => _._rev).filter(tsfun_1.isDefined);
            const updated = yield this.persistIt(document, revs);
            yield connected_docs_1.ConnectedDocs.updateForUpdate(this.datastore, this.getRelationNames(), this.getInverseRelationsMap(), updated, [oldVersion].concat(revisionsToSquash));
            return updated;
        });
    }
    removeWithConnectedDocuments(document) {
        return __awaiter(this, void 0, void 0, function* () {
            yield connected_docs_1.ConnectedDocs.updateForRemove(this.datastore, this.getRelationNames(), this.getInverseRelationsMap(), document);
            yield this.datastore.remove(document);
        });
    }
    fixIsRecordedInInLiesWithinDocs(document) {
        return __awaiter(this, void 0, void 0, function* () {
            if (tsfun_1.isUndefinedOrEmpty(document.resource.relations[RECORDED_IN]))
                return;
            const findResult = yield this.findLiesWithinDocs(document.resource.id);
            const docsToCorrect = findResult
                .documents
                .filter(tsfun_1.on(['resource', 'relations', RECORDED_IN], tsfun_1.isArray))
                .filter(tsfun_1.isNot(tsfun_1.on(['resource', 'relations', RECORDED_IN], tsfun_1.sameset)(document)));
            for (let docToCorrect of docsToCorrect) {
                const cloned = document_1.Document.clone(docToCorrect);
                cloned.resource.relations[RECORDED_IN] = document.resource.relations[RECORDED_IN];
                yield this.datastore.update(cloned, undefined);
            }
        });
    }
    persistIt(document, squashRevisionIds) {
        return document.resource.id
            ? this.datastore.update(document, squashRevisionIds.length === 0 ? undefined : squashRevisionIds) : this.datastore.create(document);
    }
    findLiesWithinDocs(resourceId) {
        return __awaiter(this, void 0, void 0, function* () {
            const query = {
                constraints: {
                    'isChildOf:contain': {
                        value: resourceId,
                        searchRecursively: true
                    }
                }
            };
            return this.datastore.find(query, { includeResourcesWithoutValidParent: true });
        });
    }
    getInverseRelationsMap() {
        return relation_1.Relation.makeInverseRelationsMap(this.projectConfiguration.getRelations());
    }
    getRelationNames() {
        return this.projectConfiguration.getRelations().map(named_1.Named.toName);
    }
}
exports.RelationsManager = RelationsManager;
//# sourceMappingURL=relations-manager.js.map