"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConnectedDocs = void 0;
const tsfun_1 = require("tsfun");
const document_1 = require("../../model/document");
const resource_1 = require("../../model/resource");
const update_relations_1 = require("./update-relations");
/**
 * Architecture note: This class deals with automatic
 * update of documents directly connected
 * to a document via relations.
 *
 * Other operations, like correcting documents' isRecordedIn relations
 * or hierarchical deletions is done in relations manager.
 *
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
var ConnectedDocs;
(function (ConnectedDocs) {
    function updateForUpdate(datastore, relationNames, inverseRelationsMap, document, otherVersions) {
        return __awaiter(this, void 0, void 0, function* () {
            const connectedDocs = yield getExistingConnectedDocs(datastore, relationNames, [document].concat(otherVersions));
            const docsToUpdate = update_relations_1.updateRelations(document, connectedDocs, inverseRelationsMap, true);
            for (const doc of docsToUpdate) {
                yield datastore.convert(doc);
            }
            yield updateDocs(datastore, docsToUpdate);
        });
    }
    ConnectedDocs.updateForUpdate = updateForUpdate;
    function updateForRemove(datastore, relationNames, inverseRelationsMap, document) {
        return __awaiter(this, void 0, void 0, function* () {
            const connectedDocs = yield getExistingConnectedDocsForRemove(datastore, relationNames, document);
            const docsToUpdate = update_relations_1.updateRelations(document, connectedDocs, inverseRelationsMap, false);
            yield updateDocs(datastore, docsToUpdate);
        });
    }
    ConnectedDocs.updateForRemove = updateForRemove;
    function updateDocs(datastore, docsToUpdate) {
        return __awaiter(this, void 0, void 0, function* () {
            // Note that this does not update a document for being target of isRecordedIn
            for (let docToUpdate of docsToUpdate) {
                yield datastore.update(docToUpdate, undefined);
            }
        });
    }
    function getExistingConnectedDocs(datastore, relationNames, documents) {
        return __awaiter(this, void 0, void 0, function* () {
            const uniqueConnectedDocIds = getUniqueConnectedDocumentsIds(documents, relationNames, datastore);
            return getDocumentsForIds(datastore, uniqueConnectedDocIds, id => {
                console.warn('connected document not found', id);
            });
        });
    }
    function getExistingConnectedDocsForRemove(datastore, relationNames, document) {
        return __awaiter(this, void 0, void 0, function* () {
            const uniqueConnectedDocIds = getUniqueConnectedDocumentsIds([document], relationNames, datastore);
            const liesWithinTargets = resource_1.Resource.getRelationTargets(document.resource, ['liesWithin']);
            const recordedInTargets = resource_1.Resource.getRelationTargets(document.resource, ['isRecordedIn']);
            return getDocumentsForIds(datastore, uniqueConnectedDocIds, id => {
                if (liesWithinTargets.includes(id) || recordedInTargets.includes(id)) {
                    // this can happen due to deletion order during deletion with descendants
                }
                else {
                    console.warn('connected document not found', id);
                }
            });
        });
    }
    function getDocumentsForIds(datastore, ids, handleError) {
        return __awaiter(this, void 0, void 0, function* () {
            const connectedDocuments = [];
            for (let id of ids) {
                try {
                    connectedDocuments.push(yield datastore.get(id));
                }
                catch (_a) {
                    handleError(id);
                }
            }
            return connectedDocuments;
        });
    }
    function getUniqueConnectedDocumentsIds(documents, allowedRelations, datastore) {
        const getAllRelationTargetsForDoc = (doc) => resource_1.Resource.getRelationTargets(doc.resource, allowedRelations)
            .concat(getPresentInDocumentIds(doc, datastore));
        return tsfun_1.flow(documents, tsfun_1.flatMap(getAllRelationTargetsForDoc), tsfun_1.subtract(documents.map(document_1.toResourceId)));
    }
    function getPresentInDocumentIds(document, datastore) {
        if (!document.resource.id)
            return [];
        return datastore.findIds({
            constraints: {
                'isPresentIn:contain': document.resource.id
            }
        }).ids;
    }
})(ConnectedDocs = exports.ConnectedDocs || (exports.ConnectedDocs = {}));
//# sourceMappingURL=connected-docs.js.map