"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateRelations = void 0;
const tsfun_1 = require("tsfun");
const document_1 = require("../../model/document");
const resource_1 = require("../../model/resource");
/**
 * Determines which targetDocuments need their relations updated, based
 * on the relations seen in <i>document</i> alone, and performs the updates.
 * Relations in targetDocuments, which correspond to other documents, are left as they are.
 *
 * @param document expected that relations is an object consisting only of proper relation names
 * @param targetDocuments
 * @param inverseRelationsMap
 * @param setInverses if false, relations of targetDocuments
 *   which point to document, get only removed, but not (re-)created
 *
 * @returns All targetDocuments that got an update in their relations.
 *   - Modified in place! -
 *
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
function updateRelations(document, targetDocuments, inverseRelationsMap, setInverses = true) {
    const cloneOfTargetDocuments = targetDocuments.map(document_1.Document.clone);
    const getInverse = tsfun_1.lookup(inverseRelationsMap);
    const hasInverseRelation = tsfun_1.compose(getInverse, tsfun_1.isDefined);
    for (let targetDocument of targetDocuments) {
        targetDocument.resource.relations =
            pruneInverseRelations(targetDocument.resource.relations, document.resource.id, setInverses, hasInverseRelation);
        if (setInverses) {
            setInverseRelations(targetDocument.resource, document.resource, getInverse, hasInverseRelation);
        }
    }
    return determineChangedDocs(targetDocuments, cloneOfTargetDocuments);
}
exports.updateRelations = updateRelations;
/**
 * { a: ['3'], b: ['3', '7'] }
 * resourceId: '3'
 * ->
 * { b: ['7'] }
 */
function pruneInverseRelations(relations, resourceId, setInverses, hasInverseRelation) {
    return tsfun_1.flow(tsfun_1.keys(relations), tsfun_1.filter(tsfun_1.cond(setInverses, hasInverseRelation, true)), tsfun_1.map(tsfun_1.pairWith(tsfun_1.lookup(relations))), tsfun_1.map(tsfun_1.update(1, tsfun_1.filter(tsfun_1.isnt(resourceId)))), tsfun_1.reduce((relations, [key, value]) => tsfun_1.assoc(key, value, relations), relations), tsfun_1.remove(tsfun_1.isEmpty));
}
function setInverseRelations(target, resource, getInverse, hasInverseRelation) {
    tsfun_1.flow(tsfun_1.keys(resource.relations), tsfun_1.filter(hasInverseRelation), tsfun_1.map(tsfun_1.pairWith(getInverse)), tsfun_1.values, tsfun_1.forEach(setInverseRelation(target, resource)));
}
/**
 * target.relations = {}
 * resource = { id: '1', relations { a: ['2'] } }
 * relation = 'a', inverse = 'a_inv'
 * ->
 * target.relations = { a: ['1'] }
 *
 * Modifies target in place!
 */
function setInverseRelation(target, resource) {
    return ([relation, inverse]) => {
        if (resource.relations[relation].includes(target.id)) {
            target.relations[inverse] =
                tsfun_1.flow(target.relations, tsfun_1.to(inverse, []), tsfun_1.filter(tsfun_1.isnt(resource.id)), tsfun_1.append(resource.id));
        }
    };
}
function determineChangedDocs(targetDocuments, cloneOfTargetDocuments) {
    return tsfun_1.zip(targetDocuments, cloneOfTargetDocuments).reduce(changedDocsReducer, []);
}
function changedDocsReducer(changedDocs, [targetDoc, cloneOfTargetDoc]) {
    return changedDocs.concat(!documentsRelationsEquivalent(targetDoc /* TODO review typings */)(cloneOfTargetDoc)
        ? targetDoc
        : []);
}
const documentsRelationsEquivalent = tsfun_1.on(document_1.Document.RESOURCE, resource_1.Resource.relationsEquivalent);
//# sourceMappingURL=update-relations.js.map