"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tsfun_1 = require("tsfun");
const build_raw_project_configuration_1 = require("../../../src/configuration/boot/build-raw-project-configuration");
const configuration_errors_1 = require("../../../src/configuration/boot/configuration-errors");
const field_1 = require("../../../src/model/configuration/field");
const group_1 = require("../../../src/model/configuration/group");
const forest_1 = require("../../../src/tools/forest");
const named_1 = require("../../../src/tools/named");
describe('buildRawProjectConfiguration', () => {
    function buildRawArray(builtInCategories, libraryCategories, libraryForms, ...rest) {
        const raw = build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, libraryCategories, libraryForms, ...rest);
        return forest_1.Tree.flatten(raw.forms);
    }
    function buildRaw(builtInCategories, libraryCategories, libraryForms, ...rest) {
        return named_1.Named.arrayToMap(buildRawArray(builtInCategories, libraryCategories, libraryForms, ...rest));
    }
    it('throw away unselected forms', () => {
        const builtInCategories = {
            A: {
                supercategory: true,
                userDefinedSubcategoriesAllowed: true,
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
            C: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                description: {},
                valuelists: {},
                createdBy: '',
                creationDate: '',
                groups: []
            }
        };
        const customForms = {
            A: {
                fields: {},
                hidden: []
            },
            B: {
                parent: 'A',
                fields: {},
                hidden: []
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms);
        expect(result['A']).toBeDefined();
        expect(result['B']).toBeDefined();
        expect(result['C']).toBeUndefined();
        expect(result['A:default']).toBeUndefined();
    });
    it('do not throw away any forms if no custom forms are provided', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
            B: {
                supercategory: true,
                userDefinedSubcategoriesAllowed: true,
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
            C: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryCategories = {
            B1: {
                parent: 'B',
                fields: {},
                description: {}
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                description: {},
                valuelists: {},
                createdBy: '',
                creationDate: '',
                groups: []
            }
        };
        const result = buildRawArray(builtInCategories, libraryCategories, libraryForms, undefined, {}, {}, {}, {}, [], undefined, [], [], undefined, ['B']);
        expect(result.length).toBe(5);
        expect(result.find(category => category.libraryId === 'A')).toBeDefined();
        expect(result.find(category => category.libraryId === 'A:default')).toBeDefined();
        expect(result.find(category => category.libraryId === 'B')).toBeDefined();
        expect(result.find(category => category.libraryId === 'B1')).toBeDefined();
        expect(result.find(category => category.libraryId === 'C')).toBeDefined();
    });
    it('inherit valuelist id from parent set in library forms without provided custom forms', () => {
        const builtInCategories = {
            A: {
                supercategory: true,
                userDefinedSubcategoriesAllowed: true,
                fields: {
                    field1: {
                        inputType: 'input'
                    }
                },
                minimalForm: {
                    groups: [{ 'name': 'stem', 'fields': ['field1'] }]
                }
            }
        };
        const libraryCategories = {
            A1: {
                parent: 'A',
                fields: {},
                description: {}
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                description: {},
                fields: {
                    field1: {
                        inputType: 'dropdown'
                    }
                },
                valuelists: {
                    field1: 'list1'
                },
                createdBy: '',
                creationDate: '',
                groups: []
            }
        };
        const result = buildRawArray(builtInCategories, libraryCategories, libraryForms, undefined, {}, {
            'list1': {
                values: { a: {} }, description: {}, createdBy: '', creationDate: ''
            }
        }, {}, {}, [], undefined, [], [], undefined, ['A:default']);
        expect(result.length).toBe(3);
        expect(result.find(category => category.libraryId === 'A').groups[0].fields[0].valuelist).toBeUndefined();
        expect(result.find(category => category.libraryId === 'A:default').groups[0].fields[0].valuelist.values).toEqual({ a: {} });
        expect(result.find(category => category.libraryId === 'A1').groups[0].fields[0].valuelist.values).toEqual({ a: {} });
    });
    it('hide fields', () => {
        const builtInCategories = {
            A: {
                fields: {
                    field1: { inputType: 'input' },
                    field2: { inputType: 'input' }
                },
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['field1', 'field2'] }]
                }
            }
        };
        const libraryCategories = {
            A: {
                fields: {
                    field3: { inputType: 'input' },
                    field4: { inputType: 'input' }
                },
                description: {}
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                groups: [{
                        name: group_1.Groups.STEM,
                        fields: ['field1', 'field2', 'field3', 'field4', 'aCommonField', 'bCommonField']
                    }],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                fields: {},
                hidden: ['field1', 'aCommonField', 'field3']
            }
        };
        const commonFields = {
            aCommonField: { inputType: 'input' },
            bCommonField: { inputType: 'input' }
        };
        const result = buildRaw(builtInCategories, libraryCategories, libraryForms, customForms, commonFields);
        expect(result['A'].groups[0].fields[0].visible).toBe(false);
        expect(result['A'].groups[0].fields[1].visible).toBe(true);
        expect(result['A'].groups[0].fields[2].visible).toBe(false);
        expect(result['A'].groups[0].fields[3].visible).toBe(true);
        expect(result['A'].groups[0].fields[4].visible).toBe(false);
        expect(result['A'].groups[0].fields[5].visible).toBe(true);
    });
    it('valuelistId - provided via valuelistId property in built-in category', () => {
        const builtInCategories = {
            A: {
                fields: {
                    aField: {
                        inputType: 'dropdown',
                        valuelistId: 'aField-valuelist-id-1'
                    }
                },
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['aField'] }]
                }
            }
        };
        const customForms = {
            'A': {
                fields: {}
            }
        };
        const result = buildRaw(builtInCategories, {}, {}, customForms, {}, {
            'aField-valuelist-id-1': {
                values: { a: {} }, description: {}, createdBy: '', creationDate: ''
            }
        });
        expect(result['A'].groups[0].fields[0]['valuelist']['values']).toEqual({ a: {} });
    });
    it('valuelistId - overwrite valuelists property in custom form, extending a library form - for a common field', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                groups: [{ name: group_1.Groups.STEM, fields: ['aCommon'] }],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const commonFields = {
            aCommon: {
                inputType: 'dropdown',
                valuelistId: 'aCommon-valuelist-id-1'
            }
        };
        const customForms = {
            'A:default': {
                valuelists: { aCommon: 'aCommon-valuelist-id-2' },
                fields: {}
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms, commonFields, {
            'aCommon-valuelist-id-1': {
                values: { a: {} }, description: {}, createdBy: '', creationDate: ''
            },
            'aCommon-valuelist-id-2': {
                values: { b: {} }, description: {}, createdBy: '', creationDate: ''
            }
        });
        expect(result['A'].groups[0].fields[0]['valuelist']['values']).toEqual({ b: {} });
    });
    it('valuelistId - overwrite valuelistId in library form', () => {
        const builtInCategories = {
            A: {
                fields: {
                    aField: {
                        inputType: 'dropdown',
                        valuelistId: 'aField-valuelist-id-1'
                    }
                },
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['aField'] }]
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                groups: [{ name: group_1.Groups.STEM, fields: ['aField'] }],
                valuelists: { aField: 'aField-valuelist-id-2' },
                description: {},
                createdBy: '',
                creationDate: ''
            }
        };
        const customForms = {
            'A:default': {
                fields: {}
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms, {}, {
            'aField-valuelist-id-1': {
                values: { a: {} }, description: {}, creationDate: '', createdBy: ''
            },
            'aField-valuelist-id-2': {
                values: { b: {} }, description: {}, creationDate: '', createdBy: ''
            }
        });
        expect(result['A'].groups[0].fields[0]['valuelist']['values']).toEqual({ b: {} });
    });
    it('valuelistId - set valuelistId in library parent form', () => {
        const builtInCategories = {
            A: {
                fields: {
                    aField: {
                        inputType: 'input'
                    }
                },
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['aField'] }]
                }
            },
            B: {
                parent: 'A',
                fields: {},
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['aField'] }]
                }
            },
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                groups: [{ name: group_1.Groups.STEM, fields: ['aField'] }],
                fields: {
                    aField: {
                        inputType: 'dropdown'
                    }
                },
                valuelists: {
                    aField: 'aField-valuelist-id-1'
                },
                description: {},
                createdBy: '',
                creationDate: ''
            }
        };
        const customForms = {
            'A:default': {
                fields: {}
            },
            B: {
                fields: {}
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms, {}, {
            'aField-valuelist-id-1': {
                values: { a: {} }, description: {}, creationDate: '', createdBy: ''
            }
        });
        expect(result['A'].groups[0].fields[0]['valuelist']['values']).toEqual({ a: {} });
        expect(result['B'].groups[0].fields[0]['valuelist']['values']).toEqual({ a: {} });
    });
    it('valuelistId - nowhere provided - minimal form selected', () => {
        const builtInCategories = {
            A: {
                fields: {
                    aField: { inputType: 'dropdown' }
                },
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['aField'] }]
                }
            }
        };
        const customForms = {
            'A': {
                fields: {}
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, {}, {}, customForms, {}, {}, {});
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([
                [configuration_errors_1.ConfigurationErrors.NO_VALUELIST_PROVIDED, 'A', 'aField']
            ]);
        }
    });
    it('valuelistId - nowhere provided - library form selected', () => {
        const builtInCategories = {
            A: {
                fields: {
                    aField: { inputType: 'dropdown' }
                },
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:0': {
                categoryName: 'A',
                valuelists: {},
                groups: [{ name: group_1.Groups.STEM, fields: ['aField'] }],
                createdBy: '',
                creationDate: '',
                description: {}
            },
        };
        const customForms = {
            'A:0': {
                fields: {}
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, {}, libraryForms, customForms, {}, {}, {});
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([
                [configuration_errors_1.ConfigurationErrors.NO_VALUELIST_PROVIDED, 'A', 'aField']
            ]);
        }
    });
    it('valuelistId - nowhere provided - custom form selected', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const customForms = {
            A: {
                fields: {
                    aField: { inputType: 'dropdown' }
                }
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, {}, {}, customForms, {}, {}, {});
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([
                [configuration_errors_1.ConfigurationErrors.NO_VALUELIST_PROVIDED, 'A', 'aField']
            ]);
        }
    });
    it('use a valuelist from custom configuration', () => {
        const builtInCategories = {
            A: {
                fields: {
                    a1: {
                        inputType: 'dropdown',
                        valuelistId: 'a1-library'
                    }
                },
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                groups: [{ name: group_1.Groups.STEM, fields: ['a1'] }],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                valuelists: { a1: 'a1-custom' },
                fields: {}
            }
        };
        const libraryValuelists = {
            'a1-library': {
                values: { a: {} }, description: {}, createdBy: '', creationDate: ''
            }
        };
        const customValuelists = {
            'a1-custom': {
                values: { b: {} }, description: {}, createdBy: '', creationDate: ''
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms, {}, libraryValuelists, customValuelists);
        expect(result['A'].groups[0].fields[0]['valuelist']['values']).toEqual({ b: {} });
        expect(result['A'].groups[0].fields[0]['valuelist']['source']).toBe('custom');
    });
    it('prevent overwriting library valuelist in custom configuration', () => {
        const builtInCategories = {
            A: {
                fields: {
                    a1: {
                        inputType: 'dropdown',
                        valuelistId: 'a1-library'
                    }
                },
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                groups: [{ name: group_1.Groups.STEM, fields: ['a1'] }],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                valuelists: {},
                fields: {}
            }
        };
        const libraryValuelists = {
            'a1-library': {
                values: { a: {} }, description: {}, createdBy: '', creationDate: ''
            }
        };
        const customValuelists = {
            'a1-library': {
                values: { b: {} }, description: {}, createdBy: '', creationDate: ''
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms, {}, libraryValuelists, customValuelists);
        expect(result['A'].groups[0].fields[0]['valuelist']['values']).toEqual({ a: {} });
        expect(result['A'].groups[0].fields[0]['valuelist']['source']).toBe('library');
    });
    it('inherit custom valuelist from parent form', () => {
        const builtInCategories = {
            A: {
                supercategory: true,
                userDefinedSubcategoriesAllowed: true,
                fields: {
                    a1: {
                        inputType: 'dropdown',
                        valuelistId: 'a1-library'
                    }
                },
                minimalForm: {
                    groups: [
                        { name: group_1.Groups.STEM, fields: ['a1'] }
                    ]
                }
            },
            B: {
                parent: 'A',
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const customForms = {
            A: {
                valuelists: { a1: 'a1-custom' },
                fields: {}
            },
            B: {
                fields: {}
            },
            C: {
                parent: 'A',
                fields: {},
                groups: [
                    { name: group_1.Groups.STEM, fields: ['a1'] }
                ]
            },
            D: {
                parent: 'A',
                valuelists: { a1: 'a1-custom-2' },
                fields: {},
                groups: [
                    { name: group_1.Groups.STEM, fields: ['a1'] }
                ]
            }
        };
        const libraryValuelists = {
            'a1-library': {
                values: { a: {} }, description: {}, createdBy: '', creationDate: ''
            }
        };
        const customValuelists = {
            'a1-custom': {
                values: { b: {} }, description: {}, createdBy: '', creationDate: ''
            },
            'a1-custom-2': {
                values: { c: {} }, description: {}, createdBy: '', creationDate: ''
            }
        };
        const result = buildRaw(builtInCategories, {}, {}, customForms, {}, libraryValuelists, customValuelists);
        expect(result['A'].groups[0].fields[0]['valuelist']['values']).toEqual({ b: {} });
        expect(result['A'].groups[0].fields[0]['valuelist']['source']).toBe('custom');
        expect(result['B'].groups[0].fields[0]['valuelist']['values']).toEqual({ b: {} });
        expect(result['B'].groups[0].fields[0]['valuelist']['source']).toBe('custom');
        expect(result['C'].groups[0].fields[0]['valuelist']['values']).toEqual({ b: {} });
        expect(result['C'].groups[0].fields[0]['valuelist']['source']).toBe('custom');
        expect(result['D'].groups[0].fields[0]['valuelist']['values']).toEqual({ b: {} });
        expect(result['D'].groups[0].fields[0]['valuelist']['source']).toBe('custom');
    });
    it('remove valuelist from field if a non-valuelist input type is set', () => {
        const builtInCategories = {
            A: {
                fields: {
                    aField: {
                        inputType: 'dropdown',
                        valuelistId: 'aField-valuelist'
                    }
                },
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['aField'] }]
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                groups: [{ name: group_1.Groups.STEM, fields: ['aField'] }],
                fields: {
                    aField: {
                        inputType: 'text'
                    }
                },
                description: {},
                createdBy: '',
                creationDate: ''
            }
        };
        const customForms = {
            'A:default': {
                fields: {}
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms, {}, {
            'aField-valuelist': {
                values: { a: {} }, description: {}, creationDate: '', createdBy: ''
            }
        });
        expect(result['A'].groups[0].fields[0]['valuelist']).toBeUndefined();
    });
    it('extend valuelist', () => {
        const builtInCategories = {
            A: {
                supercategory: true,
                fields: {
                    a1: {
                        inputType: 'dropdown',
                        valuelistId: 'a1-library'
                    }
                },
                minimalForm: {
                    groups: [
                        { name: group_1.Groups.STEM, fields: ['a1'] }
                    ]
                }
            }
        };
        const customForms = {
            A: {
                valuelists: { a1: 'a1-custom' },
                fields: {}
            }
        };
        const libraryValuelists = {
            'a1-library': {
                values: { a: {}, b: {} },
                description: {},
                createdBy: '',
                creationDate: ''
            }
        };
        const customValuelists = {
            'a1-custom': {
                extendedValuelist: 'a1-library',
                values: { c: {} },
                hidden: ['b'],
                description: {},
                createdBy: '',
                creationDate: ''
            }
        };
        const result = buildRaw(builtInCategories, {}, {}, customForms, {}, libraryValuelists, customValuelists);
        expect(result['A'].groups[0].fields[0]['valuelist']['values']).toEqual({ a: {}, c: {} });
        expect(result['A'].groups[0].fields[0]['valuelist']['source']).toBe('custom');
    });
    it('duplication in selection - two library forms of the same category', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:0': {
                categoryName: 'A',
                valuelists: {},
                groups: [],
                createdBy: '',
                creationDate: '',
                description: {}
            },
            'A:1': {
                categoryName: 'A',
                valuelists: {},
                groups: [],
                createdBy: '',
                creationDate: '',
                description: {}
            }
        };
        const customForms = {
            'A:0': {
                fields: {}
            },
            'A:1': {
                fields: {}
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, {}, libraryForms, customForms);
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([[configuration_errors_1.ConfigurationErrors.DUPLICATION_IN_SELECTION, 'A']]);
        }
    });
    it('duplication in selection - minimal form and library form of the same category', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:0': {
                categoryName: 'A',
                groups: [],
                valuelists: {},
                createdBy: '',
                creationDate: '',
                description: {}
            }
        };
        const customForms = {
            'A': { fields: {} },
            'A:0': { fields: {} }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, {}, libraryForms, customForms);
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([[configuration_errors_1.ConfigurationErrors.DUPLICATION_IN_SELECTION, 'A']]);
        }
    });
    it('subcategories - library subcategory not allowed', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryCategories = {
            B: {
                parent: 'A',
                fields: {},
                description: {}
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, libraryCategories, {}, {});
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([
                configuration_errors_1.ConfigurationErrors.TRYING_TO_SUBTYPE_A_NON_EXTENDABLE_CATEGORY, 'A'
            ]);
        }
    });
    it('subcategories - custom subcategory not allowed', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const customForms = {
            B: {
                parent: 'A',
                fields: {}
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, {}, {}, customForms);
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([
                configuration_errors_1.ConfigurationErrors.TRYING_TO_SUBTYPE_A_NON_EXTENDABLE_CATEGORY, 'A'
            ]);
        }
    });
    it('ignore attempts to overwrite fields in library categories', () => {
        const builtInCategories = {
            A: {
                fields: {
                    aField: { inputType: 'input' }
                },
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['aField', 'aCommon'] }]
                }
            }
        };
        const libraryCategories = {
            A: {
                fields: {
                    aField: { inputType: 'boolean' },
                    aCommon: { inputType: 'boolean' }
                },
                description: {}
            }
        };
        const commonFields = {
            aCommon: { inputType: 'text' }
        };
        const customForms = {
            A: {
                fields: {}
            }
        };
        const result = buildRaw(builtInCategories, libraryCategories, {}, customForms, commonFields);
        expect(result['A'].groups[0].fields[0].inputType).toBe('input');
        expect(result['A'].groups[0].fields[1].inputType).toBe('text');
    });
    it('allow overwriting inputType & constraintIndexed in library forms', () => {
        const builtInCategories = {
            A: {
                fields: {
                    aField: { inputType: 'input', constraintIndexed: true }
                },
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['aField', 'aCommon'] }]
                }
            },
            B: {
                parent: 'A',
                fields: {},
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['aField', 'aCommon'] }]
                }
            }
        };
        const commonFields = {
            aCommon: { inputType: 'text', constraintIndexed: true }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                groups: [{ name: group_1.Groups.STEM, fields: ['aField', 'aCommon'] }],
                fields: {
                    aField: {
                        inputType: 'boolean',
                        constraintIndexed: false
                    },
                    aCommon: {
                        inputType: 'boolean',
                        constraintIndexed: false
                    }
                },
                valuelists: {},
                createdBy: '',
                creationDate: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                fields: {}
            },
            B: {
                fields: {}
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms, commonFields);
        expect(result['A'].groups[0].fields[0].inputType).toBe('boolean');
        expect(result['A'].groups[0].fields[0].constraintIndexed).toBe(false);
        expect(result['A'].groups[0].fields[1].inputType).toBe('boolean');
        expect(result['A'].groups[0].fields[1].constraintIndexed).toBe(false);
        expect(result['B'].groups[0].fields[0].inputType).toBe('boolean');
        expect(result['B'].groups[0].fields[0].constraintIndexed).toBe(false);
        expect(result['B'].groups[0].fields[1].inputType).toBe('boolean');
        expect(result['B'].groups[0].fields[1].constraintIndexed).toBe(false);
    });
    it('allow overwriting inputType & constraintIndexed in custom forms', () => {
        const builtInCategories = {
            A: {
                fields: {
                    aField: { inputType: 'input', constraintIndexed: true }
                },
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['aField', 'aCommon'] }]
                }
            },
            B: {
                parent: 'A',
                fields: {},
                minimalForm: {
                    groups: [{ name: group_1.Groups.STEM, fields: ['aField', 'aCommon'] }]
                }
            }
        };
        const commonFields = {
            aCommon: { inputType: 'text', constraintIndexed: false }
        };
        const customForms = {
            A: {
                fields: {
                    aField: { inputType: 'boolean', constraintIndexed: false },
                    aCommon: { inputType: 'boolean', constraintIndexed: true }
                }
            },
            B: {
                fields: {}
            }
        };
        const result = buildRaw(builtInCategories, {}, {}, customForms, commonFields);
        expect(result['A'].groups[0].fields[0].inputType).toBe('boolean');
        expect(result['A'].groups[0].fields[0].constraintIndexed).toBe(false);
        expect(result['A'].groups[0].fields[1].inputType).toBe('boolean');
        expect(result['A'].groups[0].fields[1].constraintIndexed).toBe(true);
        expect(result['B'].groups[0].fields[0].inputType).toBe('boolean');
        expect(result['B'].groups[0].fields[0].constraintIndexed).toBe(false);
        expect(result['B'].groups[0].fields[1].inputType).toBe('boolean');
        expect(result['B'].groups[0].fields[1].constraintIndexed).toBe(true);
    });
    it('throw error if field not found', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const customForms = {
            A: {
                fields: {},
                groups: [{ name: group_1.Groups.STEM, fields: ['missing'] }]
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, {}, {}, customForms);
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([[configuration_errors_1.ConfigurationErrors.FIELD_NOT_FOUND, 'A', 'missing']]);
        }
    });
    it('commons - select common field in library form', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const commonFields = {
            aCommon: { inputType: 'input' }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                groups: [{ name: group_1.Groups.STEM, fields: ['aCommon'] }],
                valuelists: {},
                createdBy: '',
                creationDate: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                fields: {}
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms, commonFields);
        expect(result['A'].groups[0].fields[0]['inputType']).toBe('input');
    });
    it('commons - select common field in custom form', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const commonFields = {
            aCommon: { inputType: 'input' }
        };
        const customForms = {
            A: {
                fields: {},
                groups: [{ name: group_1.Groups.STEM, fields: ['aCommon'] }]
            }
        };
        const result = buildRaw(builtInCategories, {}, {}, customForms, commonFields);
        expect(result['A'].groups[0].fields[0]['inputType']).toBe('input');
    });
    it('commons - use valuelistFromProjectField if defined in commons', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const commonFields = {
            aCommon: { inputType: 'dropdown', valuelistFromProjectField: 'x' }
        };
        const libraryForms = {
            'A:0': {
                categoryName: 'A',
                groups: [{ name: group_1.Groups.STEM, fields: ['aCommon'] }],
                valuelists: {},
                createdBy: '',
                creationDate: '',
                description: {}
            }
        };
        const customForms = {
            'A:0': {
                fields: {}
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms, commonFields);
        expect(result['A'].groups[0].fields[0]['inputType']).toBe('dropdown');
        expect(result['A'].groups[0].fields[0]['valuelistFromProjectField']).toBe('x');
    });
    // err cases
    it('field property validation - invalid input type', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryCategories = {
            A: {
                fields: {
                    aField: { inputType: 'invalid' }
                },
                description: {}
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                groups: [{ name: group_1.Groups.STEM, fields: ['aField'] }],
                description: {},
                createdBy: '',
                creationDate: ''
            }
        };
        const customForms = {
            'A:default': {
                fields: {}
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, libraryCategories, libraryForms, customForms);
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([
                configuration_errors_1.ConfigurationErrors.ILLEGAL_FIELD_INPUT_TYPE, 'invalid', 'aField'
            ]);
        }
    });
    it('field property validation - missing input type in field of library category', () => {
        const builtInCategories = {
            A: {
                supercategory: true,
                userDefinedSubcategoriesAllowed: true,
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryCategories = {
            B: {
                parent: 'A',
                fields: {
                    bField: {}
                },
                description: {}
            },
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, libraryCategories, {}, {}, {}, {});
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([
                [configuration_errors_1.ConfigurationErrors.MISSING_FIELD_PROPERTY, 'inputType', 'B', 'bField']
            ]);
        }
    });
    it('field property validation - illegal property in library category field', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryCategories = {
            A: {
                fields: { aField: { xyz: 'a' } },
                description: {}
            },
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, libraryCategories, {}, {}, {}, {});
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([
                configuration_errors_1.ConfigurationErrors.ILLEGAL_FIELD_PROPERTY, 'library', 'xyz'
            ]);
        }
    });
    it('field property validation - illegal property in custom form field', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const customForms = {
            'A': {
                fields: { aField: { xyz: 'a' } }
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, {}, {}, customForms, {}, {}, {});
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([
                configuration_errors_1.ConfigurationErrors.ILLEGAL_FIELD_PROPERTY, 'custom', 'xyz'
            ]);
        }
    });
    it('apply valuelists configuration', () => {
        const builtInCategories = {
            A: {
                fields: {
                    field1: {
                        inputType: 'dropdown',
                        valuelistId: '123'
                    }
                },
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1'] }
                ],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const valuelistsConfiguration = {
            '123': {
                values: {
                    'one': { label: { de: 'Eins', en: 'One' } },
                    'two': { references: ['https://xyz.de/1234567'] },
                    'three': {}
                },
                id: '123',
                description: {},
                createdBy: '',
                creationDate: ''
            }
        };
        const customForms = {
            'A:default': {
                fields: {}
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms, {}, valuelistsConfiguration);
        expect(result['A'].groups[0].fields[0].valuelist.values).toEqual({
            one: { label: { de: 'Eins', en: 'One' } },
            two: { references: ['https://xyz.de/1234567'] },
            three: {}
        });
    });
    it('no minimal form provided', () => {
        const builtInCategories = {
            A: {
                fields: {},
                supercategory: true,
                userDefinedSubcategoriesAllowed: true
            }
        };
        const libraryCategories = {
            B: {
                parent: 'A',
                fields: {},
                description: {}
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, libraryCategories, {}, {}, {}, {});
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([[
                    configuration_errors_1.ConfigurationErrors.NO_MINIMAL_FORM_PROVIDED, 'A'
                ]]);
        }
    });
    it('missing description in library category', () => {
        const builtInCategories = {};
        const libraryCategories = {
            A: {
                fields: {}
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, libraryCategories, {}, {}, {}, {});
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([
                configuration_errors_1.ConfigurationErrors.MISSING_CATEGORY_PROPERTY, 'description', 'A'
            ]);
        }
    });
    it('missing description in library form', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryCategories = {
            A: {
                fields: {},
                description: {}
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                createdBy: '',
                creationDate: '',
                groups: [],
                valuelists: {}
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, libraryCategories, libraryForms, {}, {}, {});
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([
                configuration_errors_1.ConfigurationErrors.MISSING_FORM_PROPERTY, 'description', 'A:default'
            ]);
        }
    });
    it('missing parent in library category', () => {
        const builtInCategories = {};
        const libraryCategories = {
            'B': {
                fields: {},
                description: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, libraryCategories, {}, {}, {}, {});
            fail();
        }
        catch (expected) {
            expect(expected).toEqual([configuration_errors_1.ConfigurationErrors.MISSING_CATEGORY_PROPERTY, 'parent', 'B']);
        }
    });
    it('missing parent in custom form', () => {
        const customForms = {
            'A': { fields: {} }
        };
        try {
            build_raw_project_configuration_1.buildRawProjectConfiguration({}, {}, {}, customForms, {}, {}, {});
        }
        catch (expected) {
            expect(expected).toEqual([
                configuration_errors_1.ConfigurationErrors.MISSING_FORM_PROPERTY, 'parent', 'A', 'must be set for new categories'
            ]);
        }
    });
    it('merge library category with built-in category', () => {
        const builtInCategories = {
            A: {
                fields: {
                    field1: { inputType: 'text' }
                },
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryCategories = {
            'A': {
                fields: {
                    field1: { inputType: 'dropdown' },
                    field2: { inputType: 'text' }
                },
                minimalForm: {
                    groups: [
                        { name: group_1.Groups.STEM, fields: ['field1'] },
                        { name: group_1.Groups.PROPERTIES, fields: ['field2'] }
                    ]
                },
                description: {}
            }
        };
        const customForms = {
            'A': { hidden: [], fields: {} }
        };
        const result = buildRaw(builtInCategories, libraryCategories, {}, customForms);
        expect(result['A'].groups[0].fields[0].inputType).toBe('text');
        expect(result['A'].groups[0].name).toBe(group_1.Groups.STEM);
        expect(result['A'].groups[1].fields[0].inputType).toBe('text');
        expect(result['A'].groups[1].name).toBe(group_1.Groups.PROPERTIES);
    });
    it('merge custom forms with built-in categories', () => {
        const builtInCategories = {
            A: {
                fields: {
                    field1: { inputType: 'text' }
                },
                minimalForm: {
                    groups: [
                        { name: group_1.Groups.STEM, fields: ['field1'] }
                    ]
                }
            }
        };
        const customForms = {
            A: {
                fields: {
                    field1: { inputType: 'input' },
                    field2: { inputType: 'text' }
                },
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1', 'field2'] }
                ]
            }
        };
        const result = buildRaw(builtInCategories, {}, {}, customForms);
        expect(result['A'].groups[0].fields[0].inputType).toBe('input');
        expect(result['A'].groups[0].fields[1].inputType).toBe('text');
    });
    it('merge custom forms with library forms', () => {
        const builtInCategories = {
            A: {
                fields: {
                    field1: { inputType: 'text' },
                    field2: { inputType: 'text' },
                    field3: { inputType: 'text' }
                },
                minimalForm: {
                    groups: [
                        { name: group_1.Groups.STEM, fields: ['field1'] }
                    ]
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                fields: {
                    field3: { inputType: 'int' },
                    field4: { inputType: 'text' }
                },
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1'] },
                    { name: group_1.Groups.PROPERTIES, fields: ['field2', 'field3', 'field4'] }
                ],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                fields: {
                    field2: { inputType: 'input' },
                    field5: { inputType: 'boolean' }
                },
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1'] },
                    { name: group_1.Groups.PROPERTIES, fields: ['field2', 'field3', 'field4', 'field5'] }
                ]
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms);
        expect(result['A'].groups[0].fields[0].inputType).toBe('text');
        expect(result['A'].groups[0].name).toBe(group_1.Groups.STEM);
        expect(result['A'].groups[1].fields[0].inputType).toBe('input');
        expect(result['A'].groups[1].fields[1].inputType).toBe('int');
        expect(result['A'].groups[1].fields[2].inputType).toBe('text');
        expect(result['A'].groups[1].fields[3].inputType).toBe('boolean');
        expect(result['A'].groups[1].name).toBe(group_1.Groups.PROPERTIES);
    });
    it('add all fields from library form even if not selected in custom form group', () => {
        const builtInCategories = {
            A: {
                fields: {
                    field1: { inputType: 'text' },
                    field2: { inputType: 'boolean' }
                },
                minimalForm: {
                    groups: [
                        { name: group_1.Groups.STEM, fields: ['field1'] }
                    ]
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1', 'field2'] }
                ],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                fields: {},
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1'] }
                ],
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms);
        expect(result['A'].groups[0].fields[0].inputType).toBe('text');
        expect(result['A'].groups[1].fields[0].inputType).toBe('boolean');
        expect(result['A'].groups[0].name).toBe(group_1.Groups.STEM);
    });
    it('set source field', () => {
        const commonFields = {
            aCommon: { inputType: field_1.Field.InputType.INPUT }
        };
        const builtInCategories = {
            A: {
                fields: {
                    field1: { inputType: field_1.Field.InputType.TEXT }
                },
                minimalForm: {
                    groups: [
                        { name: group_1.Groups.STEM, fields: ['field1'] }
                    ]
                }
            }
        };
        const libraryCategories = {
            'A': {
                fields: {
                    field2: { inputType: field_1.Field.InputType.TEXT }
                },
                description: {}
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                creationDate: '',
                createdBy: '',
                description: {},
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1'] }
                ]
            }
        };
        const customForms = {
            'A:default': {
                fields: {
                    field3: { inputType: field_1.Field.InputType.TEXT }
                },
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1'] },
                    { name: group_1.Groups.PROPERTIES, fields: ['aCommon', 'field2', 'field3'] },
                ]
            }
        };
        const result = buildRaw(builtInCategories, libraryCategories, libraryForms, customForms, commonFields);
        expect(result['A'].groups[0].fields[0].source).toBe(field_1.Field.Source.BUILTIN);
        expect(result['A'].groups[0].name).toBe(group_1.Groups.STEM);
        expect(result['A'].groups[1].fields[0].source).toBe(field_1.Field.Source.COMMON);
        expect(result['A'].groups[1].fields[1].source).toBe(field_1.Field.Source.LIBRARY);
        expect(result['A'].groups[1].fields[2].source).toBe(field_1.Field.Source.CUSTOM);
        expect(result['A'].groups[1].name).toBe(group_1.Groups.PROPERTIES);
    });
    it('set group labels', () => {
        const builtInCategories = {
            A: {
                supercategory: true,
                userDefinedSubcategoriesAllowed: true,
                fields: {
                    field1: { inputType: field_1.Field.InputType.TEXT },
                    field2: { inputType: field_1.Field.InputType.TEXT },
                    field3: { inputType: field_1.Field.InputType.TEXT }
                },
                minimalForm: {
                    groups: [
                        { name: group_1.Groups.STEM, fields: ['field1', 'field2'] },
                        { name: group_1.Groups.PROPERTIES, fields: ['field3'] }
                    ]
                }
            }
        };
        const libraryCategories = {
            B: {
                parent: 'A',
                fields: {
                    field4: { inputType: field_1.Field.InputType.TEXT },
                },
                minimalForm: {
                    groups: [
                        { name: group_1.Groups.STEM, fields: ['field1', 'field2'] },
                        { name: group_1.Groups.PROPERTIES, fields: ['field3', 'field4'] }
                    ]
                },
                description: {}
            }
        };
        const customForms = {
            A: {
                fields: {}
            },
            B: {
                fields: {}
            }
        };
        const defaultLanguageConfigurations = {
            de: [{
                    groups: {
                        'stem': 'Stem',
                        'properties': 'Eigenschaften'
                    },
                    categories: {
                        A: { label: 'A_' },
                        B: { label: 'B_' }
                    }
                }]
        };
        const languageConfigurations = {
            complete: defaultLanguageConfigurations,
            custom: {},
            default: defaultLanguageConfigurations
        };
        const result = buildRaw(builtInCategories, libraryCategories, {}, customForms, {}, {}, {}, {}, [], languageConfigurations);
        expect(result['A'].groups[0].label.de).toEqual('Stem');
        expect(result['A'].groups[1].label.de).toEqual('Eigenschaften');
        expect(result['B'].groups[0].label.de).toEqual('Stem');
        expect(result['B'].groups[1].label.de).toEqual('Eigenschaften');
    });
    it('apply categories order', () => {
        const builtInCategories = {
            B: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
            C: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const customForms = {
            B: { fields: {} },
            A: { fields: {} },
            C: { fields: {} }
        };
        const orderConfiguration = ['C', 'A'];
        const result = buildRawArray(builtInCategories, {}, {}, customForms, {}, {}, {}, {}, [], { default: {}, custom: {}, complete: {} }, [], orderConfiguration).map(named_1.Named.toName);
        expect(result).toEqual(['C', 'A', 'B']);
    });
    it('apply categories order to children', () => {
        const builtInCategories = {
            B: {
                parent: 'D',
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
            A: {
                parent: 'D',
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
            C: {
                parent: 'D',
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
            D: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
        };
        const customForms = {
            B: { fields: {} },
            A: { fields: {} },
            C: { fields: {} },
            D: { fields: {} }
        };
        const orderConfiguration = ['C', 'A'];
        const result = buildRaw(builtInCategories, {}, {}, customForms, {}, {}, {}, {}, [], { default: {}, custom: {}, complete: {} }, [], orderConfiguration)['D'].children.map(tsfun_1.to(named_1.Named.NAME));
        expect(result).toEqual(['C', 'A', 'B']);
    });
    it('put relations into groups', () => {
        const builtInCategories = {
            P: {
                supercategory: true,
                userDefinedSubcategoriesAllowed: true,
                fields: {},
                minimalForm: {
                    groups: [
                        { name: group_1.Groups.POSITION, fields: ['isAbove'] }
                    ]
                }
            }
        };
        const customForms = {
            P: {
                fields: {}
            },
            C: {
                parent: 'P',
                fields: {},
                groups: [
                    { name: group_1.Groups.POSITION, fields: ['isAbove'] }
                ]
            }
        };
        const relationDefinitions = [{
                name: 'isAbove',
                inverse: 'isBelow',
                domain: ['P:inherit'],
                range: ['P:inherit'],
                inputType: 'relation',
                sameMainCategoryResource: true
            }];
        const result = buildRaw(builtInCategories, {}, {}, customForms, {}, {}, {}, {}, relationDefinitions);
        const parentGroup = result['P'].groups[0];
        const childGroup = result['P'].children[0].groups[0];
        expect(parentGroup.name).toEqual(group_1.Groups.POSITION);
        expect(parentGroup.fields[0].name).toEqual('isAbove');
        expect(childGroup.name).toEqual(group_1.Groups.POSITION);
        expect(childGroup.fields[0].name).toEqual('isAbove');
    });
    it('link parent and child instances', () => {
        const builtInCategories = {
            P: {
                supercategory: true,
                userDefinedSubcategoriesAllowed: true,
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
        };
        const customForms = {
            P: {
                fields: {}
            },
            C: {
                parent: 'P',
                fields: {}
            }
        };
        const categoryFormsTree = build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, {}, {}, customForms).forms;
        expect(forest_1.Tree.access(categoryFormsTree, 0).children[0].name).toBe('C');
        expect(forest_1.Tree.access(categoryFormsTree, 0, 0).name).toBe('C');
        expect(forest_1.Tree.access(categoryFormsTree, 0).children[0]
            === forest_1.Tree.access(categoryFormsTree, 0, 0)).toBeTruthy();
        expect(forest_1.Tree.access(categoryFormsTree, 0).name).toBe('P');
        expect(forest_1.Tree.access(categoryFormsTree, 0, 0).parentCategory.name).toBe('P');
        expect(forest_1.Tree.access(categoryFormsTree, 0, 0).parentCategory
            === forest_1.Tree.access(categoryFormsTree, 0)).toBeTruthy();
    });
    it('allow overwriting color', () => {
        const builtInCategories = {
            A: {
                color: 'blue',
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
            B: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                groups: [],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                fields: {},
                color: 'red'
            },
            B: {
                fields: {},
                color: 'red'
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms);
        expect(result['A'].color).toBe('red');
        expect(result['A'].defaultColor).toBe('blue');
        expect(result['B'].color).toBe('red');
        expect(result['B'].defaultColor).toBe('#000042'); // Auto-generated color
    });
    it('return categories, common fields and relations with correct labels', () => {
        const commonFields = {
            aCommon: { inputType: field_1.Field.InputType.TEXT }
        };
        const builtInCategories = {
            A: {
                fields: {
                    field1: { inputType: field_1.Field.InputType.TEXT }
                },
                minimalForm: {
                    groups: [
                        { name: group_1.Groups.STEM, fields: ['field1'] }
                    ]
                }
            }
        };
        const libraryCategories = {
            'A': {
                fields: {
                    field2: { inputType: field_1.Field.InputType.TEXT }
                },
                description: {}
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                creationDate: '',
                createdBy: '',
                description: {},
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1'] }
                ]
            }
        };
        const customForms = {
            'A:default': {
                fields: {
                    field3: { inputType: field_1.Field.InputType.TEXT }
                },
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1'] },
                    { name: group_1.Groups.PROPERTIES, fields: ['aCommon', 'field2', 'field3'] },
                ]
            }
        };
        const relationDefinitions = [{
                name: 'isRelated',
                domain: ['A'],
                range: ['A'],
                inputType: 'relation'
            }];
        const defaultLanguageConfiguration = {
            categories: {
                A: {
                    label: 'Default category label',
                    fields: {
                        field1: { label: 'Field 1 Default' },
                        field2: { label: 'Field 2' }
                    }
                }
            },
            commons: {
                aCommon: { label: 'Common field A' }
            },
            relations: {
                isRelated: { label: 'Default relation label' }
            }
        };
        const customLanguageConfiguration = {
            categories: {
                A: {
                    label: 'Custom category label',
                    fields: {
                        field1: { label: 'Field 1 Custom' },
                        field3: { label: 'Field 3' },
                        aCommon: { label: 'Custom common field label' }
                    }
                }
            },
            relations: {
                isRelated: { label: 'Custom relation label' }
            }
        };
        const languageConfigurations = {
            complete: {
                en: [customLanguageConfiguration, defaultLanguageConfiguration]
            },
            default: {
                en: [defaultLanguageConfiguration]
            },
            custom: {
                en: [customLanguageConfiguration]
            }
        };
        const rawConfiguration = build_raw_project_configuration_1.buildRawProjectConfiguration(builtInCategories, libraryCategories, libraryForms, customForms, commonFields, {}, {}, {}, relationDefinitions, languageConfigurations);
        const forms = named_1.Named.arrayToMap(forest_1.Tree.flatten(rawConfiguration.forms));
        expect(rawConfiguration.commonFields['aCommon'].label.en).toBe('Common field A');
        expect(rawConfiguration.commonFields['aCommon'].defaultLabel.en).toBe('Common field A');
        expect(rawConfiguration.categories['A'].label.en).toBe('Custom category label');
        expect(rawConfiguration.categories['A'].defaultLabel.en).toBe('Default category label');
        expect(rawConfiguration.categories['A'].fields['field1'].label.en).toBe('Field 1 Custom');
        expect(rawConfiguration.categories['A'].fields['field1'].defaultLabel.en).toBe('Field 1 Default');
        expect(rawConfiguration.categories['A'].fields['field2'].label.en).toBe('Field 2');
        expect(rawConfiguration.categories['A'].fields['field2'].defaultLabel.en).toBe('Field 2');
        expect(rawConfiguration.categories['A'].fields['field3']).toBe(undefined);
        expect(rawConfiguration.relations[0].label.en).toBe('Custom relation label');
        expect(rawConfiguration.relations[0].defaultLabel.en).toBe('Default relation label');
        expect(forms['A'].label.en).toBe('Custom category label');
        expect(forms['A'].defaultLabel.en).toBe('Default category label');
        expect(forms['A'].groups[0].fields[0].label.en).toBe('Field 1 Custom');
        expect(forms['A'].groups[0].fields[0].defaultLabel.en).toBe('Field 1 Default');
        expect(forms['A'].groups[1].fields[0].label.en).toBe('Custom common field label');
        expect(forms['A'].groups[1].fields[0].defaultLabel.en).toBe('Common field A');
        expect(forms['A'].groups[1].fields[1].label.en).toBe('Field 2');
        expect(forms['A'].groups[1].fields[1].defaultLabel.en).toBe('Field 2');
        expect(forms['A'].groups[1].fields[2].label.en).toBe('Field 3');
        expect(forms['A'].groups[1].fields[2].defaultLabel.en).toBe(undefined);
    });
    it('allow changing constraintIndexed via custom form', () => {
        const builtInCategories = {
            A: {
                supercategory: true,
                userDefinedSubcategoriesAllowed: true,
                fields: {
                    field1: { inputType: 'input', constraintIndexed: false },
                    field2: { inputType: 'input', constraintIndexed: true }
                },
                minimalForm: {
                    groups: []
                }
            }
        };
        const customForms = {
            A: {
                fields: {}
            },
            C: {
                parent: 'A',
                fields: {
                    field1: { constraintIndexed: true },
                    field2: { constraintIndexed: false }
                },
                groups: [{ name: group_1.Groups.STEM, fields: ['field1', 'field2'] }]
            }
        };
        const result = buildRaw(builtInCategories, {}, {}, customForms);
        expect(result['C'].groups[0].fields[0].name).toEqual('field1');
        expect(result['C'].groups[0].fields[0].constraintIndexed).toBe(true);
        expect(result['C'].groups[0].fields[0].defaultConstraintIndexed).toBe(false);
        expect(result['C'].groups[0].fields[1].name).toEqual('field2');
        expect(result['C'].groups[0].fields[1].constraintIndexed).toBe(false);
        expect(result['C'].groups[0].fields[1].defaultConstraintIndexed).toBe(true);
    });
    it('allow setting identifier prefix', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                groups: [],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                fields: {},
                color: 'red',
                identifierPrefix: 'A-'
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms);
        expect(result['A'].identifierPrefix).toBe('A-');
    });
    it('allow setting resource limit', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                groups: [],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                fields: {},
                color: 'red',
                resourceLimit: 2
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms);
        expect(result['A'].resourceLimit).toBe(2);
    });
    it('allow using scan codes', () => {
        const builtInCategories = {
            A: {
                fields: {},
                scanCodesAllowed: true,
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                groups: [],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                fields: {},
                color: 'red',
                scanCodes: {
                    type: 'qr',
                    autoCreate: true,
                    printedFields: []
                }
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms);
        expect(result['A'].scanCodes).toEqual({ type: 'qr', autoCreate: true, printedFields: [] });
    });
    it('inherit scan codes configuration from parent category', () => {
        const builtInCategories = {
            A: {
                fields: {},
                scanCodesAllowed: true,
                minimalForm: {
                    groups: []
                }
            },
            A1: {
                parent: 'A',
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
            B: {
                fields: {},
                scanCodesAllowed: true,
                minimalForm: {
                    groups: []
                }
            },
            B1: {
                parent: 'B',
                fields: {},
                minimalForm: {
                    groups: []
                }
            },
            B2: {
                parent: 'B',
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                groups: [],
                creationDate: '',
                createdBy: '',
                description: {}
            },
            'B:default': {
                categoryName: 'B',
                valuelists: {},
                groups: [],
                creationDate: '',
                createdBy: '',
                description: {}
            },
            'A1:default': {
                categoryName: 'A1',
                valuelists: {},
                groups: [],
                creationDate: '',
                createdBy: '',
                description: {}
            },
            'B1:default': {
                categoryName: 'B1',
                valuelists: {},
                groups: [],
                creationDate: '',
                createdBy: '',
                description: {}
            },
            'B2:default': {
                categoryName: 'B2',
                valuelists: {},
                groups: [],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                fields: {},
                scanCodes: {
                    type: 'qr',
                    autoCreate: true,
                    printedFields: [{ name: 'a', printLabel: true }]
                }
            },
            'A1:default': {
                fields: {}
            },
            'B:default': {
                fields: {},
                scanCodes: {
                    type: 'qr',
                    autoCreate: false,
                    printedFields: [{ name: 'a', printLabel: true }, { name: 'b', printLabel: false }]
                }
            },
            'B1:default': {
                fields: {}
            },
            'B2:default': {
                fields: {},
                scanCodes: {
                    type: 'qr',
                    autoCreate: true,
                    printedFields: [{ name: 'c', printLabel: true }, { name: 'd', printLabel: true }]
                }
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms);
        expect(result['A'].scanCodes).toEqual({ type: 'qr', autoCreate: true, printedFields: [{ name: 'a', printLabel: true }] });
        expect(result['A1'].scanCodes).toEqual({ type: 'qr', autoCreate: true, printedFields: [{ name: 'a', printLabel: true }] });
        expect(result['B'].scanCodes).toEqual({
            type: 'qr', autoCreate: false,
            printedFields: [{ name: 'a', printLabel: true }, { name: 'b', printLabel: false }]
        });
        expect(result['B1'].scanCodes).toEqual({
            type: 'qr', autoCreate: false,
            printedFields: [{ name: 'a', printLabel: true }, { name: 'b', printLabel: false }]
        });
        expect(result['B2'].scanCodes).toEqual({
            type: 'qr', autoCreate: true,
            printedFields: [
                { name: 'a', printLabel: true }, { name: 'b', printLabel: false }, { name: 'c', printLabel: true }
            ]
        });
    });
    it('prevent using scan codes if forbidden for category', () => {
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                groups: [],
                creationDate: '',
                createdBy: '',
                description: {}
            }
        };
        const customForms = {
            'A:default': {
                fields: {},
                color: 'red',
                scanCodes: { type: 'qr', autoCreate: true, printedFields: [] }
            }
        };
        const result = buildRaw(builtInCategories, {}, libraryForms, customForms);
        expect(result['A'].scanCodes).toBeUndefined();
    });
    it('set subfields', () => {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        const builtInCategories = {
            A: {
                fields: {},
                minimalForm: {
                    groups: []
                }
            }
        };
        const libraryCategories = {
            'A': {
                fields: {
                    field1: {
                        inputType: field_1.Field.InputType.COMPOSITE,
                        subfields: [
                            {
                                name: 'subfield1-1',
                                inputType: 'input'
                            },
                            {
                                name: 'subfield1-2',
                                inputType: 'dropdown',
                                valuelistId: 'valuelist-1'
                            }
                        ]
                    },
                },
                description: {}
            }
        };
        const libraryForms = {
            'A:default': {
                categoryName: 'A',
                valuelists: {},
                creationDate: '',
                createdBy: '',
                description: {},
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1'] }
                ]
            }
        };
        const customForms = {
            'A:default': {
                fields: {
                    field2: {
                        inputType: field_1.Field.InputType.COMPOSITE,
                        subfields: [
                            {
                                name: 'subfield2-1',
                                inputType: 'text'
                            },
                            {
                                name: 'subfield2-2',
                                inputType: 'boolean'
                            },
                            {
                                name: 'subfield2-3',
                                inputType: 'checkboxes',
                                condition: {
                                    subfieldName: 'subfield2-2',
                                    values: true
                                }
                            }
                        ]
                    }
                },
                groups: [
                    { name: group_1.Groups.STEM, fields: ['field1', 'field2'] }
                ],
                valuelists: {
                    'field2': {
                        'subfield2-3': 'valuelist-2'
                    }
                }
            }
        };
        const valuelists = {
            'valuelist-1': {
                values: { a: {} }, description: {}, createdBy: '', creationDate: ''
            },
            'valuelist-2': {
                values: { b: {} }, description: {}, createdBy: '', creationDate: ''
            }
        };
        const defaultLanguageConfiguration = {
            categories: {
                A: {
                    fields: {
                        field1: {
                            subfields: {
                                'subfield1-1': {
                                    'label': 'Subfield 1/1 Label',
                                    'description': 'Subfield 1/1 Description'
                                }
                            }
                        }
                    }
                }
            }
        };
        const customLanguageConfiguration = {
            categories: {
                A: {
                    fields: {
                        field2: {
                            subfields: {
                                'subfield2-1': {
                                    'label': 'Subfield 2/1 Label',
                                    'description': 'Subfield 2/1 Description'
                                }
                            }
                        }
                    }
                }
            }
        };
        const languageConfigurations = {
            complete: {
                en: [customLanguageConfiguration, defaultLanguageConfiguration]
            },
            default: {
                en: [defaultLanguageConfiguration]
            },
            custom: {
                en: [customLanguageConfiguration]
            }
        };
        const result = buildRaw(builtInCategories, libraryCategories, libraryForms, customForms, {}, valuelists, {}, {}, [], languageConfigurations);
        expect(result['A'].groups[0].fields[0].inputType).toEqual(field_1.Field.InputType.COMPOSITE);
        expect(result['A'].groups[0].fields[0].subfields.length).toBe(2);
        expect(result['A'].groups[0].fields[0].subfields[0].name).toEqual('subfield1-1');
        expect(result['A'].groups[0].fields[0].subfields[0].inputType).toEqual(field_1.Field.InputType.INPUT);
        expect((_a = result['A'].groups[0].fields[0].subfields[0].label) === null || _a === void 0 ? void 0 : _a.en).toEqual('Subfield 1/1 Label');
        expect((_b = result['A'].groups[0].fields[0].subfields[0].description) === null || _b === void 0 ? void 0 : _b.en).toEqual('Subfield 1/1 Description');
        expect(result['A'].groups[0].fields[0].subfields[1].name).toEqual('subfield1-2');
        expect(result['A'].groups[0].fields[0].subfields[1].inputType).toEqual(field_1.Field.InputType.DROPDOWN);
        expect((_c = result['A'].groups[0].fields[0].subfields[1].valuelist) === null || _c === void 0 ? void 0 : _c.id).toEqual('valuelist-1');
        expect(result['A'].groups[0].fields[1].inputType).toEqual(field_1.Field.InputType.COMPOSITE);
        expect(result['A'].groups[0].fields[1].subfields.length).toBe(3);
        expect(result['A'].groups[0].fields[1].subfields[0].name).toEqual('subfield2-1');
        expect(result['A'].groups[0].fields[1].subfields[0].inputType).toEqual(field_1.Field.InputType.TEXT);
        expect((_d = result['A'].groups[0].fields[1].subfields[0].label) === null || _d === void 0 ? void 0 : _d.en).toEqual('Subfield 2/1 Label');
        expect((_e = result['A'].groups[0].fields[1].subfields[0].description) === null || _e === void 0 ? void 0 : _e.en).toEqual('Subfield 2/1 Description');
        expect(result['A'].groups[0].fields[1].subfields[1].name).toEqual('subfield2-2');
        expect(result['A'].groups[0].fields[1].subfields[1].inputType).toEqual(field_1.Field.InputType.BOOLEAN);
        expect(result['A'].groups[0].fields[1].subfields[2].name).toEqual('subfield2-3');
        expect(result['A'].groups[0].fields[1].subfields[2].inputType).toEqual(field_1.Field.InputType.CHECKBOXES);
        expect((_f = result['A'].groups[0].fields[1].subfields[2].valuelist) === null || _f === void 0 ? void 0 : _f.id).toEqual('valuelist-2');
        expect((_g = result['A'].groups[0].fields[1].subfields[2].condition) === null || _g === void 0 ? void 0 : _g.subfieldName).toEqual('subfield2-2');
        expect((_h = result['A'].groups[0].fields[1].subfields[2].condition) === null || _h === void 0 ? void 0 : _h.values).toBe(true);
    });
});
//# sourceMappingURL=build-raw-project-configuration.spec.js.map