"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const changes_stream_1 = require("../../../src/datastore/changes/changes-stream");
/**
 * @author Thomas Kleinke
 * @author Daniel de Oliveira
 */
describe('ChangesStream', () => {
    let rcs;
    let doc;
    let pouchdbDatastore;
    let datastore;
    let indexFacade;
    let documentConverter;
    let documentCache;
    let projectConfiguration;
    let getUsername;
    let onChange;
    beforeEach(() => {
        doc = {
            resource: {
                id: 'id1',
                identifier: '1',
                category: 'Object',
                relations: {}
            },
            created: {
                user: 'remoteuser',
                date: new Date('2018-01-01T01:00:00.00Z')
            },
            modified: [
                {
                    user: 'remoteuser',
                    date: new Date('2018-01-02T07:00:00.00Z')
                }
            ]
        };
        spyOn(console, 'warn'); // suppress console.warn
        indexFacade = jasmine.createSpyObj('MockIndexFacade', ['put', 'putToSingleIndex', 'get', 'remove', 'getCount', 'notifyObservers']);
        documentConverter = jasmine.createSpyObj('MockDocumentConverter', ['convert']);
        documentCache = jasmine.createSpyObj('MockDocumentCache', ['get', 'reassign']);
        documentCache.reassign.and.callFake(document => document);
        getUsername = () => 'localuser';
        documentConverter.convert.and.returnValue(doc);
        indexFacade.getCount.and.returnValue(0);
        documentCache.get.and.returnValue({ resource: { id: '1', identifier: '1' } });
        pouchdbDatastore = jasmine.createSpyObj('MockPouchdbDatastore', ['changesNotifications', 'deletedNotifications', 'fetch', 'fetchRevision']);
        pouchdbDatastore.fetch.and.returnValue(Promise.resolve(doc));
        pouchdbDatastore.changesNotifications.and.returnValue({ subscribe: (func) => onChange = func });
        pouchdbDatastore.deletedNotifications.and.returnValue({ subscribe: (func) => undefined });
        datastore = jasmine.createSpyObj('MockDatastore', ['find']);
        datastore.find.and.returnValue(Promise.resolve({ documents: [] }));
        projectConfiguration = jasmine.createSpyObj(['MockProjectConfiguration'], ['getCategory', 'getRegularCategories', 'getCategoryWithSubcategories']);
        projectConfiguration.getCategory.and.returnValue({ name: 'Object', groups: [] });
        projectConfiguration.getRegularCategories.and.returnValue([]);
        projectConfiguration.getCategoryWithSubcategories.and.returnValue([{ name: 'Object', groups: [] }]);
        rcs = new changes_stream_1.ChangesStream(pouchdbDatastore, datastore, indexFacade, documentCache, documentConverter, projectConfiguration, getUsername);
    });
    it('should put to index facade and reassign to cache', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield onChange(doc);
        expect(indexFacade.put).toHaveBeenCalledWith(doc);
        expect(documentCache.reassign).toHaveBeenCalledWith(doc);
        done();
    }));
    it('send through category converter', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield onChange(doc);
        expect(documentConverter.convert).toHaveBeenCalledWith(doc);
        done();
    }));
    it('detect that it is remote change', (done) => __awaiter(void 0, void 0, void 0, function* () {
        documentConverter.convert.and.returnValue(doc);
        yield onChange(doc);
        expect(indexFacade.put).toHaveBeenCalledWith(doc);
        done();
    }));
    it('detect that it is local change', (done) => __awaiter(void 0, void 0, void 0, function* () {
        doc.modified[0] = {
            user: 'localuser',
            date: new Date('2018-02-08T01:00:00.00Z')
        };
        documentConverter.convert.and.returnValue(doc);
        yield onChange(doc);
        expect(indexFacade.put).not.toHaveBeenCalled();
        done();
    }));
    it('detect remote change for conflicted document', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const rev2 = {
            resource: {
                id: 'id1',
                category: 'Object',
                relations: {}
            },
            created: {
                user: 'remoteuser',
                date: new Date('2018-01-01T01:00:00.00Z')
            },
            modified: [
                {
                    user: 'remoteuser2',
                    date: new Date('2018-01-02T15:00:00.00Z')
                }
            ]
        };
        pouchdbDatastore.fetch.and.returnValue(Promise.resolve({ '_conflicts': ['first'], resource: { id: '1' } }));
        pouchdbDatastore.fetchRevision.and.returnValue(Promise.resolve(rev2));
        documentConverter.convert.and.returnValue(doc);
        yield onChange(doc);
        expect(indexFacade.put).toHaveBeenCalledWith(doc);
        done();
    }));
    it('always treat changes to conflicted documents as remote changes', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const rev2 = {
            resource: {
                id: 'id1',
                category: 'Object',
                relations: {}
            },
            created: {
                user: 'remoteuser',
                date: new Date('2018-01-01T01:00:00.00Z')
            },
            modified: [
                {
                    user: 'localuser',
                    date: new Date('2018-01-02T15:00:00.00Z')
                }
            ]
        };
        pouchdbDatastore.fetch.and.returnValue(Promise.resolve({ '_conflicts': ['first'], resource: { id: '1' } }));
        pouchdbDatastore.fetchRevision.and.returnValue(Promise.resolve(rev2));
        documentConverter.convert.and.returnValue(doc);
        yield onChange(doc);
        expect(indexFacade.put).toHaveBeenCalled();
        done();
    }));
    it('update non-unique identifier warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        let doc2 = {
            resource: {
                id: 'id2', identifier: '1', category: 'Object', relations: {}
            },
            created: {
                user: 'remoteuser',
                date: new Date('2018-01-01T01:00:00.00Z')
            },
            modified: [
                {
                    user: 'remoteuser',
                    date: new Date('2018-01-02T07:00:00.00Z')
                }
            ]
        };
        indexFacade.getCount.and.returnValue(2);
        datastore.find.and.returnValue(Promise.resolve({ documents: [doc2] }));
        documentCache.reassign.and.callFake(document => {
            if (document.id === 'id1') {
                doc = document;
                return doc;
            }
            else {
                doc2 = document;
                return doc2;
            }
        });
        yield onChange(doc);
        expect(doc.warnings.nonUniqueIdentifier).toBe(true);
        expect(doc2.warnings.nonUniqueIdentifier).toBe(true);
        doc2.resource.identifier = '2';
        documentCache.get.and.returnValue({ resource: { id: 'id2', identifier: '1' } });
        indexFacade.getCount.and.returnValue(1);
        datastore.find.and.returnValue(Promise.resolve({ documents: [doc] }));
        yield onChange(doc2);
        expect(doc.warnings).toBeUndefined();
        expect(doc2.warnings).toBeUndefined();
        done();
    }));
});
//# sourceMappingURL=changes-stream.spec.js.map