"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const document_converter_1 = require("../../src/datastore/document-converter");
const datastore_1 = require("../../src/datastore/datastore");
const document_cache_1 = require("../../src/datastore/document-cache");
const project_configuration_1 = require("../../src/services/project-configuration");
const test_helpers_1 = require("../test-helpers");
/**
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
describe('Datastore', () => {
    let datastore;
    let mockdb;
    let mockIndexFacade;
    function createMockedDatastore(mockdb) {
        const forms = [test_helpers_1.createCategory('Find')];
        const projectConfiguration = new project_configuration_1.ProjectConfiguration({
            forms, categories: {}, relations: [], commonFields: {}, valuelists: {}, projectLanguages: []
        });
        const documentCache = new document_cache_1.DocumentCache();
        return new datastore_1.Datastore(mockdb, mockIndexFacade, documentCache, new document_converter_1.DocumentConverter(projectConfiguration), projectConfiguration, () => 'username');
    }
    function verifyIsDocument(document) {
        expect(document.resource.identifier).toEqual('');
    }
    beforeEach(() => {
        mockdb = jasmine.createSpyObj('mockdb', ['create', 'update', 'fetch', 'bulkFetch', 'fetchRevision']);
        mockdb.create.and.callFake(function (document) {
            return Promise.resolve(document);
        });
        mockdb.update.and.callFake(function (document) {
            // working with the current assumption that the inner pouchdbdatastore datastore returns the same instance
            document.resource.id = '1';
            document['_rev'] = '2';
            return Promise.resolve(document);
        });
        mockdb.bulkFetch.and.returnValue(new Promise(resolve => resolve([])));
        mockIndexFacade = jasmine.createSpyObj('mockIndexFacade', ['find', 'put', 'remove', 'getCount', 'putToSingleIndex', 'notifyObservers']);
        mockIndexFacade.find.and.callFake(() => ['1']);
        mockIndexFacade.put.and.callFake(function (document) {
            return Promise.resolve(document);
        });
        mockIndexFacade.getCount.and.returnValue(0);
        datastore = createMockedDatastore(mockdb);
    });
    // get
    it('should add missing fields on get, bypassing cache', (done) => __awaiter(void 0, void 0, void 0, function* () {
        mockdb.fetch.and.returnValue(Promise.resolve({
            resource: {
                id: '1',
                category: 'Find',
                relations: {}
            }
        }));
        const document = yield datastore.get('1'); // fetch from mockdb
        verifyIsDocument(document);
        done();
    }));
    // get multiple
    it('should add missing fields on getMultiple, bypassing cache', (done) => __awaiter(void 0, void 0, void 0, function* () {
        mockdb.bulkFetch.and.returnValue(Promise.resolve([
            {
                resource: {
                    id: '1',
                    category: 'Find',
                    relations: {}
                }
            }, {
                resource: {
                    id: '2',
                    category: 'Find',
                    relations: {}
                }
            }
        ]));
        const documents = yield datastore.getMultiple(['1', '2']);
        expect(documents.length).toBe(2);
        verifyIsDocument(documents[0]);
        verifyIsDocument(documents[1]);
        done();
    }));
    it('should retain order when fetching documents from both cache and datastore on getMultiple', (done) => __awaiter(void 0, void 0, void 0, function* () {
        mockdb.fetch.and.returnValue(Promise.resolve({
            resource: {
                id: '2',
                category: 'Find',
                relations: {}
            }
        }));
        mockdb.bulkFetch.and.returnValue(Promise.resolve([
            {
                resource: {
                    id: '1',
                    category: 'Find',
                    relations: {}
                }
            }, {
                resource: {
                    id: '3',
                    category: 'Find',
                    relations: {}
                }
            }
        ]));
        yield datastore.get('2'); // Save in cache
        const documents = yield datastore.getMultiple(['1', '2', '3']);
        expect(documents.length).toBe(3);
        expect(documents[0].resource.id).toEqual('1');
        expect(documents[1].resource.id).toEqual('2');
        expect(documents[2].resource.id).toEqual('3');
        done();
    }));
    // getRevision
    it('should add missing fields on getRevision (bypassing cache)', (done) => __awaiter(void 0, void 0, void 0, function* () {
        mockdb.fetchRevision.and.returnValue(Promise.resolve({
            resource: {
                id: '1',
                category: 'Find',
                relations: {}
            }
        }));
        const document = yield datastore.getRevision('1', '1'); // fetch from mockdb
        verifyIsDocument(document);
        done();
    }));
    // find
    it('should add missing fields on find, bypassing cache', (done) => __awaiter(void 0, void 0, void 0, function* () {
        mockIndexFacade.find.and.returnValue(['1']);
        mockdb.bulkFetch.and.returnValue(Promise.resolve([
            {
                resource: {
                    id: '1',
                    category: 'Find',
                    relations: {}
                }
            }
        ]));
        const documents = (yield datastore.find({})).documents; // fetch from mockdb
        expect(documents.length).toBe(1);
        verifyIsDocument(documents[0]);
        done();
    }));
    it('should add missing fields on find', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield datastore.create({
            resource: {
                id: '1',
                category: 'Find',
                relations: {}
            }
        });
        mockIndexFacade.find.and.returnValue(['1']);
        const documents = (yield datastore.find({})).documents; // fetch from cache
        expect(documents.length).toBe(1);
        verifyIsDocument(documents[0]);
        done();
    }));
    it('should limit the number of documents returned on find', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield datastore.create({ resource: { id: '1', category: 'Find', relations: {} } });
        yield datastore.create({ resource: { id: '2', category: 'Find', relations: {} } });
        mockIndexFacade.find.and.returnValue(['1', '2']);
        const { documents, totalCount } = yield datastore.find({ limit: 1 });
        expect(documents.length).toBe(1);
        expect(totalCount).toBe(2);
        verifyIsDocument(documents[0]);
        done();
    }));
    it('limit the number of documents and use an offset', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield datastore.create({ resource: { id: '1', category: 'Find', relations: {} } });
        yield datastore.create({ resource: { id: '2', category: 'Find', relations: {} } });
        yield datastore.create({ resource: { id: '3', category: 'Find', relations: {} } });
        mockIndexFacade.find.and.returnValue(['3', '1', '2']);
        const { documents, totalCount } = yield datastore.find({ limit: 1, offset: 1 });
        expect(documents.length).toBe(1);
        expect(totalCount).toBe(3);
        verifyIsDocument(documents[0]);
        expect(documents[0].resource.id).toBe('1');
        done();
    }));
    it('offset excludes everything', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield datastore.create({ resource: { id: '1', category: 'Find', relations: {} } });
        yield datastore.create({ resource: { id: '2', category: 'Find', relations: {} } });
        yield datastore.create({ resource: { id: '3', category: 'Find', relations: {} } });
        mockIndexFacade.find.and.returnValue([
            { id: '1', identifier: 'eins' },
            { id: '2', identifier: 'zwei' },
            { id: '3', identifier: 'drei' }
        ]);
        const { documents, totalCount } = yield datastore.find({ offset: 3 });
        expect(documents.length).toBe(0);
        expect(totalCount).toBe(3);
        done();
    }));
    it('cant find one and only document', (done) => __awaiter(void 0, void 0, void 0, function* () {
        mockIndexFacade.find.and.returnValue(['1']);
        mockdb.bulkFetch.and.returnValue(Promise.resolve([]));
        const { documents, totalCount } = yield datastore.find({});
        expect(documents.length).toBe(0);
        expect(totalCount).toBe(0);
        done();
    }));
    it('cant find second document', (done) => __awaiter(void 0, void 0, void 0, function* () {
        mockIndexFacade.find.and.returnValue(['1', '2']);
        mockdb.bulkFetch.and.returnValues(Promise.resolve([
            {
                resource: {
                    id: '1',
                    category: 'Find',
                    relations: {}
                }
            }
        ]), Promise.resolve([]), Promise.resolve([]), Promise.resolve([]));
        const { documents, totalCount } = yield datastore.find({});
        expect(documents.length).toBe(1);
        expect(totalCount).toBe(1);
        done();
    }));
    // find ids
    it('should return only ids', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield datastore.create({ resource: { id: '1', category: 'Find', relations: {} } });
        yield datastore.create({ resource: { id: '2', category: 'Find', relations: {} } });
        mockIndexFacade.find.and.returnValue(['1', '2']);
        const result = datastore.findIds({});
        expect(result.ids.length).toBe(2);
        expect(result.totalCount).toBe(2);
        expect(result['documents']).toBe(undefined);
        done();
    }));
    // update
    it('should add missing fields on update', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield datastore.update({ resource: {
                id: '1',
                category: 'Find',
                relations: {}
            } });
        const document = yield datastore.get('1'); // fetch from cache
        verifyIsDocument(document);
        done();
    }));
    it('should add missing fields on update with reassign', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield datastore.update({ resource: {
                id: '1',
                category: 'Find',
                val: 'a',
                relations: {}
            } });
        yield datastore.update({ resource: {
                id: '1',
                category: 'Find',
                val: 'b',
                relations: {}
            } });
        const document = yield datastore.get('1'); // fetch from cache
        expect(document.resource['val']).toEqual('b');
        verifyIsDocument(document);
        done();
    }));
    // create
    it('should add missing fields on create', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield datastore.create({ resource: {
                id: '1',
                category: 'Find',
                relations: {}
            } });
        const document = yield datastore.get('1'); // fetch from cache
        verifyIsDocument(document);
        done();
    }));
    it('should return the cached instance on create', (done) => __awaiter(void 0, void 0, void 0, function* () {
        let document1 = test_helpers_1.doc('sd1', 'identifier1');
        mockdb.create.and.callFake(function (document) {
            // working with the current assumption that the inner pouchdbdatastore datastore returns the same instance
            document.resource.id = '1';
            return Promise.resolve(document);
        });
        yield datastore.create(document1);
        try {
            const documents = (yield datastore.find({ q: 'sd1' })).documents; // mockdb returns other instance
            expect((documents[0]).resource['identifier']).toBe('identifier1');
            document1.resource['shortDescription'] = 's4';
            expect((documents[0]).resource['shortDescription']).toBe('s4');
        }
        catch (error) {
            fail(error);
        }
        done();
    }));
    it('should return cached instance on update', (done) => __awaiter(void 0, void 0, void 0, function* () {
        let document1 = test_helpers_1.doc('sd1', 'identifier1');
        let document2;
        yield datastore.create(document1);
        document2 = test_helpers_1.doc('sd1', 'identifier_');
        document2.resource.id = '1';
        yield datastore.update(document2);
        const result = yield datastore.find({ q: 'sd1' }); // mockdb returns other instance
        expect((result.documents[0])['_rev']).toBe('2');
        expect((result.documents[0]).resource['identifier']).toBe('identifier_');
        document2.resource['shortDescription'] = 's4';
        expect((result.documents[0]).resource['shortDescription']).toBe('s4');
        done();
    }));
});
//# sourceMappingURL=datastore.spec.js.map