"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const datastore_errors_1 = require("../../../src/datastore/datastore-errors");
const pouchdb_datastore_1 = require("../../../src/datastore/pouchdb/pouchdb-datastore");
const test_helpers_1 = require("../../test-helpers");
/**
 * @author Daniel de Oliveira
 * @author Sebastian Cuy
 * @author Thomas Kleinke
 */
describe('PouchdbDatastore', () => {
    let datastore;
    let pouchdbProxy;
    let res;
    beforeEach(() => {
        let idGenerator = jasmine.createSpyObj('idGenerator', ['generateId']);
        idGenerator.generateId.and.returnValues(1, 2);
        pouchdbProxy = jasmine.createSpyObj('pouchdbProxy', ['get', 'put', 'bulkDocs', 'remove', 'allDocs']);
        pouchdbProxy.put.and.callFake((doc) => __awaiter(void 0, void 0, void 0, function* () { return res = doc; }));
        pouchdbProxy.get.and.callFake(() => __awaiter(void 0, void 0, void 0, function* () { return res; }));
        pouchdbProxy.remove.and.returnValue(Promise.resolve(undefined));
        pouchdbProxy.bulkDocs.and.callFake((documents) => __awaiter(void 0, void 0, void 0, function* () { return res = documents; }));
        pouchdbProxy.allDocs.and.callFake(() => Promise.resolve({
            rows: res ? res.map(document => { return { doc: document }; }) : []
        }));
        datastore = new pouchdb_datastore_1.PouchdbDatastore(_ => pouchdbProxy, idGenerator);
        datastore.createDbForTesting('test');
    });
    it('create: create an id', (done) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const result = yield datastore.create(test_helpers_1.doc('sd1'), 'u');
            expect(result.resource.id).toBe(1);
        }
        catch (e) {
            fail(e);
        }
        done();
    }));
    it('create: should create a document and take the existing resource.id', (done) => __awaiter(void 0, void 0, void 0, function* () {
        let called = false;
        pouchdbProxy.get.and.callFake((doc) => __awaiter(void 0, void 0, void 0, function* () {
            if (!called) {
                called = true;
                throw undefined;
            }
            return {
                resource: {
                    id: doc, category: 'some', relations: []
                },
                created: { date: '2011/01/01' },
                modified: [{ date: '2011/01/01' }]
            };
        }));
        const docToCreate = test_helpers_1.doc('sd1');
        docToCreate.resource.id = 'a1';
        yield datastore.create(docToCreate, 'u');
        // this step was added to address a problem where a document
        // with an existing resource.id was stored but could not
        // get refreshed later
        yield datastore.fetch(docToCreate.resource.id);
        // and the same may occur on get
        try {
            const createdDoc = yield datastore.fetch(docToCreate.resource.id);
            expect(createdDoc.resource.id).toBe('a1');
        }
        catch (e) {
            fail(e);
        }
        done();
    }));
    it('create: should not create a document with the resource.id of an already existing doc', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const docToCreate1 = test_helpers_1.doc('sd1');
        docToCreate1.resource.id = 'a1';
        const docToCreate2 = test_helpers_1.doc('sd1');
        docToCreate2.resource.id = 'a1';
        try {
            yield datastore.create(docToCreate1, 'u');
            yield datastore.create(docToCreate2, 'u');
            fail();
        }
        catch (expected) {
            expect(expected[0]).toEqual(datastore_errors_1.DatastoreErrors.DOCUMENT_RESOURCE_ID_EXISTS);
        }
        done();
    }));
    it('create: create dates', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const newDoc = test_helpers_1.doc('sd1');
        delete newDoc.created;
        const result = yield datastore.create(newDoc, 'u');
        expect(result.created.user).toEqual('u');
        expect(result.created.date instanceof Date).toBeTruthy();
        expect(Array.isArray(result.modified)).toBeTruthy();
        expect(result.modified.length).toBe(0);
        done();
    }));
    it('bulkCreate: create ids', (done) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const result = yield datastore.bulkCreate([test_helpers_1.doc('sd1'), test_helpers_1.doc('sd2')], 'u');
            expect(result[0].resource.id).toBe(1);
            expect(result[1].resource.id).toBe(2);
        }
        catch (e) {
            fail(e);
        }
        done();
    }));
    it('bulkCreate: should not create a document with the resource.id of an already existing doc', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const docToCreate1 = test_helpers_1.doc('sd1');
        docToCreate1.resource.id = 'a1';
        const docToCreate2 = test_helpers_1.doc('sd1');
        docToCreate2.resource.id = 'a1';
        try {
            yield datastore.bulkCreate([docToCreate1], 'u');
            yield datastore.bulkCreate([docToCreate2], 'u');
            fail();
        }
        catch (expected) {
            expect(expected[0]).toEqual(datastore_errors_1.DatastoreErrors.DOCUMENT_RESOURCE_ID_EXISTS);
        }
        done();
    }));
    it('update: should update an existing document with no identifier conflict', (done) => __awaiter(void 0, void 0, void 0, function* () {
        let doc2 = test_helpers_1.doc('id2');
        yield datastore.create(test_helpers_1.doc('id1'), 'u');
        doc2 = yield datastore.create(doc2, 'u');
        yield datastore.update(doc2, 'u');
        done();
    }));
    it('update: should not update if resource id not present', (done) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            yield datastore.update(test_helpers_1.doc('sd1'), 'u');
            fail();
        }
        catch (expected) {
            expect(expected[0]).toBe(datastore_errors_1.DatastoreErrors.DOCUMENT_NO_RESOURCE_ID);
        }
        done();
    }));
    it('update: should not update if created not present', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const newDoc = test_helpers_1.doc('sd1');
        newDoc.resource.id = '1';
        delete newDoc.created;
        try {
            yield datastore.update(newDoc, 'u');
            fail();
        }
        catch (expected) {
            expect(expected[0]).toBe(datastore_errors_1.DatastoreErrors.INVALID_DOCUMENT);
        }
        done();
    }));
    it('update: should not update if not existent', (done) => __awaiter(void 0, void 0, void 0, function* () {
        pouchdbProxy.get.and.returnValue(Promise.reject(undefined));
        try {
            yield datastore.update(test_helpers_1.doc('sd1', 'identifier1', 'Find', 'id1'), 'u');
            fail();
        }
        catch (expectedErr) {
            expect(expectedErr[0]).toBe(datastore_errors_1.DatastoreErrors.DOCUMENT_NOT_FOUND);
        }
        done();
    }));
    it('update: should squash old revisions', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const newDoc = test_helpers_1.doc('sd1');
        newDoc.resource.id = '1';
        const rev = test_helpers_1.doc('sd1');
        rev['_rev'] = 'r-1';
        try {
            yield datastore.update(newDoc, 'u3', ['r-1']);
            expect(pouchdbProxy.remove).toHaveBeenCalledWith('1', 'r-1');
        }
        catch (e) {
            fail(e);
        }
        done();
    }));
    it('update: should merge modified dates of squash revisions', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const newDoc = test_helpers_1.doc('sd1');
        newDoc.resource.id = '1';
        let i = 0;
        pouchdbProxy.get.and.callFake((resourceId, params) => __awaiter(void 0, void 0, void 0, function* () {
            if (params && params.rev) {
                const rev = test_helpers_1.doc('sd1');
                rev.modified[0] = { user: 'u2', date: new Date('2018-04-27T11:07:05.760Z') };
                rev.created = rev.modified[0];
                rev.resource.id = '1';
                rev['_rev'] = 'r-1';
                return rev;
            }
            else if (i === 0) {
                i = i + 1;
                const existingDoc = test_helpers_1.doc('sd1');
                existingDoc.created = { user: 'u1', date: new Date('2018-04-26T11:07:05.760Z') };
                existingDoc.modified = [];
                existingDoc.resource.id = '1';
                return existingDoc;
            }
            else {
                return res;
            }
        }));
        const result = yield datastore.update(newDoc, 'u3', ['r-1']);
        expect(result.created.user).toEqual('u1');
        expect(result.modified.length).toBe(2);
        expect(result.modified[0].user).toEqual('u2');
        expect(result.modified[1].user).toEqual('u3');
        done();
    }));
    it('update: add modified date', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const newDoc = test_helpers_1.doc('id2');
        newDoc.resource.id = '1';
        let i = 0;
        pouchdbProxy.get.and.callFake(() => __awaiter(void 0, void 0, void 0, function* () {
            if (i === 0) {
                i = i + 1;
                const existingDoc = test_helpers_1.doc('sd1');
                existingDoc.created = { user: 'u1', date: new Date('2018-04-26T11:07:05.760Z') };
                existingDoc.modified = [];
                existingDoc.resource.id = '1';
                return existingDoc;
            }
            else {
                return res;
            }
        }));
        const result = yield datastore.update(newDoc, 'u');
        expect(result.modified.length).toBe(1);
        expect(result.modified[0].user).toEqual('u');
        expect(result.modified[0].date instanceof Date).toBeTruthy();
        done();
    }));
    it('bulkUpdate: should not update if resource id not present', (done) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            yield datastore.bulkUpdate([test_helpers_1.doc('sd1')], 'u');
            fail();
        }
        catch (expected) {
            expect(expected[0]).toBe(datastore_errors_1.DatastoreErrors.DOCUMENT_NO_RESOURCE_ID);
        }
        done();
    }));
    it('get: should get if existent', (done) => __awaiter(void 0, void 0, void 0, function* () {
        pouchdbProxy.get.and.callFake((resourceId) => __awaiter(void 0, void 0, void 0, function* () {
            return { resource: {
                    id: resourceId, category: 'some', relations: []
                }, created: { date: '2011/01/01' },
                modified: [{ date: '2011/01/01' }]
            };
        }));
        try {
            const result = yield datastore.fetch('id5');
            expect(result.resource.id).toBe('id5');
        }
        catch (e) {
            fail(e);
        }
        done();
    }));
    it('get: should reject with keyOfM in when trying to get a non existing document', (done) => __awaiter(void 0, void 0, void 0, function* () {
        pouchdbProxy.get.and.returnValue(Promise.reject(undefined));
        try {
            yield datastore.fetch('nonexisting');
            fail();
        }
        catch (expected) {
            expect(expected[0]).toEqual(datastore_errors_1.DatastoreErrors.DOCUMENT_NOT_FOUND);
        }
        done();
    }));
    it('remove: should remove if existent', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield datastore.remove(test_helpers_1.doc('sd1', 'identifier1', 'Find', 'id1'));
        expect(pouchdbProxy.remove).toHaveBeenCalled();
        done();
    }));
    it('remove: should throw when no resource id', (done) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            yield datastore.remove(test_helpers_1.doc('sd2'));
            fail();
        }
        catch (expected) {
            expect(expected[0]).toEqual(datastore_errors_1.DatastoreErrors.DOCUMENT_NO_RESOURCE_ID);
        }
        done();
    }));
    it('remove: should throw when trying to remove and not existent', (done) => __awaiter(void 0, void 0, void 0, function* () {
        pouchdbProxy.get.and.returnValue(Promise.reject(undefined));
        const d = test_helpers_1.doc('sd1');
        d['resource']['id'] = 'hoax';
        try {
            yield datastore.remove(d);
            fail();
        }
        catch (expected) {
            expect(expected[0]).toEqual(datastore_errors_1.DatastoreErrors.DOCUMENT_NOT_FOUND);
        }
        done();
    }));
    it('remove: remove all conflicting revisions', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const document = test_helpers_1.doc('shortDescription1', 'identifier1', 'Find', 'id1');
        document['_conflicts'] = ['revision1', 'revision2'];
        pouchdbProxy.get.and.returnValue(Promise.resolve(document));
        yield datastore.remove(document);
        expect(pouchdbProxy.remove).toHaveBeenCalledWith(document);
        expect(pouchdbProxy.remove).toHaveBeenCalledWith('id1', 'revision1');
        expect(pouchdbProxy.remove).toHaveBeenCalledWith('id1', 'revision2');
        done();
    }));
});
//# sourceMappingURL=pouchdb-datastore.spec.js.map