"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const warnings_updater_1 = require("../../src/datastore/warnings-updater");
const warnings_1 = require("../../src/model/warnings");
const field_1 = require("../../src/model/configuration/field");
const test_helpers_1 = require("../test-helpers");
const createDocument = (id, category = 'Category') => test_helpers_1.doc('sd', 'identifier' + id, category, id);
function getMockProjectConfiguration(categoryDefinition, parentCategoryDefinition) {
    const mockProjectConfiguration = jasmine.createSpyObj('projectConfiguration', ['getCategory', 'getCategories', 'getCategoryWithSubcategories', 'isAllowedRelationDomainCategory',
        'getRegularCategories']);
    mockProjectConfiguration.getCategory.and.callFake(categoryName => {
        if (categoryName === 'Category')
            return categoryDefinition;
        if (categoryName === 'ParentCategory')
            return parentCategoryDefinition;
    });
    if (parentCategoryDefinition) {
        mockProjectConfiguration.getCategories.and.returnValue([{
                item: parentCategoryDefinition, trees: [
                    { item: categoryDefinition, trees: [] }
                ]
            }
        ]);
    }
    else {
        mockProjectConfiguration.getCategories.and.returnValue([{ item: categoryDefinition, trees: [] }]);
    }
    mockProjectConfiguration.getCategoryWithSubcategories.and.callFake(categoryName => {
        if (categoryName === 'Category')
            return [categoryDefinition];
        if (categoryName === 'ParentCategory' && parentCategoryDefinition) {
            return [parentCategoryDefinition, categoryDefinition];
        }
        return [];
    });
    mockProjectConfiguration.isAllowedRelationDomainCategory.and.returnValue(true);
    mockProjectConfiguration.getRegularCategories.and.returnValue([categoryDefinition]);
    return mockProjectConfiguration;
}
function getMockIndexFacade() {
    return jasmine.createSpyObj('mockIndexFacade', ['putToSingleIndex', 'getCount', 'notifyObservers', 'find']);
}
/**
 * @author Thomas Kleinke
 */
describe('WarningsUpdater', () => {
    it('update index independent warnings', () => {
        const categoryDefinition = {
            name: 'Category',
            identifierPrefix: 'C',
            groups: [
                {
                    fields: [
                        {
                            name: 'shortDescription',
                            inputType: field_1.Field.InputType.INPUT
                        },
                        {
                            name: 'number',
                            inputType: field_1.Field.InputType.FLOAT
                        }
                    ]
                }
            ]
        };
        const documents = [
            createDocument('1'),
            createDocument('2'),
            createDocument('3')
        ];
        documents[0]._conflicts = ['123'];
        documents[0].resource.identifier = '1';
        documents[0].resource.number = 'text';
        documents[0].resource.unconfiguredField = 'text';
        documents[1].resource.identifier = 'C2';
        delete documents[1].resource.category;
        documents[2].resource.identifier = 'C3';
        documents[2].resource.number = 1;
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition);
        warnings_updater_1.WarningsUpdater.updateIndexIndependentWarnings(documents[0], mockProjectConfiguration);
        warnings_updater_1.WarningsUpdater.updateIndexIndependentWarnings(documents[1], mockProjectConfiguration);
        warnings_updater_1.WarningsUpdater.updateIndexIndependentWarnings(documents[2], mockProjectConfiguration);
        expect(documents[0].warnings).toEqual({
            unconfiguredFields: ['unconfiguredField'],
            invalidFields: ['number'],
            conflicts: true,
            missingIdentifierPrefix: true
        });
        expect(documents[1].warnings).toEqual({
            unconfiguredFields: [],
            invalidFields: [],
            unconfiguredCategory: true
        });
        expect(documents[2].warnings).toBeUndefined();
    });
    it('set non-unique identifier warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b;
        const documents = [
            createDocument('1'),
            createDocument('2')
        ];
        const mockIndexFacade = getMockIndexFacade();
        mockIndexFacade.getCount.and.returnValue(2);
        const mockDatastore = jasmine.createSpyObj('mockDatastore', ['find']);
        mockDatastore.find.and.returnValue(Promise.resolve({ documents: [documents[1]] }));
        yield warnings_updater_1.WarningsUpdater.updateNonUniqueIdentifierWarning(documents[0], mockIndexFacade, mockDatastore, undefined, true);
        expect((_a = documents[0].warnings) === null || _a === void 0 ? void 0 : _a.nonUniqueIdentifier).toBe(true);
        expect((_b = documents[1].warnings) === null || _b === void 0 ? void 0 : _b.nonUniqueIdentifier).toBe(true);
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'nonUniqueIdentifier:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'nonUniqueIdentifier:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'warnings:exist');
        done();
    }));
    it('remove non-unique identifier warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const documents = [
            createDocument('1'),
            createDocument('2')
        ];
        documents[0].warnings = warnings_1.Warnings.createDefault();
        documents[0].warnings.nonUniqueIdentifier = true;
        documents[1].warnings = warnings_1.Warnings.createDefault();
        documents[1].warnings.nonUniqueIdentifier = true;
        const mockIndexFacade = getMockIndexFacade();
        mockIndexFacade.getCount.and.returnValue(1);
        const mockDatastore = jasmine.createSpyObj('mockDatastore', ['find']);
        mockDatastore.find.and.returnValue(Promise.resolve({ documents: [documents[1]] }));
        yield warnings_updater_1.WarningsUpdater.updateNonUniqueIdentifierWarning(documents[0], mockIndexFacade, mockDatastore, 'previousIdentifier', true);
        expect(documents[0].warnings).toBeUndefined();
        expect(documents[1].warnings).toBeUndefined();
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'nonUniqueIdentifier:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'nonUniqueIdentifier:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'warnings:exist');
        done();
    }));
    it('set resource limit warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _c, _d;
        const categoryDefinition = {
            name: 'Category',
            resourceLimit: 1
        };
        const documents = [
            createDocument('1'),
            createDocument('2')
        ];
        const mockIndexFacade = getMockIndexFacade();
        mockIndexFacade.find.and.returnValue(['1', '2']);
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition);
        const mockDatastore = jasmine.createSpyObj('mockDatastore', ['find']);
        mockDatastore.find.and.returnValue(Promise.resolve({ documents: [documents[0], documents[1]] }));
        yield warnings_updater_1.WarningsUpdater.updateResourceLimitWarning(documents[0], categoryDefinition, mockIndexFacade, mockProjectConfiguration, mockDatastore, true);
        expect((_c = documents[0].warnings) === null || _c === void 0 ? void 0 : _c.resourceLimitExceeded).toBe(true);
        expect((_d = documents[1].warnings) === null || _d === void 0 ? void 0 : _d.resourceLimitExceeded).toBe(true);
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'resourceLimitExceeded:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'resourceLimitExceeded:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'warnings:exist');
        done();
    }));
    it('remove resource limit warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const categoryDefinition = {
            name: 'Category',
            resourceLimit: 2
        };
        const documents = [
            createDocument('1'),
            createDocument('2')
        ];
        documents[0].warnings = warnings_1.Warnings.createDefault();
        documents[0].warnings.resourceLimitExceeded = true;
        documents[1].warnings = warnings_1.Warnings.createDefault();
        documents[1].warnings.resourceLimitExceeded = true;
        const mockIndexFacade = getMockIndexFacade();
        mockIndexFacade.find.and.returnValue(['1', '2']);
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition);
        const mockDatastore = jasmine.createSpyObj('mockDatastore', ['find']);
        mockDatastore.find.and.returnValue(Promise.resolve({ documents: [documents[0], documents[1]] }));
        yield warnings_updater_1.WarningsUpdater.updateResourceLimitWarnings(mockDatastore, mockIndexFacade, mockProjectConfiguration, categoryDefinition);
        expect(documents[0].warnings).toBeUndefined();
        expect(documents[1].warnings).toBeUndefined();
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'resourceLimitExceeded:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'resourceLimitExceeded:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'warnings:exist');
        done();
    }));
    it('set resource limit warnings for limit in parent category', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _e, _f;
        const parentCategoryDefinition = {
            name: 'ParentCategory',
            resourceLimit: 1
        };
        const categoryDefinition = {
            name: 'Category',
            parentCategory: parentCategoryDefinition
        };
        const documents = [
            createDocument('1'),
            createDocument('2')
        ];
        const mockIndexFacade = getMockIndexFacade();
        mockIndexFacade.find.and.returnValue(['1', '2']);
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition, parentCategoryDefinition);
        const mockDatastore = jasmine.createSpyObj('mockDatastore', ['find']);
        mockDatastore.find.and.returnValue(Promise.resolve({ documents: [documents[0], documents[1]] }));
        yield warnings_updater_1.WarningsUpdater.updateResourceLimitWarning(documents[0], categoryDefinition, mockIndexFacade, mockProjectConfiguration, mockDatastore, true);
        expect((_e = documents[0].warnings) === null || _e === void 0 ? void 0 : _e.resourceLimitExceeded).toBe(true);
        expect((_f = documents[1].warnings) === null || _f === void 0 ? void 0 : _f.resourceLimitExceeded).toBe(true);
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'resourceLimitExceeded:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'resourceLimitExceeded:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'warnings:exist');
        done();
    }));
    it('remove resource limit warnings for limit in parent category', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const parentCategoryDefinition = {
            name: 'ParentCategory',
            resourceLimit: 2
        };
        const categoryDefinition = {
            name: 'Category',
            parentCategory: parentCategoryDefinition
        };
        const documents = [
            createDocument('1'),
            createDocument('2')
        ];
        documents[0].warnings = warnings_1.Warnings.createDefault();
        documents[0].warnings.resourceLimitExceeded = true;
        documents[1].warnings = warnings_1.Warnings.createDefault();
        documents[1].warnings.resourceLimitExceeded = true;
        const mockIndexFacade = getMockIndexFacade();
        mockIndexFacade.find.and.returnValue(['1', '2']);
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition, parentCategoryDefinition);
        const mockDatastore = jasmine.createSpyObj('mockDatastore', ['find']);
        mockDatastore.find.and.returnValue(Promise.resolve({ documents: [documents[0], documents[1]] }));
        yield warnings_updater_1.WarningsUpdater.updateResourceLimitWarnings(mockDatastore, mockIndexFacade, mockProjectConfiguration, categoryDefinition);
        expect(documents[0].warnings).toBeUndefined();
        expect(documents[1].warnings).toBeUndefined();
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'resourceLimitExceeded:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'resourceLimitExceeded:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'warnings:exist');
        done();
    }));
    it('set relation target warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _g;
        const documents = [
            createDocument('1'),
            createDocument('2')
        ];
        documents[0].resource.relations['relation1'] = ['2'];
        documents[0].resource.relations['relation2'] = ['missing1'];
        documents[0].resource.relations['relation3'] = ['missing2'];
        const mockIndexFacade = getMockIndexFacade();
        const mockDocumentCache = jasmine.createSpyObj('mockDocumentCache', ['get']);
        mockDocumentCache.get.and.callFake(resourceId => {
            return documents.find(document => document.resource.id === resourceId);
        });
        yield warnings_updater_1.WarningsUpdater.updateRelationTargetWarning(documents[0], mockIndexFacade, mockDocumentCache);
        expect((_g = documents[0].warnings) === null || _g === void 0 ? void 0 : _g.missingRelationTargets).toEqual({
            relationNames: ['relation2', 'relation3'],
            targetIds: ['missing1', 'missing2']
        });
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'missingRelationTargets:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'missingRelationTargetIds:contain');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        done();
    }));
    it('remove relation target warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const documents = [
            createDocument('1'),
            createDocument('2')
        ];
        documents[0].warnings = warnings_1.Warnings.createDefault();
        documents[0].warnings.missingRelationTargets = {
            relationNames: ['relation'],
            targetIds: ['missing']
        };
        documents[0].resource.relations['relation'] = ['2'];
        const mockIndexFacade = jasmine.createSpyObj('mockIndexFacade', ['putToSingleIndex']);
        const mockDocumentCache = jasmine.createSpyObj('mockDocumentCache', ['get']);
        mockDocumentCache.get.and.callFake(resourceId => {
            return documents.find(document => document.resource.id === resourceId);
        });
        yield warnings_updater_1.WarningsUpdater.updateRelationTargetWarning(documents[0], mockIndexFacade, mockDocumentCache);
        expect(documents[0].warnings).toBeUndefined();
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'missingRelationTargets:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'missingRelationTargetIds:contain');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        done();
    }));
    it('remove relation target warnings in linking resources', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const documents = [
            createDocument('1'),
            createDocument('2')
        ];
        documents[1].warnings = warnings_1.Warnings.createDefault();
        documents[1].warnings.missingRelationTargets = {
            relationNames: ['relation'],
            targetIds: ['1']
        };
        documents[1].resource.relations['relation'] = ['1'];
        const mockIndexFacade = getMockIndexFacade();
        const mockDocumentCache = jasmine.createSpyObj('mockDocumentCache', ['get']);
        mockDocumentCache.get.and.callFake(resourceId => {
            return documents.find(document => document.resource.id === resourceId);
        });
        const mockDatastore = jasmine.createSpyObj('mockDatastore', ['find']);
        mockDatastore.find.and.returnValue(Promise.resolve({ documents: [documents[1]] }));
        yield warnings_updater_1.WarningsUpdater.updateRelationTargetWarning(documents[0], mockIndexFacade, mockDocumentCache, mockDatastore, true);
        expect(documents[1].warnings).toBeUndefined();
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'missingRelationTargets:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'missingRelationTargetIds:contain');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'warnings:exist');
        done();
    }));
    it('set invalid parent warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _h;
        const documents = [
            createDocument('1'),
            createDocument('2')
        ];
        documents[0].resource.relations['isRecordedIn'] = ['2'];
        documents[1].resource.category = 'ParentCategory';
        const parentCategoryDefinition = {
            name: 'ParentCategory',
            groups: []
        };
        const categoryDefinition = {
            name: 'Category',
            mustLieWithin: true,
            groups: []
        };
        const mockIndexFacade = getMockIndexFacade();
        const mockDocumentCache = jasmine.createSpyObj('mockDocumentCache', ['get']);
        mockDocumentCache.get.and.callFake(resourceId => {
            return documents.find(document => document.resource.id === resourceId);
        });
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition, parentCategoryDefinition);
        yield warnings_updater_1.WarningsUpdater.updateMissingOrInvalidParentWarning(documents[0], mockProjectConfiguration, mockIndexFacade, mockDocumentCache);
        expect((_h = documents[0].warnings) === null || _h === void 0 ? void 0 : _h.missingOrInvalidParent).toBe(true);
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'missingOrInvalidParent:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        done();
    }));
    it('set invalid parent warnings for invalid ancestors', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _j;
        const documents = [
            createDocument('1'),
            createDocument('2'),
            createDocument('3'),
            createDocument('4'),
            createDocument('5')
        ];
        documents[0].resource.relations['liesWithin'] = ['2'];
        documents[0].resource.relations['isRecordedIn'] = ['5'];
        documents[1].resource.relations['liesWithin'] = ['3'];
        documents[1].resource.relations['isRecordedIn'] = ['5'];
        documents[2].resource.relations['liesWithin'] = ['4'];
        documents[2].resource.relations['isRecordedIn'] = ['5'];
        documents[3].resource.relations['isRecordedIn'] = ['5'];
        documents[3].resource.category = 'UnconfiguredCategory';
        documents[4].resource.category = 'ParentCategory';
        const parentCategoryDefinition = {
            name: 'ParentCategory',
            groups: []
        };
        const categoryDefinition = {
            name: 'Category',
            groups: []
        };
        const mockIndexFacade = getMockIndexFacade();
        const mockDocumentCache = jasmine.createSpyObj('mockDocumentCache', ['get']);
        mockDocumentCache.get.and.callFake(resourceId => {
            return documents.find(document => document.resource.id === resourceId);
        });
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition, parentCategoryDefinition);
        yield warnings_updater_1.WarningsUpdater.updateMissingOrInvalidParentWarning(documents[0], mockProjectConfiguration, mockIndexFacade, mockDocumentCache);
        expect((_j = documents[0].warnings) === null || _j === void 0 ? void 0 : _j.missingOrInvalidParent).toBe(true);
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'missingOrInvalidParent:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        done();
    }));
    it('set invalid parent warnings for circular relations', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _k;
        const documents = [
            createDocument('1'),
            createDocument('2'),
            createDocument('3'),
            createDocument('4')
        ];
        documents[0].resource.relations['liesWithin'] = ['2'];
        documents[0].resource.relations['isRecordedIn'] = ['4'];
        documents[1].resource.relations['liesWithin'] = ['3'];
        documents[1].resource.relations['isRecordedIn'] = ['4'];
        documents[2].resource.relations['liesWithin'] = ['1'];
        documents[2].resource.relations['isRecordedIn'] = ['4'];
        documents[3].resource.category = 'ParentCategory';
        const parentCategoryDefinition = {
            name: 'ParentCategory',
            groups: []
        };
        const categoryDefinition = {
            name: 'Category',
            groups: []
        };
        const mockIndexFacade = getMockIndexFacade();
        const mockDocumentCache = jasmine.createSpyObj('mockDocumentCache', ['get']);
        mockDocumentCache.get.and.callFake(resourceId => {
            return documents.find(document => document.resource.id === resourceId);
        });
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition, parentCategoryDefinition);
        yield warnings_updater_1.WarningsUpdater.updateMissingOrInvalidParentWarning(documents[0], mockProjectConfiguration, mockIndexFacade, mockDocumentCache);
        expect((_k = documents[0].warnings) === null || _k === void 0 ? void 0 : _k.missingOrInvalidParent).toBe(true);
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'missingOrInvalidParent:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        done();
    }));
    it('remove invalid parent warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const documents = [
            createDocument('1'),
            createDocument('2')
        ];
        documents[0].resource.relations['isRecordedIn'] = ['2'];
        documents[0].warnings = warnings_1.Warnings.createDefault();
        documents[0].warnings.missingOrInvalidParent = true;
        documents[1].resource.category = 'ParentCategory';
        const parentCategoryDefinition = {
            name: 'ParentCategory',
            groups: []
        };
        const categoryDefinition = {
            name: 'Category',
            groups: []
        };
        const mockIndexFacade = getMockIndexFacade();
        const mockDocumentCache = jasmine.createSpyObj('mockDocumentCache', ['get']);
        mockDocumentCache.get.and.callFake(resourceId => {
            return documents.find(document => document.resource.id === resourceId);
        });
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition, parentCategoryDefinition);
        yield warnings_updater_1.WarningsUpdater.updateMissingOrInvalidParentWarning(documents[0], mockProjectConfiguration, mockIndexFacade, mockDocumentCache);
        expect(documents[0].warnings).toBeUndefined();
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'missingOrInvalidParent:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[0], 'warnings:exist');
        done();
    }));
    it('remove invalid parent warnings for descendants', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const documents = [
            createDocument('1'),
            createDocument('2'),
            createDocument('3'),
            createDocument('4'),
            createDocument('5'),
            createDocument('6')
        ];
        documents[0].resource.category = 'ParentCategory';
        documents[1].resource.relations['isRecordedIn'] = ['1'];
        documents[1].warnings = warnings_1.Warnings.createDefault();
        documents[1].warnings.missingOrInvalidParent = true;
        documents[2].resource.relations['liesWithin'] = ['2'];
        documents[2].resource.relations['isRecordedIn'] = ['1'];
        documents[2].warnings = warnings_1.Warnings.createDefault();
        documents[2].warnings.missingOrInvalidParent = true;
        documents[3].resource.relations['liesWithin'] = ['2'];
        documents[3].resource.relations['isRecordedIn'] = ['1'];
        documents[3].warnings = warnings_1.Warnings.createDefault();
        documents[3].warnings.missingOrInvalidParent = true;
        documents[4].resource.relations['liesWithin'] = ['3'];
        documents[4].resource.relations['isRecordedIn'] = ['1'];
        documents[4].warnings = warnings_1.Warnings.createDefault();
        documents[4].warnings.missingOrInvalidParent = true;
        documents[5].resource.relations['liesWithin'] = ['4'];
        documents[5].resource.relations['isRecordedIn'] = ['1'];
        documents[5].warnings = warnings_1.Warnings.createDefault();
        documents[5].warnings.missingOrInvalidParent = true;
        const parentCategoryDefinition = {
            name: 'ParentCategory',
            groups: []
        };
        const categoryDefinition = {
            name: 'Category',
            groups: []
        };
        const mockIndexFacade = getMockIndexFacade();
        const mockDocumentCache = jasmine.createSpyObj('mockDocumentCache', ['get']);
        mockDocumentCache.get.and.callFake(resourceId => {
            return documents.find(document => document.resource.id === resourceId);
        });
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition, parentCategoryDefinition);
        const mockDatastore = jasmine.createSpyObj('mockDatastore', ['find']);
        mockDatastore.find.and.returnValue(Promise.resolve({
            documents: [documents[1], documents[2], documents[3], documents[4], documents[5]]
        }));
        yield warnings_updater_1.WarningsUpdater.updateMissingOrInvalidParentWarning(documents[0], mockProjectConfiguration, mockIndexFacade, mockDocumentCache, mockDatastore, true);
        expect(documents[0].warnings).toBeUndefined();
        expect(documents[1].warnings).toBeUndefined();
        expect(documents[2].warnings).toBeUndefined();
        expect(documents[3].warnings).toBeUndefined();
        expect(documents[4].warnings).toBeUndefined();
        expect(documents[5].warnings).toBeUndefined();
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'missingOrInvalidParent:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'warnings:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[2], 'missingOrInvalidParent:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[2], 'warnings:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[3], 'missingOrInvalidParent:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[3], 'warnings:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[4], 'missingOrInvalidParent:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[4], 'warnings:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[5], 'missingOrInvalidParent:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[5], 'warnings:exist');
        done();
    }));
    it('set outlier warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _l, _m, _o, _p;
        const categoryDefinition = {
            name: 'Category',
            identifierPrefix: 'C',
            groups: [
                {
                    fields: [
                        {
                            name: 'editor',
                            inputType: field_1.Field.InputType.CHECKBOXES,
                            valuelistFromProjectField: 'staff'
                        },
                        {
                            name: 'dropdown',
                            inputType: field_1.Field.InputType.DROPDOWN,
                            valuelist: {
                                values: { 'valueDropdown': {} }
                            }
                        },
                        {
                            name: 'checkboxes',
                            inputType: field_1.Field.InputType.CHECKBOXES,
                            valuelist: {
                                values: { 'valueCheckboxes': {} }
                            }
                        },
                        {
                            name: 'dimension',
                            inputType: field_1.Field.InputType.DIMENSION,
                            valuelist: {
                                values: { 'valueDimension': {} }
                            }
                        },
                        {
                            name: 'composite',
                            inputType: field_1.Field.InputType.COMPOSITE,
                            subfields: [
                                {
                                    name: 'dropdown',
                                    inputType: field_1.Field.InputType.DROPDOWN,
                                    valuelist: {
                                        values: { 'valueSubfieldDropdown': {} }
                                    }
                                },
                                {
                                    name: 'checkboxes',
                                    inputType: field_1.Field.InputType.CHECKBOXES,
                                    valuelist: {
                                        values: { 'valueSubfieldCheckboxes': {} }
                                    }
                                },
                                {
                                    name: 'url',
                                    inputType: field_1.Field.InputType.URL
                                }
                            ]
                        }
                    ]
                }
            ]
        };
        const documents = [
            createDocument('project', 'Project'),
            createDocument('1')
        ];
        documents[0].resource.staff = ['Person'];
        documents[1].resource.editor = ['outlierValue1'];
        documents[1].resource.dropdown = 'outlierValue2';
        documents[1].resource.checkboxes = ['outlierValue3'];
        documents[1].resource.dimension = [{ measurementPosition: 'outlierValue4', inputValue: 1, inputUnit: 'cm' }];
        documents[1].resource.composite = [
            { dropdown: 'outlierValue5', checkboxes: ['outlierValue6'], url: 'http://www.example.de' }
        ];
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition);
        const mockIndexFacade = getMockIndexFacade();
        const mockDocumentCache = jasmine.createSpyObj('mockDocumentCache', ['get']);
        mockDocumentCache.get.and.callFake(resourceId => {
            return documents.find(document => document.resource.id === resourceId);
        });
        yield warnings_updater_1.WarningsUpdater.updateOutlierWarning(documents[1], mockProjectConfiguration, categoryDefinition, mockIndexFacade, mockDocumentCache);
        expect((_m = (_l = documents[1].warnings) === null || _l === void 0 ? void 0 : _l.outliers) === null || _m === void 0 ? void 0 : _m.fields)
            .toEqual({
            editor: ['outlierValue1'],
            dropdown: ['outlierValue2'],
            checkboxes: ['outlierValue3'],
            dimension: ['outlierValue4'],
            composite: { dropdown: ['outlierValue5'], checkboxes: ['outlierValue6'] }
        });
        expect((_p = (_o = documents[1].warnings) === null || _o === void 0 ? void 0 : _o.outliers) === null || _p === void 0 ? void 0 : _p.values)
            .toEqual(['outlierValue1', 'outlierValue2', 'outlierValue3', 'outlierValue4', 'outlierValue5',
            'outlierValue6']);
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'outliers:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'outlierValues:contain');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'warnings:exist');
        done();
    }));
    it('remove outlier warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const categoryDefinition = {
            name: 'Category',
            identifierPrefix: 'C',
            groups: [
                {
                    fields: [
                        {
                            name: 'editor',
                            inputType: field_1.Field.InputType.CHECKBOXES,
                            valuelistFromProjectField: 'staff'
                        },
                        {
                            name: 'dropdown',
                            inputType: field_1.Field.InputType.DROPDOWN,
                            valuelist: {
                                values: { 'valueDropdown': {} }
                            }
                        },
                        {
                            name: 'checkboxes',
                            inputType: field_1.Field.InputType.CHECKBOXES,
                            valuelist: {
                                values: { 'valueCheckboxes': {} }
                            }
                        },
                        {
                            name: 'dimension',
                            inputType: field_1.Field.InputType.DIMENSION,
                            valuelist: {
                                values: { 'valueDimension': {} }
                            }
                        },
                        {
                            name: 'composite',
                            inputType: field_1.Field.InputType.COMPOSITE,
                            subfields: [
                                {
                                    name: 'dropdown',
                                    inputType: field_1.Field.InputType.DROPDOWN,
                                    valuelist: {
                                        values: { 'valueSubfieldDropdown': {} }
                                    }
                                },
                                {
                                    name: 'checkboxes',
                                    inputType: field_1.Field.InputType.CHECKBOXES,
                                    valuelist: {
                                        values: { 'valueSubfieldCheckboxes': {} }
                                    }
                                }
                            ]
                        }
                    ]
                }
            ]
        };
        const documents = [
            createDocument('project', 'Project'),
            createDocument('1')
        ];
        documents[0].resource.staff = ['Person'];
        documents[1].warnings = warnings_1.Warnings.createDefault();
        documents[1].warnings.outliers = {
            fields: {
                editor: ['outlierValue'],
                dropdown: ['outlierValue'],
                checkboxes: ['outlierValue'],
                dimension: ['outlierValue'],
                composite: { dropdown: ['outlierValue'], checkboxes: ['outlierValue'] }
            },
            values: ['outlierValue']
        };
        documents[1].resource.editor = ['Person'];
        documents[1].resource.dropdown = 'valueDropdown';
        documents[1].resource.checkboxes = ['valueCheckboxes'];
        documents[1].resource.dimension = [{ measurementPosition: 'valueDimension', inputValue: 1, inputUnit: 'cm' }];
        documents[1].resource.composite = [
            { dropdown: 'valueSubfieldDropdown', checkboxes: ['valueSubfieldCheckboxes'] }
        ];
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition);
        const mockIndexFacade = getMockIndexFacade();
        const mockDocumentCache = jasmine.createSpyObj('mockDocumentCache', ['get']);
        mockDocumentCache.get.and.callFake(resourceId => {
            return documents.find(document => document.resource.id === resourceId);
        });
        yield warnings_updater_1.WarningsUpdater.updateOutlierWarning(documents[1], mockProjectConfiguration, categoryDefinition, mockIndexFacade, mockDocumentCache);
        expect(documents[1].warnings).toBeUndefined();
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'outliers:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'outlierValues:contain');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'warnings:exist');
        done();
    }));
    it('remove outlier warnings for valuelists from project field after updating project document', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _q, _r, _s, _t, _u, _v, _w, _x;
        const categoryDefinition = {
            name: 'Category',
            identifierPrefix: 'C',
            groups: [
                {
                    fields: [
                        {
                            name: 'editor',
                            inputType: field_1.Field.InputType.CHECKBOXES,
                            valuelistFromProjectField: 'staff'
                        },
                        {
                            name: 'checkboxes',
                            inputType: field_1.Field.InputType.CHECKBOXES,
                            valuelist: {
                                values: { 'valueCheckboxes': {} }
                            }
                        }
                    ]
                }
            ]
        };
        const documents = [
            createDocument('project', 'Project'),
            createDocument('1'),
            createDocument('2'),
            createDocument('3')
        ];
        documents[0].resource.staff = ['Person'];
        documents[1].warnings = warnings_1.Warnings.createDefault();
        documents[1].warnings.outliers = { fields: { editor: ['outlierValue'] }, values: ['outlierValue'] };
        documents[1].resource.editor = ['Person'];
        documents[2].warnings = warnings_1.Warnings.createDefault();
        documents[2].warnings.outliers = {
            fields: { editor: ['outlierValue'], checkboxes: ['outlierValue'] },
            values: ['outlierValue']
        };
        documents[2].resource.editor = ['Person'];
        documents[2].resource.checkboxes = ['outlierValue'];
        documents[3].warnings = warnings_1.Warnings.createDefault();
        documents[3].warnings.outliers = { fields: { checkboxes: ['outlierValue'] }, values: ['outlierValue'] };
        documents[3].resource.checkboxes = ['outlierValue'];
        const mockProjectConfiguration = getMockProjectConfiguration(categoryDefinition);
        const mockIndexFacade = getMockIndexFacade();
        const mockDocumentCache = jasmine.createSpyObj('mockDocumentCache', ['get']);
        mockDocumentCache.get.and.callFake(resourceId => {
            return documents.find(document => document.resource.id === resourceId);
        });
        const mockDatastore = jasmine.createSpyObj('mockDatastore', ['find']);
        mockDatastore.find.and.returnValue(Promise.resolve({ documents: [documents[1], documents[2], documents[3]] }));
        yield warnings_updater_1.WarningsUpdater.updateOutlierWarning(documents[0], mockProjectConfiguration, categoryDefinition, mockIndexFacade, mockDocumentCache, mockDatastore, true);
        expect(documents[1].warnings).toBeUndefined();
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'outliers:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'outlierValues:contain');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[1], 'warnings:exist');
        expect((_r = (_q = documents[2].warnings) === null || _q === void 0 ? void 0 : _q.outliers) === null || _r === void 0 ? void 0 : _r.fields).toEqual({ checkboxes: ['outlierValue'] });
        expect((_t = (_s = documents[2].warnings) === null || _s === void 0 ? void 0 : _s.outliers) === null || _t === void 0 ? void 0 : _t.values).toEqual(['outlierValue']);
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[2], 'outliers:exist');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[2], 'outlierValues:contain');
        expect(mockIndexFacade.putToSingleIndex).toHaveBeenCalledWith(documents[2], 'warnings:exist');
        expect((_v = (_u = documents[3].warnings) === null || _u === void 0 ? void 0 : _u.outliers) === null || _v === void 0 ? void 0 : _v.fields).toEqual({ checkboxes: ['outlierValue'] });
        expect((_x = (_w = documents[3].warnings) === null || _w === void 0 ? void 0 : _w.outliers) === null || _x === void 0 ? void 0 : _x.values).toEqual(['outlierValue']);
        expect(mockIndexFacade.putToSingleIndex).not.toHaveBeenCalledWith(documents[3], 'outliers:exist');
        expect(mockIndexFacade.putToSingleIndex).not.toHaveBeenCalledWith(documents[3], 'outlierValues:contain');
        expect(mockIndexFacade.putToSingleIndex).not.toHaveBeenCalledWith(documents[3], 'warnings:exist');
        done();
    }));
});
//# sourceMappingURL=warnings-updater.spec.js.map