"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tsfun_1 = require("tsfun");
const fulltext_index_1 = require("../../src/index/fulltext-index");
const index_item_1 = require("../../src/index/index-item");
const model_1 = require("../../src/model");
const test_helpers_1 = require("../test-helpers");
/**
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
describe('FulltextIndex', () => {
    let fulltextIndex;
    const fieldsToIndex = [
        { name: 'identifier', inputType: model_1.Field.InputType.IDENTIFIER },
        { name: 'shortDescription', inputType: model_1.Field.InputType.SIMPLE_INPUT }
    ];
    function doc(id, identifier, category, shortDescription = 'short') {
        const newDoc = test_helpers_1.doc(shortDescription, identifier, category, id);
        newDoc.created = { date: '2017-12-31' };
        newDoc.modified = [{ date: '2018-01-01' }];
        return newDoc;
    }
    beforeEach(() => {
        fulltextIndex = {};
    });
    it('match one with different search terms', () => {
        const d = doc('1', 'identifier1', 'category');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'identifier1', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'ide', ['category'])).toEqual(['1']);
    });
    it('match two with the same search term', () => {
        const d1 = doc('1', 'identifier1', 'category');
        const d2 = doc('2', 'identifier2', 'category');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d1, fieldsToIndex);
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d2, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'identifier', ['category'])).toEqual(['1', '2']);
    });
    it('match in all categories', () => {
        const d = doc('1', 'identifier1', 'category');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'identifier', undefined)).toEqual(['1']);
    });
    it('index short description', () => {
        const d = doc('1', 'identifier1', 'category', 'short');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'short', undefined)).toEqual(['1']);
    });
    it('match in multiple selected categories', () => {
        const d1 = doc('1', 'identifier1', 'category1');
        const d2 = doc('2', 'identifier2', 'category2');
        const d3 = doc('3', 'identifier3', 'category3');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d1, fieldsToIndex);
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d2, fieldsToIndex);
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d3, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'identifier', ['category1', 'category2'])).toEqual(['1', '2']);
    });
    it('do not match search term', () => {
        const d = doc('1', 'iden', 'category');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'identifier', ['category'])).toEqual([]);
    });
    it('do not match search in category', () => {
        const d = doc('1', 'iden', 'category1');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'identifier', ['category2'])).toEqual([]);
    });
    it('match one with two search terms', () => {
        const d = doc('1', 'identifier1', 'category', 'a short description');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'short description', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'a description', ['category'])).toEqual(['1']);
    });
    it('ignore additional spaces and hyphens', () => {
        const d = doc('1', 'identifier1', 'category', 'a short description');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, ' a    short  description  ', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, '-a----short--description--', ['category'])).toEqual(['1']);
    });
    it('no categories present', () => {
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'identifier', ['category'])).toEqual([]);
    });
    it('remove', () => {
        const d = doc('1', 'identifier1', 'category');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        fulltext_index_1.FulltextIndex.remove(fulltextIndex, d);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'identifier', ['category'])).toEqual([]);
    });
    it('search *', () => {
        const d = doc('1', 'identifier1', 'category');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, '*', ['category'])).toEqual(['1']);
    });
    it('index other field', () => {
        const d = doc('1', 'identifier1', 'category');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'short', ['category'])).toEqual(['1']);
    });
    it('tokenize fields', () => {
        const d1 = doc('1', 'hello token', 'category');
        const d2 = doc('2', 'another-one', 'category');
        const d3 = doc('3', 'last_one', 'category');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d1, fieldsToIndex);
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d2, fieldsToIndex);
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d3, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'hello', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'token', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'another', ['category'])).toEqual(['2']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'one', ['category'])).toEqual(['2', '3']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'last', ['category'])).toEqual(['3']);
    });
    it('find case insensitive', () => {
        const d1 = doc('1', 'Hello', 'category');
        const d2 = doc('2', 'something', 'category');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d1, fieldsToIndex);
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d2, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'hello', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'Something', ['category'])).toEqual(['2']);
    });
    it('put overwrite', () => {
        const d = doc('1', 'identifier1', 'category');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        d['resource']['identifier'] = 'identifier2';
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'identifier1', ['category'])).toEqual([]);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'identifier2', ['category'])).toEqual(['1']);
    });
    it('shortDescription empty', () => {
        const d = doc('1', 'identifier1', 'category');
        d['resource']['shortDescription'] = '';
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'short', ['category'])).toEqual([]);
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        d['resource']['shortDescription'] = undefined;
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'short', ['category'])).toEqual([]);
        delete d['resource']['shortDescription'];
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, fieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'short', ['category'])).toEqual([]);
    });
    it('index all language entries of shortDescription', () => {
        const clonedFieldsToIndex = tsfun_1.clone(fieldsToIndex);
        clonedFieldsToIndex[1].inputType = model_1.Field.InputType.INPUT;
        const d = doc('1', 'identifier1', 'category');
        d['resource']['shortDescription'] = { de: 'Deutsch', en: 'English', it: 'Italiano' };
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d, clonedFieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'Deutsch', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'English', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'Italiano', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'other', ['category'])).toEqual([]);
    });
    it('index all value labels of valuelist based shortDescription', () => {
        const clonedFieldsToIndex = tsfun_1.clone(fieldsToIndex);
        clonedFieldsToIndex[1].inputType = model_1.Field.InputType.DROPDOWN;
        clonedFieldsToIndex[1].valuelist = {
            values: {
                testValue1: {
                    label: { de: 'Deutsch', en: 'English', it: 'Italiano' }
                },
                testValue2: {} // Use value ID as fallback if no value labels are provided
            }
        };
        const d1 = doc('1', 'identifier1', 'category');
        d1['resource']['shortDescription'] = 'testValue1';
        const d2 = doc('2', 'identifier1', 'category');
        d2['resource']['shortDescription'] = 'testValue2';
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d1, clonedFieldsToIndex);
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d2, clonedFieldsToIndex);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'Deutsch', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'English', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'Italiano', ['category'])).toEqual(['1']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'testValue1', ['category'])).toEqual([]);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'testValue2', ['category'])).toEqual(['2']);
        expect(fulltext_index_1.FulltextIndex.get(fulltextIndex, 'other', ['category'])).toEqual([]);
    });
    it('do a placeholder search', () => {
        const d1 = doc('1', 'Hello-A-0033', 'category');
        const d2 = doc('2', 'Hello-A-0021', 'category');
        const d3 = doc('3', 'Hello-A-0059', 'category');
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d1, fieldsToIndex);
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d2, fieldsToIndex);
        fulltext_index_1.FulltextIndex.put(fulltextIndex, d3, fieldsToIndex);
        const results = fulltext_index_1.FulltextIndex.get(fulltextIndex, 'Hello-A-00[23]', ['category']);
        expect(results.length).toBe(2);
        expect(results).toContain('1');
        expect(results).toContain('2');
    });
    it('index field specified in search configuration', () => {
        const customField = { name: 'customField', inputType: model_1.Field.InputType.SIMPLE_INPUT };
        const document = doc('1', 'identifier1', 'category');
        document.resource.customField = 'testValue';
        const ie = index_item_1.IndexItem.from(document);
        fulltext_index_1.FulltextIndex.put(fulltextIndex, document, fieldsToIndex.concat([customField]));
        const results = fulltext_index_1.FulltextIndex.get(fulltextIndex, 'testValue', ['category']);
        expect(results.length).toBe(1);
        expect(results[0]).toEqual('1');
    });
});
//# sourceMappingURL=fulltext-index.spec.js.map