"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const project_configuration_1 = require("../../../src/services/project-configuration");
const connected_docs_1 = require("../../../src/services/utilities/connected-docs");
const model_1 = require("../../../src/model");
const tools_1 = require("../../../src/tools");
/**
 * @author Daniel de Oliveira
 * @author Thomas Kleinke
 */
describe('ConnectedDocs', () => {
    const projectConfiguration = new project_configuration_1.ProjectConfiguration({
        forms: [],
        categories: {},
        relations: [
            {
                name: 'BelongsTo',
                inverse: 'Contains',
                domain: [],
                range: [],
                editable: false,
                inputType: 'relation'
            },
            {
                name: 'Contains',
                inverse: 'BelongsTo',
                domain: [],
                range: [],
                editable: false,
                inputType: 'relation'
            },
            {
                name: 'isRecordedIn',
                domain: [],
                range: [],
                editable: false,
                inputType: 'relation'
            },
            {
                name: 'isPresentIn',
                domain: [],
                range: [],
                editable: false,
                inputType: 'relation'
            }
        ],
        commonFields: {},
        valuelists: {},
        projectLanguages: []
    });
    let mockDatastore;
    let inverseRelationsMap;
    let relationNames;
    let doc;
    let relatedDoc;
    let anotherRelatedDoc;
    beforeEach(() => {
        spyOn(console, 'warn');
        mockDatastore = jasmine.createSpyObj('mockDatastore', ['get', 'update', 'convert', 'findIds']);
        inverseRelationsMap = model_1.Relation.makeInverseRelationsMap(projectConfiguration.getRelations());
        relationNames = projectConfiguration.getRelations().map(tools_1.Named.toName);
        mockDatastore.update.and.returnValue(Promise.resolve(doc));
        mockDatastore.get.and.callFake((id) => __awaiter(void 0, void 0, void 0, function* () {
            return id === relatedDoc['resource']['id'] ? relatedDoc : anotherRelatedDoc;
        }));
        mockDatastore.findIds.and.returnValue({ ids: [] });
        doc = { 'resource': {
                'id': '1', 'identifier': 'ob1',
                'category': 'object',
                'relations': {}
            } };
        relatedDoc = { 'resource': {
                'id': '2', 'identifier': 'ob2',
                'category': 'object',
                'relations': {}
            } };
        anotherRelatedDoc = { 'resource': {
                'id': '3', 'identifier': 'ob3',
                'category': 'object',
                'relations': {}
            } };
    });
    it('should save the related document', (done) => __awaiter(void 0, void 0, void 0, function* () {
        doc.resource.relations['BelongsTo'] = ['2'];
        mockDatastore.update.and.returnValue(Promise.resolve(doc));
        yield connected_docs_1.ConnectedDocs.updateForUpdate(mockDatastore, relationNames, inverseRelationsMap, doc, [doc]);
        expect(mockDatastore.update).toHaveBeenCalledWith(relatedDoc, undefined);
        expect(relatedDoc.resource.relations['Contains'][0]).toBe('1');
        done();
    }));
    it('should save an object with a one way relation', (done) => __awaiter(void 0, void 0, void 0, function* () {
        doc.resource.relations['isRecordedIn'] = ['2'];
        mockDatastore.update.and.returnValue(Promise.resolve(doc));
        yield connected_docs_1.ConnectedDocs.updateForUpdate(mockDatastore, relationNames, inverseRelationsMap, doc, [doc]);
        expect(mockDatastore.update).not.toHaveBeenCalledWith(relatedDoc, undefined);
        done();
    }));
    it('delete a relation which was present in old version', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const oldVersion = { 'resource': {
                'id': '1', 'identifier': 'ob1',
                'category': 'object',
                'relations': { 'BelongsTo': ['2'] }
            } };
        relatedDoc.resource.relations['Contains'] = ['1'];
        mockDatastore.update.and.returnValue(Promise.resolve(doc));
        yield connected_docs_1.ConnectedDocs.updateForUpdate(mockDatastore, relationNames, inverseRelationsMap, doc, [oldVersion]);
        expect(mockDatastore.update).toHaveBeenCalledWith(relatedDoc, undefined);
        expect(doc.resource.relations['BelongsTo']).toBe(undefined);
        expect(relatedDoc.resource.relations['Contains']).toBe(undefined);
        done();
    }));
    it('should add two relations of the same category', (done) => __awaiter(void 0, void 0, void 0, function* () {
        doc.resource.relations['BelongsTo'] = ['2', '3'];
        mockDatastore.update.and.returnValue(Promise.resolve(doc));
        yield connected_docs_1.ConnectedDocs.updateForUpdate(mockDatastore, relationNames, inverseRelationsMap, doc, [doc]);
        expect(mockDatastore.update).toHaveBeenCalledWith(anotherRelatedDoc, undefined);
        expect(anotherRelatedDoc['resource']['relations']['Contains'][0]).toBe('1');
        done();
    }));
    it('remove: should remove a document', (done) => __awaiter(void 0, void 0, void 0, function* () {
        doc.resource.relations['BelongsTo'] = ['2'];
        relatedDoc.resource.relations['Contains'] = ['1'];
        yield connected_docs_1.ConnectedDocs.updateForRemove(mockDatastore, relationNames, inverseRelationsMap, doc);
        expect(mockDatastore.update).toHaveBeenCalledWith(relatedDoc, undefined);
        expect(relatedDoc.resource.relations['Contains']).toBe(undefined);
        done();
    }));
    it('remove: where a connected document does not exist anymore', (done) => __awaiter(void 0, void 0, void 0, function* () {
        doc.resource.relations['BelongsTo'] = ['nonexistent'];
        const get = () => Promise.reject('not exists');
        yield connected_docs_1.ConnectedDocs.updateForRemove(mockDatastore, relationNames, inverseRelationsMap, doc);
        expect(mockDatastore.update).not.toHaveBeenCalled();
        done();
    }));
    it('remove: should remove a document with a one way relation', (done) => __awaiter(void 0, void 0, void 0, function* () {
        doc.resource.relations['isRecordedIn'] = ['2'];
        yield connected_docs_1.ConnectedDocs.updateForRemove(mockDatastore, relationNames, inverseRelationsMap, doc);
        expect(mockDatastore.update).not.toHaveBeenCalled();
        done();
    }));
    it('remove: delete isPresentIn relation pointing to deleted document', (done) => __awaiter(void 0, void 0, void 0, function* () {
        relatedDoc.resource.relations['isPresentIn'] = ['1'];
        mockDatastore.findIds.and.returnValue({ ids: ['2'] });
        yield connected_docs_1.ConnectedDocs.updateForRemove(mockDatastore, relationNames, inverseRelationsMap, doc);
        expect(mockDatastore.update).toHaveBeenCalledWith(relatedDoc, undefined);
        expect(relatedDoc.resource.relations['isPresentIn']).toBe(undefined);
        done();
    }));
});
//# sourceMappingURL=connected-docs.spec.js.map