"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const model_1 = require("../../../src/model");
const test_helpers_1 = require("../../test-helpers");
const subsystem_helper_1 = require("../subsystem-helper");
describe('subsystem/datastore', () => {
    let app;
    let helpers;
    let image0;
    let trench0;
    beforeEach((done) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            app = yield subsystem_helper_1.createCoreApp();
            helpers = yield subsystem_helper_1.createHelpers(app);
        }
        catch (err) {
            console.error(err);
        }
        done();
    }));
    it('create document', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield app.datastore.create(test_helpers_1.doc1('abc', 'Abc', 'Trench'));
        yield helpers.expectDocuments('abc');
        done();
    }));
    it('do not throw and return everything with all categories', (done) => __awaiter(void 0, void 0, void 0, function* () {
        image0 = test_helpers_1.doc('Image', 'Image', 'Image', 'image0');
        trench0 = test_helpers_1.doc('Trench', 'Trench', 'Trench', 'trench0');
        yield app.datastore.create(image0);
        yield app.datastore.create(trench0);
        try {
            const result = yield app.datastore.find({ categories: ['Trench', 'Image'] });
            expect(result.documents.length).toBe(2);
        }
        catch (err) {
            fail(err);
        }
        done();
    }));
    it('return everything when called without categories', (done) => __awaiter(void 0, void 0, void 0, function* () {
        image0 = test_helpers_1.doc('Image', 'Image', 'Image', 'image0');
        trench0 = test_helpers_1.doc('Trench', 'Trench', 'Trench', 'trench0');
        yield app.datastore.create(image0);
        yield app.datastore.create(trench0);
        try {
            const result = yield app.datastore.find({});
            expect(result.documents.length).toBe(2);
        }
        catch (err) {
            fail(err);
        }
        done();
    }));
    it('sort documents in different sort modes', (done) => __awaiter(void 0, void 0, void 0, function* () {
        const doc1 = test_helpers_1.doc('sd1', 'A-B-100', 'Find', '1');
        const doc2 = test_helpers_1.doc('sd2', 'B-100', 'Find', '2');
        const doc3 = test_helpers_1.doc('sd3', 'C-100', 'Find', '3');
        yield app.datastore.create(doc1);
        yield app.datastore.create(doc2);
        yield app.datastore.create(doc3);
        const { documents: documents1, totalCount: totalCount1 } = yield app.datastore.find({ q: 'B-100', sort: { mode: 'default' } }, { includeResourcesWithoutValidParent: true });
        expect(documents1.length).toBe(2);
        expect(totalCount1).toBe(2);
        expect(documents1[0].resource.id).toBe('1');
        expect(documents1[1].resource.id).toBe('2');
        const { documents: documents2, totalCount: totalCount2 } = yield app.datastore.find({ q: 'B-100', sort: { mode: 'exactMatchFirst' } }, { includeResourcesWithoutValidParent: true });
        expect(documents2.length).toBe(2);
        expect(totalCount2).toBe(2);
        expect(documents2[0].resource.id).toBe('2');
        expect(documents2[1].resource.id).toBe('1');
        done();
    }));
    it('use isChildOf constraint', (done) => __awaiter(void 0, void 0, void 0, function* () {
        yield helpers.createDocuments([
            ['id1', 'Trench', ['id2']],
            ['id2', 'Feature', ['id3']],
            ['id3', 'Find', []],
        ]);
        const result = yield app.datastore.find({
            constraints: { 'isChildOf:contain': { value: ['id1'], searchRecursively: true } }
        });
        expect(result.ids).toEqual(['id2', 'id3']);
        done();
    }));
    it('do not return documents without valid parent resource in find result', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        let document1 = test_helpers_1.doc('sd1', 'Trench', 'Trench', 'id1');
        let document2 = test_helpers_1.doc('sd2', '1', 'Find', 'id2');
        let document3 = test_helpers_1.doc('sd3', '2', 'Find', 'id3');
        document2.resource.relations.isRecordedIn = ['id1'];
        document1 = yield app.datastore.create(document1);
        document2 = yield app.datastore.create(document2);
        document3 = yield app.datastore.create(document3);
        expect((_a = document3.warnings) === null || _a === void 0 ? void 0 : _a.missingOrInvalidParent).toBe(true);
        const result = yield app.datastore.find({ categories: ['Find'] });
        expect(result.documents.length).toBe(1);
        expect(result.documents[0].resource.id).toBe('id2');
        done();
    }));
    it('return documents without valid parent resource in find result if datastore option is enabled', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _b;
        let document1 = test_helpers_1.doc('sd1', 'Trench', 'Trench', 'id1');
        let document2 = test_helpers_1.doc('sd2', '1', 'Find', 'id2');
        let document3 = test_helpers_1.doc('sd3', '2', 'Find', 'id3');
        document2.resource.relations.isRecordedIn = ['id1'];
        document1 = yield app.datastore.create(document1);
        document2 = yield app.datastore.create(document2);
        document3 = yield app.datastore.create(document3);
        expect((_b = document3.warnings) === null || _b === void 0 ? void 0 : _b.missingOrInvalidParent).toBe(true);
        const result = yield app.datastore.find({ categories: ['Find'] }, { includeResourcesWithoutValidParent: true });
        expect(result.documents.length).toBe(2);
        expect(result.documents[0].resource.id).toBe('id2');
        expect(result.documents[1].resource.id).toBe('id3');
        done();
    }));
    it('update non-unique identifier warnings', (done) => __awaiter(void 0, void 0, void 0, function* () {
        var _c, _d;
        let document1 = test_helpers_1.doc('sd1', 'Trench', 'Trench', 'id1');
        let document2 = test_helpers_1.doc('sd2', '1', 'Find', 'id2');
        let document3 = test_helpers_1.doc('sd3', '1', 'Find', 'id3');
        document2.resource.relations.isRecordedIn = ['id1'];
        document3.resource.relations.isRecordedIn = ['id1'];
        document1 = yield app.datastore.create(document1);
        document2 = yield app.datastore.create(document2);
        document3 = yield app.datastore.create(document3);
        expect((_c = document2.warnings) === null || _c === void 0 ? void 0 : _c.nonUniqueIdentifier).toBe(true);
        expect((_d = document3.warnings) === null || _d === void 0 ? void 0 : _d.nonUniqueIdentifier).toBe(true);
        const clonedDocument3 = model_1.Document.clone(document3);
        clonedDocument3.resource.identifier = '2';
        yield app.datastore.update(clonedDocument3);
        expect(document1.warnings).toBeUndefined();
        expect(document2.warnings).toBeUndefined();
        done();
    }));
});
//# sourceMappingURL=datastore.spec.js.map